Package: Cascade
Type: Package
Title: Selection, Reverse-Engineering and Prediction in Cascade
        Networks
Version: 2.4
Date: 2026-04-30
Depends: R (>= 3.5.0)
biocViews:
Imports: abind, animation, cluster, igraph, lars, lattice, limma,
        methods, nnls, splines, stats4, tnet, VGAM
Suggests: R.rsp, CascadeData, knitr, testthat (>= 3.0.0)
Authors@R: c(
	person(given = "Frederic", family= "Bertrand", role = c("cre", "aut"), email = "frederic.bertrand@lecnam.net", comment = c(ORCID = "0000-0002-0837-8281")),
	person(given = "Myriam", family= "Maumy-Bertrand", role = c("aut"), email = "myriam.maumy@ehesp.fr", comment = c(ORCID = "0000-0002-4615-1512")),
	person(given = "Laurent", family= "Vallat", role = c("ctb"), email = "vallat@unistra.fr"),
	person(given = "Nicolas", family= "Jung", role = c("ctb"), email = "nicolas.jung@unistra.fr"))
Author: Frederic Bertrand [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-0837-8281>),
  Myriam Maumy-Bertrand [aut] (ORCID:
    <https://orcid.org/0000-0002-4615-1512>),
  Laurent Vallat [ctb],
  Nicolas Jung [ctb]
Maintainer: Frederic Bertrand <frederic.bertrand@lecnam.net>
Description: A modeling tool allowing gene selection, reverse engineering, and prediction in cascade networks. Jung, N., Bertrand, F., Bahram, S., Vallat, L., and Maumy-Bertrand, M. (2014) <doi:10.1093/bioinformatics/btt705>.
License: GPL (>= 2)
Encoding: UTF-8
Collate: Cascade-package.R global.R micro_array.R network.R
        micro_array-network.R micropredict.R datasets.R
Classification/MSC: 62J05, 62J07, 62J99, 92C42
VignetteBuilder: R.rsp
RoxygenNote: 7.3.3
URL: https://fbertran.github.io/Cascade/,
        https://github.com/fbertran/Cascade
BugReports: https://github.com/fbertran/Cascade/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-30 08:59:30 UTC; bertran7
Repository: CRAN
Date/Publication: 2026-04-30 10:00:08 UTC

# Generated by roxygen2: do not edit by hand

export(as.micro_array)
export(network_random)
exportClasses(micro_array)
exportClasses(micropredict)
exportClasses(network)
exportMethods(analyze_network)
exportMethods(compare)
exportMethods(cutoff)
exportMethods(dim)
exportMethods(evolution)
exportMethods(geneNeighborhood)
exportMethods(genePeakSelection)
exportMethods(geneSelection)
exportMethods(gene_expr_simulation)
exportMethods(head)
exportMethods(inference)
exportMethods(plot)
exportMethods(position)
exportMethods(predict)
exportMethods(print)
exportMethods(summary)
exportMethods(unionMicro)
import(grid)
import(igraph)
import(lattice)
import(limma)
import(magic)
import(nnls)
import(splines)
import(stats4)
import(survival)
import(tnet)
importFrom(VGAM,rlaplace)
importFrom(VGAM,rpareto)
importFrom(VGAM,zeta)
importFrom(abind,abind)
importFrom(animation,ani.options)
importFrom(animation,saveHTML)
importFrom(cluster,agnes)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRamp)
importFrom(grDevices,dev.new)
importFrom(grDevices,grey)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(graphics,rect)
importFrom(graphics,text)
importFrom(methods,new)
importFrom(stats,aggregate)
importFrom(stats,cor)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,model.matrix)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,reshape)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(utils,sessionInfo)

<!-- README.md is generated from README.Rmd. Please edit that file -->



# Cascade <img src="man/figures/logo.png" align="right" width="200"/>

# Cascade, Selection, Reverse-Engineering and Prediction in Cascade Networks
## Frédéric Bertrand and Myriam Maumy-Bertrand

<https://doi.org/10.32614/CRAN.package.Cascade>

<!-- badges: start -->
[![DOI](https://img.shields.io/badge/doi-10.32614/CRAN.package.Cascade-blue.svg)](https://doi.org/10.32614/CRAN.package.Cascade)
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-green.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![Project Status: Active – The project has reached a stable, usable state and is being actively developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![R-CMD-check](https://github.com/fbertran/Cascade/workflows/R-CMD-check/badge.svg)](https://github.com/fbertran/Cascade/actions)
[![Codecov test coverage](https://codecov.io/gh/fbertran/Cascade/branch/master/graph/badge.svg)](https://app.codecov.io/gh/fbertran/Cascade?branch=master)
[![CRAN status](https://www.r-pkg.org/badges/version/Cascade)](https://cran.r-project.org/package=Cascade)
[![CRAN RStudio mirror downloads](https://cranlogs.r-pkg.org/badges/Cascade)](https://cran.r-project.org/package=Cascade)
[![GitHub Repo stars](https://img.shields.io/github/stars/fbertran/Cascade?style=social)](https://github.com/fbertran/Cascade)
<!-- badges: end -->

Cascade is a modeling tool allowing gene selection, reverse engineering, and prediction in cascade networks. Jung, N., Bertrand, F., Bahram, S., Vallat, L., and Maumy-Bertrand, M. (2014) <https://doi.org/10.1093/bioinformatics/btt705>.


The package was presented at the [User2014!](https://user2014.r-project.org/) conference. Jung, N., Bertrand, F., Bahram, S., Vallat, L., and Maumy-Bertrand, M. (2014). "Cascade: a R-package to study, predict and simulate the diffusion of a signal through a temporal genenetwork", *book of abstracts*, User2014!, Los Angeles, page 153, <https://user2014.r-project.org/abstracts/posters/181_Jung.pdf>.


![Reverse-engineered network.](man/figures/figure_article_def.png)

![Simulation of an intervention on a gene of the network.](man/figures/figure_network_pred2.png)

This website and these examples were created by F. Bertrand and M. Maumy-Bertrand.

## Installation

You can install the released version of Cascade from [CRAN](https://CRAN.R-project.org) with:


``` r
install.packages("Cascade")
```

You can install the development version of Cascade from [github](https://github.com) with:


``` r
devtools::install_github("fbertran/Cascade")
```

## Examples

### Data management
Import Cascade Data (repeated measurements on several subjects) from the CascadeData package and turn them into a micro array object. The second line makes sure the CascadeData package is installed.

``` r
library(Cascade)
if(!require(CascadeData)){install.packages("CascadeData")}
#> Loading required package: CascadeData
data(micro_US)
micro_US<-as.micro_array(micro_US,time=c(60,90,210,390),subject=6)
```

Get a summay and plots of the data:

``` r
summary(micro_US)
#> Loading required package: cluster
#>    N1_US_T60        N1_US_T90        N1_US_T210       N1_US_T390       N2_US_T60     
#>  Min.   :   1.0   Min.   :   1.0   Min.   :   1.0   Min.   :   1.0   Min.   :   1.0  
#>  1st Qu.:  19.7   1st Qu.:  18.8   1st Qu.:  15.2   1st Qu.:  20.9   1st Qu.:  18.5  
#>  Median :  38.0   Median :  37.2   Median :  34.9   Median :  40.2   Median :  36.9  
#>  Mean   : 107.5   Mean   : 106.9   Mean   : 109.6   Mean   : 105.7   Mean   : 110.6  
#>  3rd Qu.:  80.6   3rd Qu.:  82.1   3rd Qu.:  82.8   3rd Qu.:  84.8   3rd Qu.:  85.3  
#>  Max.   :8587.9   Max.   :8311.7   Max.   :7930.3   Max.   :7841.8   Max.   :7750.3  
#>    N2_US_T90        N2_US_T210       N2_US_T390       N3_US_T60        N3_US_T90     
#>  Min.   :   1.0   Min.   :   1.0   Min.   :   1.0   Min.   :   1.0   Min.   :   1.0  
#>  1st Qu.:  17.1   1st Qu.:  15.8   1st Qu.:  17.7   1st Qu.:  17.3   1st Qu.:  19.5  
#>  Median :  36.7   Median :  36.0   Median :  37.4   Median :  34.4   Median :  38.2  
#>  Mean   : 102.1   Mean   : 106.8   Mean   : 111.3   Mean   : 101.6   Mean   : 107.1  
#>  3rd Qu.:  78.2   3rd Qu.:  83.5   3rd Qu.:  86.4   3rd Qu.:  75.4   3rd Qu.:  82.3  
#>  Max.   :8014.3   Max.   :8028.6   Max.   :7498.4   Max.   :8072.2   Max.   :7889.2  
#>    N3_US_T210       N3_US_T390       N4_US_T60        N4_US_T90        N4_US_T210    
#>  Min.   :   1.0   Min.   :   1.0   Min.   :   1.0   Min.   :   1.0   Min.   :   1.0  
#>  1st Qu.:  16.4   1st Qu.:  20.9   1st Qu.:  20.4   1st Qu.:  19.5   1st Qu.:  20.5  
#>  Median :  34.7   Median :  41.0   Median :  38.9   Median :  38.5   Median :  39.9  
#>  Mean   : 100.3   Mean   : 113.9   Mean   : 113.6   Mean   : 114.8   Mean   : 110.1  
#>  3rd Qu.:  76.3   3rd Qu.:  89.2   3rd Qu.:  84.6   3rd Qu.:  86.1   3rd Qu.:  86.8  
#>  Max.   :8278.2   Max.   :6856.2   Max.   :9502.3   Max.   :9193.4   Max.   :9436.0  
#>    N4_US_T390       N5_US_T60        N5_US_T90        N5_US_T210       N5_US_T390    
#>  Min.   :   1.0   Min.   :   1.0   Min.   :   1.0   Min.   :   1.0   Min.   :   1.0  
#>  1st Qu.:  19.9   1st Qu.:  16.8   1st Qu.:  18.8   1st Qu.:  19.5   1st Qu.:  19.9  
#>  Median :  38.8   Median :  34.5   Median :  36.9   Median :  38.2   Median :  39.0  
#>  Mean   : 111.7   Mean   : 111.3   Mean   : 108.0   Mean   : 107.4   Mean   : 109.8  
#>  3rd Qu.:  85.4   3rd Qu.:  82.0   3rd Qu.:  81.4   3rd Qu.:  82.4   3rd Qu.:  84.9  
#>  Max.   :8771.0   Max.   :8569.3   Max.   :7970.1   Max.   :8371.0   Max.   :7686.5  
#>    N6_US_T60        N6_US_T90        N6_US_T210       N6_US_T390    
#>  Min.   :   1.0   Min.   :   1.0   Min.   :   1.0   Min.   :   1.0  
#>  1st Qu.:  21.1   1st Qu.:  21.5   1st Qu.:  19.9   1st Qu.:  20.2  
#>  Median :  40.9   Median :  40.8   Median :  39.1   Median :  39.4  
#>  Mean   : 110.1   Mean   : 108.5   Mean   : 112.0   Mean   : 109.5  
#>  3rd Qu.:  86.3   3rd Qu.:  85.6   3rd Qu.:  86.3   3rd Qu.:  86.6  
#>  Max.   :8241.0   Max.   :8355.0   Max.   :8207.1   Max.   :9520.0
```

<div class="figure">
<img src="man/figures/README-plotmicroarrayclass-1.png" alt="plot of chunk plotmicroarrayclass" width="100%" />
<p class="caption">plot of chunk plotmicroarrayclass</p>
</div><div class="figure">
<img src="man/figures/README-plotmicroarrayclass-2.png" alt="plot of chunk plotmicroarrayclass" width="100%" />
<p class="caption">plot of chunk plotmicroarrayclass</p>
</div>

### Gene selection
There are several functions to carry out gene selection before the inference. They are detailed in the package vignette and in the website-only E-MTAB-1475 re-analysis PDF.

### Data simulation
Let's simulate some cascade data and then do some reverse engineering.

We first design the F matrix

``` r
T<-4
F<-array(0,c(T-1,T-1,T*(T-1)/2))

for(i in 1:(T*(T-1)/2)){diag(F[,,i])<-1}
F[,,2]<-F[,,2]*0.2
F[2,1,2]<-1
F[3,2,2]<-1
F[,,4]<-F[,,2]*0.3
F[3,1,4]<-1
F[,,5]<-F[,,2]
```

We set the seed to make the results reproducible and draw a scale free random network.

``` r
set.seed(1)
Net<-Cascade::network_random(
  nb=100,
  time_label=rep(1:4,each=25),
  exp=1,
  init=1,
  regul=round(rexp(100,1))+1,
  min_expr=0.1,
  max_expr=2,
  casc.level=0.4
)
Net@F<-F
```

We simulate gene expression according to the network that was previously drawn

``` r
M <- Cascade::gene_expr_simulation(
  network=Net,
  time_label=rep(1:4,each=25),
  subject=5,
  level_peak=200)
#> Loading required package: VGAM
#> Loading required package: stats4
#> Loading required package: splines
```

Get a summay and plots of the simulated data:

``` r
summary(M)
#>  log(S/US) : P1T1   log(S/US) : P1T2    log(S/US) : P1T3    log(S/US) : P1T4  
#>  Min.   :-887.428   Min.   :-2060.695   Min.   :-837.5811   Min.   :-2189.33  
#>  1st Qu.: -53.644   1st Qu.:  -70.993   1st Qu.: -85.2176   1st Qu.: -115.23  
#>  Median :   6.122   Median :    2.206   Median :  -6.6303   Median :  -10.49  
#>  Mean   :   3.928   Mean   :   -3.800   Mean   :   0.5563   Mean   :  -75.60  
#>  3rd Qu.:  74.894   3rd Qu.:   79.999   3rd Qu.:  71.9501   3rd Qu.:   43.85  
#>  Max.   : 747.779   Max.   : 1461.163   Max.   :1279.6850   Max.   :  492.80  
#>  log(S/US) : P2T1   log(S/US) : P2T2   log(S/US) : P2T3   log(S/US) : P2T4  log(S/US) : P3T1  
#>  Min.   :-431.905   Min.   :-845.072   Min.   :-557.840   Min.   :-660.17   Min.   :-868.834  
#>  1st Qu.: -50.282   1st Qu.: -21.668   1st Qu.: -42.958   1st Qu.: -63.34   1st Qu.: -44.656  
#>  Median :  -3.782   Median :   2.059   Median :  -2.448   Median : -10.97   Median :   1.839  
#>  Mean   :   8.264   Mean   :  18.235   Mean   :  25.290   Mean   :  18.10   Mean   :  -2.150  
#>  3rd Qu.:  40.975   3rd Qu.:  40.323   3rd Qu.:  46.573   3rd Qu.:  42.56   3rd Qu.:  55.072  
#>  Max.   :1287.819   Max.   : 699.912   Max.   :1754.081   Max.   :1418.99   Max.   : 597.562  
#>  log(S/US) : P3T2   log(S/US) : P3T3    log(S/US) : P3T4   log(S/US) : P4T1  
#>  Min.   :-953.894   Min.   :-1182.316   Min.   :-1027.24   Min.   :-1012.76  
#>  1st Qu.: -59.964   1st Qu.:  -87.170   1st Qu.:  -61.31   1st Qu.:  -33.79  
#>  Median :  -1.306   Median :   -2.614   Median :   16.27   Median :   11.57  
#>  Mean   :  24.123   Mean   :   10.224   Mean   :   27.52   Mean   :   10.98  
#>  3rd Qu.:  78.430   3rd Qu.:   79.246   3rd Qu.:   60.93   3rd Qu.:   74.09  
#>  Max.   :1808.233   Max.   : 2761.291   Max.   : 1926.45   Max.   :  891.60  
#>  log(S/US) : P4T2   log(S/US) : P4T3   log(S/US) : P4T4   log(S/US) : P5T1  
#>  Min.   :-1569.32   Min.   :-577.799   Min.   :-661.083   Min.   :-555.708  
#>  1st Qu.: -118.83   1st Qu.: -62.623   1st Qu.: -41.705   1st Qu.: -64.469  
#>  Median :  -13.84   Median :  -8.788   Median :  -1.468   Median :   2.697  
#>  Mean   :  -63.39   Mean   : -14.803   Mean   :  10.111   Mean   :   9.403  
#>  3rd Qu.:   40.22   3rd Qu.:  37.779   3rd Qu.:  67.888   3rd Qu.:  57.251  
#>  Max.   :  678.27   Max.   : 430.737   Max.   : 492.723   Max.   : 654.771  
#>  log(S/US) : P5T2    log(S/US) : P5T3  log(S/US) : P5T4  
#>  Min.   :-1467.268   Min.   :-911.18   Min.   :-621.705  
#>  1st Qu.:  -68.769   1st Qu.: -71.66   1st Qu.: -62.466  
#>  Median :   -1.565   Median :   1.54   Median :  -5.789  
#>  Mean   :    8.180   Mean   :  13.03   Mean   :   7.083  
#>  3rd Qu.:   62.256   3rd Qu.:  86.29   3rd Qu.:  62.041  
#>  Max.   :  990.550   Max.   :1386.11   Max.   : 689.323
```

<div class="figure">
<img src="man/figures/README-summarysimuldata-1.png" alt="plot of chunk summarysimuldata" width="100%" />
<p class="caption">plot of chunk summarysimuldata</p>
</div><div class="figure">
<img src="man/figures/README-summarysimuldata-2.png" alt="plot of chunk summarysimuldata" width="100%" />
<p class="caption">plot of chunk summarysimuldata</p>
</div><div class="figure">
<img src="man/figures/README-summarysimuldata-3.png" alt="plot of chunk summarysimuldata" width="100%" />
<p class="caption">plot of chunk summarysimuldata</p>
</div>


``` r
plot(M)
```

<div class="figure">
<img src="man/figures/README-plotsimuldata-1.png" alt="plot of chunk plotsimuldata" width="100%" />
<p class="caption">plot of chunk plotsimuldata</p>
</div><div class="figure">
<img src="man/figures/README-plotsimuldata-2.png" alt="plot of chunk plotsimuldata" width="100%" />
<p class="caption">plot of chunk plotsimuldata</p>
</div><div class="figure">
<img src="man/figures/README-plotsimuldata-3.png" alt="plot of chunk plotsimuldata" width="100%" />
<p class="caption">plot of chunk plotsimuldata</p>
</div><div class="figure">
<img src="man/figures/README-plotsimuldata-4.png" alt="plot of chunk plotsimuldata" width="100%" />
<p class="caption">plot of chunk plotsimuldata</p>
</div><div class="figure">
<img src="man/figures/README-plotsimuldata-5.png" alt="plot of chunk plotsimuldata" width="100%" />
<p class="caption">plot of chunk plotsimuldata</p>
</div><div class="figure">
<img src="man/figures/README-plotsimuldata-6.png" alt="plot of chunk plotsimuldata" width="100%" />
<p class="caption">plot of chunk plotsimuldata</p>
</div><div class="figure">
<img src="man/figures/README-plotsimuldata-7.png" alt="plot of chunk plotsimuldata" width="100%" />
<p class="caption">plot of chunk plotsimuldata</p>
</div>

### Network inference
We infer the new network using subjectwise leave one out cross-validation (all measurement from the same subject are removed from the dataset)

``` r
Net_inf_C <- Cascade::inference(M, cv.subjects=TRUE)
#> Loading required package: nnls
#> We are at step :  1
#> The convergence of the network is (L1 norm) : 0.0072
#> We are at step :  2
#> The convergence of the network is (L1 norm) : 0.00139
#> We are at step :  3
#> The convergence of the network is (L1 norm) : 0.00106
#> We are at step :  4
#> The convergence of the network is (L1 norm) : 0.00095
```

<div class="figure">
<img src="man/figures/README-netinf-1.png" alt="plot of chunk netinf" width="100%" />
<p class="caption">plot of chunk netinf</p>
</div><div class="figure">
<img src="man/figures/README-netinf-2.png" alt="plot of chunk netinf" width="100%" />
<p class="caption">plot of chunk netinf</p>
</div>

Heatmap of the coefficients of the Omega matrix of the network

``` r
stats::heatmap(Net_inf_C@network, Rowv=NA, Colv=NA, scale="none", revC=TRUE)
```

<div class="figure">
<img src="man/figures/README-heatresults-1.png" alt="plot of chunk heatresults" width="100%" />
<p class="caption">plot of chunk heatresults</p>
</div>

###Post inferrence network analysis
We switch to data that were derived from the inferrence of a real biological network and try to detect the optimal cutoff value: the best cutoff value for a network to fit a scale free network.


```r
data("network")
set.seed(1)
cutoff(network)
#> [1] "This calculation may be long"
#> [1] "1/10"
#> [1] "2/10"
#> [1] "3/10"
#> [1] "4/10"
#> [1] "5/10"
#> [1] "6/10"
#> [1] "7/10"
#> [1] "8/10"
#> [1] "9/10"
#> [1] "10/10"
#>  [1] 0.000 0.001 0.126 0.112 0.091 0.584 0.885 0.677 0.604 0.363
```

<img src="man/figures/README-cutoff-1.png" title="plot of chunk cutoff" alt="plot of chunk cutoff" width="100%" />

```
#> $p.value
#>  [1] 0.000 0.001 0.126 0.112 0.091 0.584 0.885 0.677 0.604 0.363
#> 
#> $p.value.inter
#>  [1] 0.0003073808 0.0222769859 0.0521597921 0.0819131661
#>  [5] 0.1859011443 0.5539131661 0.8106723719 0.7795175496
#>  [9] 0.6267996116 0.3396322205
#> 
#> $sequence
#>  [1] 0.00000000 0.04444444 0.08888889 0.13333333 0.17777778
#>  [6] 0.22222222 0.26666667 0.31111111 0.35555556 0.40000000
```

Analyze the network with a cutoff set to the previouly found 0.14 optimal value.


``` r
analyze_network(network,nv=0.14)
#>    node betweenness degree    output  closeness
#> 1     1           0      3 0.8133348 16.4471148
#> 2     2           0      3 0.8884602  7.9547696
#> 3     3           0      1 0.1749376 10.0055952
#> 4     4           0      3 0.5159878 11.4854812
#> 5     5           0      0 0.0000000  0.0000000
#> 6     6           0     13 3.5794097 25.6388630
#> 7     7           0      4 0.9685114  7.0356510
#> 8     8           0      0 0.0000000  0.0000000
#> 9     9           0      0 0.0000000  0.0000000
#> 10   10           3      2 0.6047036  3.0439695
#> 11   11          31     10 1.9146802  8.2263869
#> 12   12           1      1 0.2056836  0.8489352
#> 13   13          97     19 3.7578360 18.6066356
#> 14   14           0      0 0.0000000  0.0000000
#> 15   15           0      0 0.0000000  0.0000000
#> 16   16           0      0 0.0000000  0.0000000
#> 17   17           2      2 0.3985715  1.6450577
#> 18   18           9      1 0.1408025  0.5811461
#> 19   19           0      0 0.0000000  0.0000000
#> 20   20           0      0 0.0000000  0.0000000
#> 21   21           0      0 0.0000000  0.0000000
#> 22   22           0      0 0.0000000  0.0000000
#> 23   23           0      0 0.0000000  0.0000000
#> 24   24           0      0 0.0000000  0.0000000
#> 25   25           0      0 0.0000000  0.0000000
#> 26   26           0      0 0.0000000  0.0000000
#> 27   27           0      0 0.0000000  0.0000000
#> 28   28           9      2 0.3786198  1.5627095
#> 29   29           0      0 0.0000000  0.0000000
#> 30   30          28      6 1.1216028  4.6292854
#> 31   31           0      0 0.0000000  0.0000000
#> 32   32           0      0 0.0000000  0.0000000
#> 33   33           0      0 0.0000000  0.0000000
#> 34   34           3      1 0.1988608  0.8207750
#> 35   35           0      0 0.0000000  0.0000000
#> 36   36           0      0 0.0000000  0.0000000
#> 37   37           0      0 0.0000000  0.0000000
#> 38   38           0      0 0.0000000  0.0000000
#> 39   39           0      0 0.0000000  0.0000000
#> 40   40           0      0 0.0000000  0.0000000
#> 41   41           0      0 0.0000000  0.0000000
#> 42   42           0      0 0.0000000  0.0000000
#> 43   43           0      0 0.0000000  0.0000000
#> 44   44           0      0 0.0000000  0.0000000
#> 45   45           0      0 0.0000000  0.0000000
#> 46   46           0      0 0.0000000  0.0000000
#> 47   47           0      0 0.0000000  0.0000000
#> 48   48           0      0 0.0000000  0.0000000
#> 49   49           0      0 0.0000000  0.0000000
#> 50   50           0      0 0.0000000  0.0000000
#> 51   51           0      0 0.0000000  0.0000000
#> 52   52           0      0 0.0000000  0.0000000
#> 53   53           0      0 0.0000000  0.0000000
#> 54   54           0      0 0.0000000  0.0000000
#> 55   55           0     10 3.2277268 22.2234874
#> 56   56          13      3 0.7360000  3.5063367
#> 57   57           0      0 0.0000000  0.0000000
#> 58   58           1      1 0.2291004  0.9455855
#> 59   59           0      0 0.0000000  0.0000000
#> 60   60           0      2 0.3955933  7.4313822
#> 61   61           0      3 1.2813639  7.0435303
#> 62   62           0      0 0.0000000  0.0000000
#> 63   63           0      0 0.0000000  0.0000000
#> 64   64           2      2 0.3878745  1.6009071
#> 65   65           0      2 1.2169141 10.8093303
#> 66   66           5      1 0.3016614  1.2450723
#> 67   67           3      3 0.5958934  2.4594808
#> 68   68           0      0 0.0000000  0.0000000
#> 69   69           0      0 0.0000000  0.0000000
#> 70   70           0      0 0.0000000  0.0000000
#> 71   71          26      8 1.6479964  6.8019142
#> 72   72           0      0 0.0000000  0.0000000
#> 73   73           0      0 0.0000000  0.0000000
#> 74   74           0      0 0.0000000  0.0000000
```

$Id: README,v 1.2 2001/09/05 17:52:46 dan Exp $

Cascade

A program for performing noise figure, gain, and third order intercept
analysis of a cascade.

See doc/index.html for documentation on using the program

See INSTALL for installation help

This program is Copyright 1997-2001 Dan McMahill, please refer
to the file 'COPYING' for details.

#' Simulated M data for examples.
#' 
#' Simulated M microarray.
#' 
#' 
#' @name M
#' @docType data
#' @keywords datasets
#' @examples
#' 
#' data(M)
#' head(M)
#' 
NULL

#' Reverse-engineered network of the simulated data.
#' 
#' The reverse-engineered network of the simulated data (M and Net).
#' 
#' 
#' @name Net_inf
#' @docType data
#' @keywords datasets
#' @examples
#' 
#' data(Net_inf)
#' str(Net_inf)
#' 
NULL

#' Simulated network data for examples.
#' 
#' Simulated network.
#' 
#' 
#' @name Net
#' @docType data
#' @keywords datasets
#' @examples
#' 
#' data(Net)
#' str(Net)
#' 
NULL

#' A network object data.
#' 
#' A network object. It is the same as the result in the vignette for the
#' inference of the network.
#' 
#' 
#' @name network
#' @docType data
#' @keywords datasets
#' @examples
#' 
#' data(network)
#' plot(network)
#' print(network)
#' 
NULL

#' Selection of genes.
#' 
#' 20 (at most) genes with differential expression at t1, 20 (at most) genes
#' with differential expression at t2, 20 (at most) genes with differential
#' expression at t3, 20 (at most) genes with differential expression at t4 et
#' 20 (at most) genes with global differential expression were selected.
#' 
#' 
#' @name Selection
#' @docType data
#' @keywords datasets
#' @examples
#' 
#' data(Selection)
#' head(Selection)
#' summary(Selection,3)
#' 
NULL

#' @keywords internal
#' @aliases Cascade-package Cascade NULL
#' @author This package has been written by Frédéric Bertrand, Myriam
#' Maumy-Bertrand and Nicolas Jung with biological insights from Laurent
#' Vallat. Maintainer: Frédéric Bertrand <frederic.bertrand@@lecnam.net>
#' 
#' @references Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
#' Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
#' simulate the diffusion of a signal through a temporal gene network.
#' \emph{Bioinformatics}, btt705.
#' 
#' Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
#' Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
#' circuitry of a cancer cell with predicted intervention in chronic
#' lymphocytic leukemia. \emph{Proceedings of the National Academy of
#' Sciences}, 110(2), 459-464.
"_PACKAGE"

#' @importFrom VGAM rlaplace rpareto zeta 
#' @importFrom grDevices col2rgb colorRamp dev.new grey rainbow rgb
#' @importFrom graphics abline hist legend lines matplot par rect text
#' @importFrom methods new
#' @importFrom stats aggregate cor lm loess model.matrix quantile rbinom reshape rmultinom rnorm runif var wilcox.test
#' @importFrom cluster agnes
#' @importFrom animation ani.options saveHTML
#' @importFrom abind abind
#' @importFrom igraph as_edgelist graph_from_adjacency_matrix layout_with_fr
#' @importFrom igraph neighborhood vcount V
#' @importFrom limma contrasts.fit eBayes lmFit makeContrasts topTable
#' @import nnls
#' @import splines 
#' @import stats4
#' @importFrom tnet as.tnet betweenness_w closeness_w degree_w
#' 
NULL

setGeneric("geneSelection",package="Cascade",def = function(x,y,tot.number,... ){standardGeneric("geneSelection")})
setGeneric("genePeakSelection",package="Cascade",def = function(x,peak,... ){standardGeneric("genePeakSelection")})
setGeneric("unionMicro",package="Cascade",def = function(M1,M2 ){standardGeneric("unionMicro")})
setGeneric("position",package="Cascade",def = function(net,... ){standardGeneric("position")})
setGeneric("geneNeighborhood",package="Cascade",def = function(net,targets,... ){standardGeneric("geneNeighborhood")})
setGeneric("evolution",package="Cascade",def = function(net,list_nv,... ){standardGeneric("evolution")})
setGeneric("inference",package="Cascade",def = function(M,... ){standardGeneric("inference")})
setGeneric("cutoff",package="Cascade",def = function(Omega,... ){standardGeneric("cutoff")})
setGeneric("analyze_network",package="Cascade",def = function(Omega,nv,...){standardGeneric("analyze_network")})
#setGeneric("predict",def = function(object,...){standardGeneric("predict")})
setGeneric("gene_expr_simulation",package="Cascade",def = function(network,...){standardGeneric("gene_expr_simulation")})
setGeneric("compare",package="Cascade",def = function(Net,Net_inf,nv){standardGeneric("compare")})

#' Class \code{"micro_array"}
#' 
#' The \code{"micro_array"} class
#' 
#' 
#' @name micro_array-class
#' @docType class
#' @section Objects from the Class: Objects can be created by calls of the form
#' \code{new("micro_array", ...)}. %% ~~ describe objects here ~~
#' @keywords classes
#' @examples
#' 
#' showClass("micro_array")
#' 
#' @export
setClass(
  Class = "micro_array",
  representation(
    microarray = "matrix",
    name = "vector",
    group = c("vector", NULL),
    start_time = c("vector", NULL),
    time = c("vector", NULL),
    subject = "numeric"
  ),
  prototype = prototype(group = 0, start_time = 0),
  validity = function(object) {
    
    if(dim(object@microarray)[2] != length(object@time)*object@subject){
      
      stop("[Error: ]Number of colomns must be equal to the number of time points * the number of subject")
    }
    if(dim(object@microarray)[1] != length(object@name)&&length(object@name)!=0){
      
      stop("[Error: ] Length of the vector of names must equal to the number of genes")
    }
    
    if(dim(object@microarray)[1] != length(object@group)&&length(object@group)!=1){
      
      print(object@group)
      stop("[Error: ] Length of the vector of group must equal to the number of genes or null")
    }
    
    if(dim(object@microarray)[1] != length(object@start_time)&&length(object@start_time)!=1){
      
      
      stop("[Error: ] Length of the vector of starting time must equal to the number of genes or null")
    }
    
    
    if(object@subject<1){
      
      stop("[Error: ] There must be at least one subject")
    }	
    
  }
  
)

#' Class \code{"network"}
#' 
#' The \code{"network"} class
#' 
#' 
#' @name network-class
#' @docType class
#' @section Objects from the Class: Objects can be created by calls of the form
#' \code{new("network", ...)}. %% ~~ describe objects here ~~
#' @keywords classes
#' @examples
#' 
#' showClass("network")
#' 
#' @export
setClass(Class = "network",
         representation(network="matrix",name="vector",F="array",convF="matrix",convO="vector",time_pt="vector")
)


#' Class \code{"micropredict"}
#' 
#' The \code{"micropredict"} class
#' 
#' 
#' @name micropredict-class
#' @docType class
#' @section Objects from the Class: Objects can be created by calls of the form
#' \code{new("micropredict", ...)}.
#' @keywords classes
#' @examples
#' 
#' showClass("micropredict")
#' 
#' @export
setClass(Class = "micropredict",
         representation(microarray_unchanged="micro_array"
                        ,microarray_changed="micro_array"
                        ,microarray_predict="micro_array"
                        ,nv="numeric"
                        ,network="network"
                        ,targets="numeric")
)

# Helper file for Cascade tests
# Use this space for shared utilities or global skips if needed.

# Muffle only warnings whose message matches a regex pattern,
# keeping other warnings visible so we don't hide real issues.
muffle_warnings_matching <- function(expr, patterns) {
  pats <- as.character(patterns)
  withCallingHandlers(expr, warning = function(w) {
    msg <- conditionMessage(w)
    if (any(vapply(pats, function(p) grepl(p, msg, ignore.case = TRUE), logical(1)))) {
      invokeRestart("muffleWarning")
    }
  })
}

skip_on_cran()
skip_on_ci()

.loess_warn_patterns <- c(
  "singularit",
  "span too small",
  "fewer data values",
  "pseudoinverse used",
  "neighborhood radius",
  "reciprocal condition number"
)

test_that("cutoff() returns p-values and smoothed p-values", {
  skip_if_not_installed("VGAM")
  data(Net, package = "Cascade")
  seq_nv <- seq(0, 0.2, length.out = 5)
  
  res <- muffle_warnings_matching(
    cutoff(Net, sequence = seq_nv),
    patterns = .loess_warn_patterns
  )
  
  expect_type(res, "list")
  expect_true(all(c("p.value","p.value.inter","sequence") %in% names(res)))
  expect_equal(length(res$p.value), length(seq_nv))
  expect_equal(length(res$p.value.inter), length(seq_nv))
  expect_true(all(is.finite(res$p.value)))
  expect_true(all(res$p.value >= 0 & res$p.value <= 1))
})

test_that("compare() returns 5 performance metrics between 0 and 1", {
  data(Net, package = "Cascade")
  data(Net_inf, package = "Cascade")
  nv <- 0
  perf <- Cascade::compare(Net, Net_inf, nv)
  expect_type(perf, "double")
  expect_equal(length(perf), 5L)
  expect_true(all(perf >= 0 & perf <= 1))
})

#' Overview of a micro_array object
#' 
#' Overview of a micro_array object.
#' 
#' 
#' @aliases methods head-methods head,ANY-method head,micro_array-method
#' @section Methods: \describe{
#' 
#' \item{list("signature(x = \"ANY\")")}{ Gives an overview. }
#' 
#' \item{list("signature(x = \"micro_array\")")}{ Gives an overview. } }
#' 
#' @param x an object of class `micro_array`.
#' @param ... additional parameters
#' @keywords methods
#' @examples
#' 
#'  if(require(CascadeData)){
#' 	data(micro_US)
#' 	micro_US<-as.micro_array(micro_US,time=c(60,90,210,390),subject=6)
#' 	head(micro_US)
#' 	}
#' @exportMethod head
setMethod("head","micro_array",function(x,...)
{
	cat("The matrix :")
	cat("\n")
	cat("\n")
	K<-dim(x@microarray)[2]
	K<-min(K,3)
	
	print(head(x@microarray[,1:K]))
	cat("...")
	cat("\n")
	cat("\n")
	cat("Vector of names :")
	cat("\n")
	print(head(x@name))
	cat("...")
	cat("\n")
	cat("Vector of group :")
	cat("\n")
	print(head(x@group))
	cat("...")
	cat("\n")
	cat("Vector of starting time :")
	cat("\n")
	print(head(x@start_time))
	cat("...")
	cat("\n")
	cat("Vector of time :")
	cat("\n")
	print(x@time)
	cat("\n")
	cat("Number of subject :")
	cat("\n")
	print(x@subject)
}
)


#' Methods for Function \code{print}
#' 
#' Methods for function \code{print}
#' 
#' 
#' @name print-methods
#' @aliases print-methods print,ANY-method print,micro_array-method
#' print,network-method
#' @param x an object of class micro-array or network
#' @param ... additional parameters
#' @docType methods
#' @keywords methods
#' @examples
#' 
#' data(Net)
#' print(Net)
#' 
#' data(M)
#' print(M)
#' 
#' @exportMethod print
setMethod("print","micro_array",function(x,...)
{
cat(paste("This is a micro_array S4 class. It contains : \n - (@microarray) a matrix of dimension ",dim(x@microarray)[1],"*",dim(x@microarray)[2],"\n          .... [gene expressions] \n - (@name) a vector of length ",length(x@name)," .... [gene names] \n","- (@group) a vector of length ",length(x@group)," .... [groups for genes] \n","- (@start_time) a vector of length ",length(x@start_time),"\n          .... [first differential expression for genes] \n","- (@time)a vector of length ",length(x@time)," .... [time points]\n","- (@subject) an integer  .... [number of subject]")) 	
})


#' Methods for Function \code{summary}
#' 
#' Methods for function \code{summary}
#' 
#' 
#' @name summary-methods
#' @aliases summary-methods summary,ANY-method summary,micro_array-method
#' @docType methods
#' @keywords methods
#' @param object an object of class micro-array
#' @param nb.graph (optionnal) choose the graph to plot. Displays all graphs by default.
#' @param ... additional parameters.
#' @examples
#' 
#' data(M)
#' summary(M)
#' 
#' @exportMethod summary
setMethod("summary","micro_array",function(object,nb.graph=NULL,...)
{
	require(cluster)
	print(summary(object@microarray))
	G<-object@microarray
	colnames(G)<-paste(rep(paste("T",object@time),object@subject), as.character(rep(paste("subject",1:object@subject),each=length(object@time))))
	z <- cor(G)
	#require(lattice)
	#rownames(z)<-NULL
	if(is.null(nb.graph) || nb.graph==1 ){
	print(lattice::levelplot(z,aspect="iso",xlab="",ylab="", 
scales=list(x=list(rot=90)), 
ylab.right = "Level of correlation",
par.settings = list(layout.widths = list(axis.key.padding = 0,
                                                      ylab.right = 2))))
 	}
#	if(is.null(nb.graph)){dev.new()}
	G<-object@microarray
	colnames(G)<-paste(rep(paste("T",object@time),object@subject), as.character(rep(paste("subject",1:object@subject),each=length(object@time))))
	w<-agnes(t(G))[1]$order
	G<-G[,w]
	z <- cor(G)
	
	if(is.null(nb.graph) || nb.graph==2 ){
	print(lattice::levelplot(z,aspect="iso",xlab="",ylab="", 
scales=list(x=list(rot=90)), 
ylab.right = "Level of correlation",
par.settings = list(layout.widths = list(axis.key.padding = 0,
                                                      ylab.right = 2))))
                     }
	if(dim(object@microarray)[1]<1000){
#		if(is.null(nb.graph)){dev.new()}
		
	R<-object@microarray
	w<-agnes(R)[1]$order
	R<-R[w,]
	z <- cor(t(R))

	if(is.null(nb.graph) || nb.graph==3 ){
	print(lattice::levelplot(z,aspect="iso",xlab="",ylab="", 
scales=list(x=list(rot=90)), 
ylab.right = "Level of correlation",
par.settings = list(layout.widths = list(axis.key.padding = 0,
                                                      ylab.right = 2))))
 	}
	}}
	)

#' Dimension of the data
#' 
#' Dimension of the data
#' 
#' 
#' @name dim
#' @aliases dim dim-methods dim,micro_array-method
#' @docType methods
#' @section Methods: \describe{
#' 
#' \item{list("signature(x = \"micro_array\")")}{ Gives the dimension of the
#' matrix of measurements. } }
#' 
#' @param x an object of class "micro-array
#' @keywords methods
#' @examples
#' 
#'  if(require(CascadeData)){
#' 	data(micro_US)
#' 	micro_US<-as.micro_array(micro_US,time=c(60,90,210,390),subject=6)
#' 	dim(micro_US)
#' 	}
#' 
#' @exportMethod dim
setMethod("dim","micro_array",function(x)
{
	return(dim(x@microarray))
})


#' Plot
#' 
#' Considering the class of the argument which is passed to plot, the graphical
#' output differs.
#' 
#' 
#' @name plot-methods
#' @aliases plot-methods plot,ANY,ANY-method plot,micro_array,ANY-method
#' plot,network,ANY-method plot,micropredict,ANY-method
#' @docType methods
#' @section Methods: \describe{
#' 
#' \item{list("signature(x = \"micro_array\", y = \"ANY\",...)")}{ \describe{
#' \item{x}{a micro_array object} \item{list_nv}{a vector of cutoff at which
#' the network should be shown} } } \item{list("signature(x = \"network\", y =
#' \"ANY\",...)")}{ \describe{ \item{x}{a network object}
#' \item{list()}{Optionnal arguments: \describe{ \item{gr}{a vector giving the
#' group of each gene} \item{choice}{what graphic should be plotted: either "F"
#' (for a representation of the matrices F) or "network".} \item{nv}{the level
#' of cutoff. Defaut to 0.} \item{ini}{using the ``position'' function, you can
#' fix the position of the nodes} \item{color.vertex}{a vector defining the
#' color of the vertex} \item{ani}{animated plot?} \item{size}{vector giving the size of the plot. Default
#' to c(2000,1000)} \item{video}{if ani is TRUE and video is TRUE, the
#' animation result is a GIF video} \item{label_v}{vector defining the vertex
#' labels} \item{legend.position}{position of the legend}
#' \item{frame.color}{color of the frames} \item{label.hub}{logical ; if TRUE
#' only the hubs are labeled} \item{edge.arrow.size}{size of the arrows ;
#' default to 0.7} \item{edge.thickness}{edge thickness ; default to 1.} } }}}
#' 
#' \item{list("signature(x = \"micropredict\", y = \"ANY\",...)")}{ \describe{
#' \item{x}{a micropredict object} \item{list()}{Optionnal arguments: see plot
#' for network} }} }
#' 
#' @param x a micro_array object, a network object or a micropredict object
#' @param y optional and not used if x is an appropriate structure
#' @param gr a vector giving the group of each gene 
#' @param choice what graphic should be plotted: either "F"
#' (for a representation of the matrices F) or "network".
#' @param nv the level of cutoff. Defaut to `0`.
#' @param ini using the ``position'' function, you can
#' fix the position of the nodes.
#' @param color.vertex a vector defining the color of the vertex.
#' @param ani animated plot?
#' @param taille vector giving the size of the plot. Default to `c(2000,1000)`.
#' @param video if ani is TRUE and video is TRUE, the result of the animation is saved as an animated GIF. 
#' @param label_v vector defining the vertex labels.
#' @param legend.position position of the legend.
#' @param frame.color color of the frames.
#' @param label.hub logical ; if TRUE only the hubs are labeled.
#' @param edge.arrow.size size of the arrows ; default to 0.7.
#' @param edge.thickness edge thickness ; default to 1.
#' @param weight.node nodes weighting. Defaults to `NULL`.
#' @param horiz landscape? Defaults to `TRUE`.
#' @param time sets the time for plot of the prediction. Defaults to `NULL`
#' @param ... additional parameters
#' 
#' @keywords methods
#' @examples
#' 
#' data(Net)
#' plot(Net)
#' 
#' data(M)
#' plot(M)
#' 
#' data(Selection)
#' data(network)
#' nv<-0.11
#' plot(network,choice="network",gr=Selection@group,nv=nv,label_v=Selection@name,
#' edge.arrow.size=0.9,edge.thickness=1.5)
#' 
#' @exportMethod plot
setMethod("plot","micro_array",function(x,y,...)
{
		
#require(lattice)
#require(grid)	
xs<-t(x@microarray)

rownames(xs)<-1:dim(xs)[1]
ys<-x@time
YS<-rep(ys,x@subject)
suj<- rep(paste("Subject",1:x@subject,sep=" "),each=length(ys))
ID<-paste("ID",1:dim(xs)[1],sep="")
cclus<-unique(x@group)
cclus<-cclus[order(cclus)]
U<-data.frame(xs,suj,ys)
V<- reshape(U,idvar="ID",varying=list(1:dim(xs)[2]), v.names = "conc", direction = "long")


if(length(unique(x@group))>1){
gr<-rep(paste("Cluster",x@group,sep=" "),each=x@subject*length(x@time))


#dev.new()
print(lattice::xyplot(V$conc~V$ys|V$suj,as.table=TRUE,xlab="Time",ylab="Gene Expression",group=rep(1:(x@subject*dim(xs)[2]),each=length(x@time)),type="l",scales=list(x=list(relation="free",at=x@time),y=list(relation="free")),col=rep(x@group,each=x@subject),key=list(
space="right",
lines=list(type="l",col=cclus),
text=list(text=paste("Cluster",as.character(cclus)))
)))
#dev.new()
print(lattice::xyplot(V$conc~V$ys|gr,as.table=TRUE,xlab="Time",ylab="Gene Expression",group=rep(1:(x@subject*dim(xs)[2]),each=length(x@time)),type="l",scales=list(x=list(relation="free",at=x@time),y=list(relation="free")),col=rep(1:x@subject,dim(xs)[2]),key=list(
space="right",
lines=list(type="l",col=1:x@subject),
text=list(text=paste("Subject",as.character(1:x@subject)))
)))
for(i in 1:x@subject){
ss<-V$suj
sss<-ss==paste("Subject",i,sep=" ")
#dev.new()
print(lattice::xyplot(V$conc[sss]~V$ys[sss]|gr[sss],as.table=TRUE,xlab="Time",ylab="Gene Expression",type="l",main=paste("Subject",i,sep=" "),group=rep(1:(x@subject*dim(xs)[2]),each=length(x@time))[sss],scales=list(x=list(relation="free",at=x@time),y=list(relation="free"))))
}
}
else{
  lattice::xyplot(V$conc~V$ys|V$suj,xlab="Time",as.table=TRUE,group=rep(1:(x@subject*dim(xs)[2]),each=length(x@time)),ylab="Gene Expression",scales=list(x=list(relation="free",at=x@time),y=list(relation="free")),type="l",col="black")
	}
}
)

 

#' Methods for selecting genes
#' 
#' Selection of differentially expressed genes.
#' 
#' @name geneSelection
#' @aliases genePeakSelection geneSelection genePeakSelection-methods
#' geneSelection-methods geneSelection,list,list,numeric-method
#' geneSelection,micro_array,micro_array,numeric-method
#' genePeakSelection,micro_array,numeric-method
#' @param x either a micro_array object or a list of micro_array objects. In
#' the first case, the micro_array object represents the stimulated
#' measurements. In the second case, the control unstimulated data (if present)
#' should be the first element of the list.
#' @param y either a micro_array object or a list of strings. In the first
#' case, the micro_array object represents the stimulated measurements. In the
#' second case, the list is the way to specify the contrast: \describe{
#' \item{First element:}{ condition, condition&time or pattern. The condition
#' specification is used when the overall is to compare two conditions.  The
#' condition&time specification is used when comparing two conditions at two
#' precise time points. The pattern specification allows to decide which time
#' point should be differentially expressed.} \item{Second element:}{a vector
#' of length 2. The two conditions which should be compared. If a condition is
#' used as control, it should be the first element of the vector. However, if
#' this control is not measured throught time, the option cont=TRUE should be
#' used.} \item{Third element:}{depends on the first element.  It is no needed
#' if condition has been specified.  If condition&time has been specified, then
#' this is a vector containing the time point at which the comparison should be
#' done. If pattern has been specified, then this is a vector of 0 and 1 of
#' length T, where T is the number of time points. The time points with desired
#' differential expression are provided with 1.  }}
#' @param tot.number an integer. The number of selected genes. If tot.number <0
#' all differentially genes are selected. If tot.number > 1, tot.number is the
#' maximum of diffenrtially genes that will be selected.  If 0<tot.number<1,
#' tot.number represents the proportion of diffenrentially genes that are
#' selected.
#' @param peak interger. At which time points measurements should the genes be
#' selected [optionnal for geneSelection].
#' @param data_log logical (default to TRUE); should data be logged ?
#' @param wanted.patterns a matrix with wanted patterns [only for geneSelection].
#' @param forbidden.patterns a matrix with forbidden patterns [only for geneSelection].
#' @param durPeak vector of size 2 (default to c(1,1)) ; the first elements gives the length of the peak at
#' the left, the second at the right. [only for genePeakSelection]
#' @param abs_val logical (default to TRUE) ; should genes be selected on the
#' basis of their absolute value expression ? [only for genePeakSelection]
#' @param alpha_diff float; the risk level
#' @param alpha float; the risk level. Default to `alpha=0.05`
#' @param Design the design matrix of the experiment. Defaults to `NULL`.
#' @param lfc log fold change value used in limma's `topTable`. Defaults to 0.
#' @param cont use contrasts. Defaults to `FALSE`. 
#' @param f.asso function used to assess the association between the genes. 
#' The default value `NULL` implies the use of the usual `mean` function. 
#' 
#' @return A micro_array object.
#' @author Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
#' @references Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
#' Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
#' simulate the diffusion of a signal through a temporal gene network.
#' \emph{Bioinformatics}, btt705.
#' 
#' Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
#' Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
#' circuitry of a cancer cell with predicted intervention in chronic
#' lymphocytic leukemia. \emph{Proceedings of the National Academy of
#' Sciences}, 110(2), 459-464.
#' @keywords methods
#' @examples
#' 
#' \donttest{
#'  if(require(CascadeData)){
#' 	data(micro_US)
#' 	micro_US<-as.micro_array(micro_US,time=c(60,90,210,390),subject=6)
#' 	data(micro_S)
#' 	micro_S<-as.micro_array(micro_S,time=c(60,90,210,390),subject=6)
#' 
#'   #Basically, to find the 50 more significant expressed genes you will use:
#'   Selection_1<-geneSelection(x=micro_S,y=micro_US,
#'   tot.number=50,data_log=TRUE)
#'   summary(Selection_1)
#'   
#'   #If we want to select genes that are differentially 
#'   #at time t60 or t90 :
#'   Selection_2<-geneSelection(x=micro_S,y=micro_US,tot.number=30,
#'   wanted.patterns=
#'   rbind(c(0,1,0,0),c(1,0,0,0),c(1,1,0,0)))
#'   summary(Selection_2)
#' 
#'   #To select genes that have a differential maximum of expression at a specific time point.
#'   
#'   Selection_3<-genePeakSelection(x=micro_S,y=micro_US,peak=1,
#'   abs_val=FALSE,alpha_diff=0.01)
#'   summary(Selection_3)
#'   }
#' 
#'  if(require(CascadeData)){
#' data(micro_US)
#' micro_US<-as.micro_array(micro_US,time=c(60,90,210,390),subject=6)
#' data(micro_S)
#' micro_S<-as.micro_array(micro_S,time=c(60,90,210,390),subject=6)
#' #Genes with differential expression at t1
#' Selection1<-geneSelection(x=micro_S,y=micro_US,20,wanted.patterns= rbind(c(1,0,0,0)))
#' #Genes with differential expression at t2
#' Selection2<-geneSelection(x=micro_S,y=micro_US,20,wanted.patterns= rbind(c(0,1,0,0)))
#' #Genes with differential expression at t3
#' Selection3<-geneSelection(x=micro_S,y=micro_US,20,wanted.patterns= rbind(c(0,0,1,0)))
#' #Genes with differential expression at t4
#' Selection4<-geneSelection(x=micro_S,y=micro_US,20,wanted.patterns= rbind(c(0,0,0,1)))
#' #Genes with global differential expression 
#' Selection5<-geneSelection(x=micro_S,y=micro_US,20)
#' 
#' #We then merge these selections:
#' Selection<-unionMicro(list(Selection1,Selection2,Selection3,Selection4,Selection5))
#' print(Selection)
#' 
#' #Prints the correlation graphics Figure 4:
#' summary(Selection,3)
#' 
#' ##Uncomment this code to retrieve geneids.
#' #library(org.Hs.eg.db)
#' #
#' #ff<-function(x){substr(x, 1, nchar(x)-3)}
#' #ff<-Vectorize(ff)
#' #
#' ##Here is the function to transform the probeset names to gene ID.
#' #
#' #library("hgu133plus2.db")
#' #
#' #probe_to_id<-function(n){  
#' #x <- hgu133plus2SYMBOL
#' #mp<-mappedkeys(x)
#' #xx <- unlist(as.list(x[mp]))
#' #genes_all = xx[(n)]
#' #genes_all[is.na(genes_all)]<-"unknown"
#' #return(genes_all)
#' #}
#' #Selection@name<-probe_to_id(Selection@name)
#' }
#' }
#' 
#' @exportMethod geneSelection
setMethod(f="geneSelection", signature=c("micro_array","micro_array","numeric"),
	definition=function(x,
	                    y,
	                    tot.number,
	                    data_log=TRUE,
	                    wanted.patterns=NULL,
	                    forbidden.patterns=NULL,
	                    peak=NULL,
	                    alpha=0.05,
	                    Design=NULL,
	                    lfc=0){
   
	  limma_version <- tryCatch(utils::packageVersion("limma"),
	                           error = function(e) NULL)
	  if(is.null(limma_version)){
	    stop("Please install the limma BioConductor package")
	  }
	  if(limma_version < "3.18.13"){
	    stop("Upgrade your version of Limma (>= 3.18.13)")
	  }
    
    
    indic<-0
    M1<-x
    M2<-y
  if(is.null(M2)){
  indic<-1
  M2<-M1
  }

		if(data_log==TRUE){
			M1_mic<-log(M1@microarray)
			M2_mic<-log(M2@microarray)
		} else{
			M1_mic<-(M1@microarray)
			M2_mic<-(M2@microarray)
		}
		
		if(is.null(rownames(M1_mic))){rownames(M1_mic)<-paste("probe ",1:dim(M1_mic)[1])}
if(is.null(rownames(M2_mic))){rownames(M2_mic)<-paste("probe ",1:dim(M2_mic)[1])}

      if(indic==1){ M2_mic<-M2_mic*0}
      
      

		colnames(M1_mic)<-paste(rep("US",length(M1@time)*M1@subject),rep(M1@time,M1@subject), sep="")
		colnames(M2_mic)<-paste(rep("S",length(M2@time)*M2@subject),rep(M2@time,M2@subject), sep="")
		#rownames(M1_mic)<- paste("probe",rownames(M1_mic) ) 
		#rownames(M2_mic)<-  paste("probe",rownames(M2_mic) ) 
    M<-cbind(M1_mic,M2_mic)

		T<-length(M1@time)
		
		#Construction de la matrice de design

    if(is.null(Design)){
		design<-t(matrix(rep(0,(M1@subject+M2@subject)*T*(T*2)),2*T))
		
		for(i in 1:(T)){
			design[which(colnames(M)%in%paste("US",M1@time[i],sep="")),i]<-1
			design[which(colnames(M)%in%paste("S",M2@time[i],sep="")),i+T]<-1	
		}

     
		vnom<-paste(c(paste(rep("US_time",length(M1@time)),M1@time[1:(length(M1@time))],sep=""),paste(rep("S_time",length(M1@time)),M1@time[1:(length(M1@time))],sep="")),sep="")
		colnames(design)<-vnom
		}else{
		 design<-Design$X		
		}
		#block<-rep(1:(M1@subject*2),each=T)
		#dupcor <- duplicateCorrelation(M,design,block=block)
		#model<-lmFit(M,design,block=block)
		model<-lmFit(M,design)
		if(is.null(Design)){
    diff<-paste(vnom[1:T],vnom[1:T+length(M1@time)],sep="-")
		contr<-makeContrasts(contrasts=diff,levels=vnom)  		}else{
		 contr<-Design$contr
		}
		model2<-contrasts.fit(model,contr)
		model.test<-eBayes(model2)
		

		
p.val.ind<-model.test$p.value
rownames(p.val.ind)<-rownames(M1_mic) 

p.val.all<-topTable(model.test,p.value=alpha,number=dim(M)[1],lfc=lfc)
kkkk<-0
if(is.null(p.val.all$ID)){
  kkkk<-1
		ID<-row.names(p.val.all)
		p.val.all<-cbind(ID,p.val.all)
		f_nom<-function(nom){
			which(x@name %in% nom)			
		}
		f_nom<-Vectorize(f_nom)
		row.names(p.val.all)<-unlist(f_nom(as.character(p.val.all$ID)))
		}
if(tot.number>0 && tot.number<=1){
 tot.number<-round(tot.number*dim(p.val.all)[1])
}

if(kkkk<-0){
ind.class<-rownames(p.val.all)
p.val.ind.class<-p.val.ind[(ind.class),]
}else{
  ind.class<-rownames(p.val.all)
  p.val.ind.class<-p.val.ind[as.numeric(ind.class),]
  
}	
		f.p.val<-function(x){
			if(x<alpha){return(1)}
			else{return(0)}
		}

		p.val.ind.class.cat<-apply(p.val.ind.class,c(1,2),f.p.val)
		choix<-cbind(p.val.ind.class.cat,rep(0,dim(p.val.ind.class.cat)[1]))
		ch<-dim(choix)[2]
		f_test_patt<-function(x,pat){
				if(sum(abs(x-pat))==0){
					return(1)
				}
				else{
					return(0)		
				}}

		if(!is.null(forbidden.patterns)){			
			
			for(i in 1:dim(forbidden.patterns)[1]){
				f_test2<-function(x){f_test_patt(x,forbidden.patterns[i,])}
				S<-apply(choix[,1:(ch-1)],1,f_test2)
				choix<-choix[S==0,]
			}
		}


		if(!is.null(wanted.patterns)){
			sel<-rep(0,dim(choix)[1])
			choix[,ch]<-choix[,ch]*0
			for(i in 1:dim(wanted.patterns)[1]){
				f_test2<-function(x){f_test_patt(x,wanted.patterns[i,1:T])}
				sel<-sel+apply(choix[,1:(ch-1)],1,f_test2)

			}				
			for(j in 1:length(sel)){
					if((sum(choix[1:(j-1),ch])<tot.number || tot.number<0) && sel[j]>=1){choix[j,ch]<-1}
				}			
		}


		f_test_peak<-function(x,peak){
				x[peak]
				}

		if(!is.null(peak)){
				f_test2<-function(x){f_test_peak(x,peak)}
				
				# if(tot.number>0){
				# S<-apply(choix[,1:(ch-1)],1,f_test2)
				# for(j in 1:length(S)){
					# if(sum(S[1:j])<=tot.number && S[j]==1){choix[j,ch]<-1}
				# }
				# }
				# else{
					
					choix[,ch]<-apply(choix[,1:(ch-1)],1,f_test2)
					
					#}
				
			}
		
		
		if(tot.number>0 && is.null(wanted.patterns) ){
			i<-1
			PN<-dim(choix)[1]
			PN<-1:PN
			
			if(is.null(peak)){
			while(sum(choix[,ch])<tot.number){
				
				choix[i,ch]<-1
				i<-i+1
			}
			choix[PN>=i,ch]<-0
			choix<-choix[choix[,ch]==1,]
		}
		else{
			choix<-choix[choix[,ch]==1,]
			PN<-dim(choix)[1]
			choix<-choix[1:min(PN,tot.number),]
			}
		
		}

		if((!is.null(wanted.patterns))  ){choix<-choix[choix[,ch]==1,]}
		if(!is.null(peak)&&tot.number<=0){choix<-choix[choix[,ch]==1,]}
	
		temps.prem<-function(x){
			u<-0
			i<-1
			while(u==0){
				if(x[i]==1){return(i)}
				else
				i=i+1
			}
		}
		
		
		R<-apply(choix,1,temps.prem)
		n<-length(R)
		MM1<-M1_mic[rownames(choix),]-M2_mic[rownames(choix),]


r3<-function(choix){
sortie<-NULL
for(i in 1:length(choix)){
sortie<-c(sortie,which(rownames(M1_mic)%in%rownames(choix)[i]))
}
sortie
}


		
		M<-new("micro_array",microarray=MM1,name=M1@name[r3(choix)],time=M1@time,subject=M1@subject,group=as.vector(R),start_time=as.vector(R))
		
		
			
			return(M)

	}

)

#' @rdname geneSelection
setMethod(f="geneSelection", 
	signature=c("list","list","numeric"),
	definition=function(x,
	                    y,
	                    tot.number,
	                    data_log=TRUE,
	                    alpha=0.05,
	                    cont=FALSE,
	                    lfc=0,
	                    f.asso=NULL){
  
  
	  limma_version <- tryCatch(utils::packageVersion("limma"),
	                           error = function(e) NULL)
	  if(is.null(limma_version)){
	    stop("Please install the limma BioConductor package")
	  }
	  
	  if(limma_version < "3.18.13"){
	    stop("Upgrade your version of Limma (>= 3.18.13)")
	  }

  
  M<-x
  contrast<-y
 
  M_mic<-M
n<-length(M)
Time<-length(M[[2]]@time)

if(cont==TRUE && is.null(f.asso) ){
	
	f.asso<-"mean"
	
}

Subj<-(M[[2]]@subject)


		if(data_log==TRUE){
			
			for(i in 1:n){
			M_mic[[i]]<-log(M[[i]]@microarray)
			}
		} else{
			for(i in 1:n){
			M_mic[[i]]<-(M[[i]]@microarray)
			}
		}
		
		
		
		
		#colN<-function(N,i){
#			colnames(N)<-paste(rep("Time ",Time*Subj),rep(M[[1]]@time,Subj), sep="")
#			return((N))
#			}
	#	M_mic<-rapply(M_mic,colN,how="list")
#		
#		
#		
    	Ma<-abind::abind(M_mic,along=2)
		T<-Time
		
		
		#Construction de la matrice de design

    
		condition<-as.factor(paste("condition",rep(1:n,unlist(lapply(M,ncol))),sep=""))
		
    if(cont==FALSE){
    timeT<-paste("time",rep(1:T,sum(unlist(lapply(M,function(x){x@subject})))
),sep="") }else{
       timeT<-c(rep("time0",ncol(M[[1]]) ),paste("time",rep(1:T,sum(unlist(lapply( M[2:length(M)],function(x){x@subject})))
),sep=""))

}
        
     Fac<-as.factor( paste(condition,timeT,sep="."))

if(contrast[[1]]!="condition"){
		formule<-~-1+Fac
		design<-model.matrix(formule)
		colnames(design)<-levels(Fac)
   }else{
   	formule<-~-1+condition
   	design<-model.matrix(formule)
		colnames(design)<-levels(condition)
   	}
		  
    model<-lmFit(Ma,design)
       
	
		if(contrast[[1]]=="condition"){
			contrastM<-makeContrasts(contrasts=paste("condition",contrast[[2]][1],"-condition",
      contrast[[2]][2],
			sep=""),levels=design)
		}	
		#if(contrast[[1]]=="time"){
#			
#			if(contrast[[2]][1]!=1){
#				contrastM[contrast[[2]][1]+n]<-contrastM[contrast[[2]][1]+n]+1				
#			}else{
#				contrastM[1:n]<-contrastM[1:n]+1
#				contrastM[(n+1):(n+T-1)]<-contrastM[(n+1):(n+T-1)]-1
#				}
#				
#			if(contrast[[2]][2]!=1){
#				contrastM[contrast[[2]][2]+n]<-contrastM[contrast[[2]][2]+n]-1				
#			}else{
#				contrastM[1:n]<-contrastM[1:n]-1
#				contrastM[(n+1):(n+T-1)]<-contrastM[(n+1):(n+T-1)]+1
#				}
#	
#			
#		}
#		

			if(contrast[[1]]=="patterns" && (cont==FALSE || contrast[[2]][1] != 1)){
			coma=NULL	
			for(j in 1:Time){	
			coma<-c(coma,paste("condition", contrast[[2]][1],".time", j,"-condition", contrast[[2]][2],".time", j,sep=""))
			}
			contrastM<-makeContrasts(contrasts=coma,levels=design)
		
				 }
				 
				 if(contrast[[1]]=="patterns" && (cont==TRUE  && contrast[[2]][1] == 1)){
			coma=NULL	
			for(j in 1:Time){	
			coma<-c(coma,paste("condition", contrast[[2]][1],".time", 0,"-condition", contrast[[2]][2],".time", j,sep=""))
			}
			contrastM<-makeContrasts(contrasts=coma,levels=design)
			
				 }
			 
				if(contrast[[1]]=="condition&time"){
				
				
			coma<-paste("condition", contrast[[2]][1],".time", contrast[[3]][1],"-condition", contrast[[2]][2],".time", contrast[[3]][2],sep="")
			contrastM<-makeContrasts(contrasts=coma,levels=design)
		
				 } 
		
		
     
				model2<-contrasts.fit(model,-contrastM)
		    model.test<-eBayes(model2)
		
 	      p.val.ind<-model.test$p.value



 
nb.tot<-length(M[[1]]@name)



if(contrast[[1]]=="patterns"){
tableT<-matrix(0,nb.tot,Time+1)	
row.names(tableT)<-1:nb.tot
tableT[,Time+1]<-model.test$F.p.value
	for(j in 1:Time){
		
		p.val.all<-topTable(model.test,p.value=alpha,number=nb.tot,lfc=lfc,coef=j)
		
		if(is.null(p.val.all$ID)){
		ID<-row.names(p.val.all)
		p.val.all<-cbind(ID,p.val.all)
		f_nom<-function(nom){
			which(x[[1]]@name %in% nom)			
		}
		f_nom<-Vectorize(f_nom)
		row.names(p.val.all)<-f_nom(p.val.all$ID)
		}
		tableT[as.numeric(row.names(p.val.all)),j]<-1
	
	}
	
	f.test<-function(x1){
		rep<-FALSE
		for(i in 1:nrow(contrast[[3]])){
			if(sum(x1==contrast[[3]][i,])==Time){
				rep<-TRUE
			}
		}
		return(rep)
	}
	
	B<-apply(tableT[,1:Time],1,f.test)
	tableT<-tableT[B,]
	tableT<-tableT[which(tableT[,Time+1]<alpha),]
	tableT<-tableT[order(tableT[,Time+1]),]
	nb.ret<-nrow(tableT)
	if(tot.number>=1 && nb.ret>tot.number){
	nb.ret<-tot.number
}

if(tot.number<1 && tot.number>0){
	nb.ret<-round(nb.ret*tot.number)
	
}
	K1<-M_mic[[contrast[[2]][1]]][as.numeric(row.names(tableT[1:nb.ret,])),]
		K2<-M_mic[[contrast[[2]][2]]][as.numeric(row.names(tableT[1:nb.ret,])),]
	
	
	
	}else{

p.val.all<-topTable(model.test,p.value=alpha,number=nb.tot,lfc=lfc)
	if(is.null(p.val.all$ID)){
		ID<-row.names(p.val.all)
		p.val.all<-cbind(ID,p.val.all)
		f_nom<-function(nom){
			which(x[[1]]@name %in% nom)			
		}
		f_nom<-Vectorize(f_nom)
		row.names(p.val.all)<-f_nom(p.val.all$ID)
		}
nb.ret<-dim(p.val.all)[1]

if(tot.number>=1 && nb.ret>tot.number){
	nb.ret<-tot.number
}

if(tot.number<1 && tot.number>0){
	nb.ret<-round(nb.ret*tot.number)
	
}

p.val.all<-p.val.all[1:nb.ret,]

		
		K1<-M_mic[[contrast[[2]][1]]][p.val.all$ID,]
		K2<-M_mic[[contrast[[2]][2]]][p.val.all$ID,]
	}
	
		
		if(!is.null(f.asso) && cont==TRUE){
		K1<-apply(K1,1,f.asso)
		}
		if(!is.null(f.asso) && cont==FALSE){
		for(i in 1:Time){
    K1[i,]<-apply(K1[seq(i,ncol(K1),by=Time),],1,f.asso)
		}
		}
		 MM1<-K2 -K1
		 
		 if(contrast[[1]]=="patterns"){
		 
     f_gr<-function(x){ min(which(x==1))}
		 
		 group=apply(tableT[1:nb.ret,1:Time],1,f_gr)
		  
      
      } else{
          group=rep(0,nb.ret)
      }

		M<-new("micro_array",microarray=MM1,name=row.names(MM1),time=M[[contrast[[2]][2]]]@time,subject=Subj,group=group,start_time=group)
		
			
			return(M)

	}

)


#' @rdname geneSelection
#' @exportMethod genePeakSelection
setMethod(f="genePeakSelection", 
	signature=c("micro_array","numeric"),
	definition=function(x,peak,y=NULL,data_log=TRUE,durPeak=c(1,1),abs_val=TRUE,alpha_diff=0.05){
			  M1<-x
			  M2<-y
			Select<-geneSelection(M1,tot.number=-1,M2,data_log=data_log,peak=peak)
			
			if(abs_val==FALSE){
			M<-Select@microarray
			sel<-rep(0,length(M[,1]))
			comp<-M[,(1:M1@subject-1)*length(M1@time)+peak]
			test1<-function(y){
				if(wilcox.test(y[1:M1@subject],y[(M1@subject+1):(2*M1@subject)], alternative="less")$p.value<alpha_diff){
					return(1)
				}
				else{
					return(0)
					}
				}
				uu<-0
			if((peak-durPeak[1])>0){
			for(i in 1:(peak-durPeak[1])){
				comp1<-M[,(1:M1@subject-1)*length(M1@time)+i]
				comp2<-cbind(comp1,comp)
				sel<-sel+apply(comp2,1,test1)
				uu<-uu+1				
			}
			}
		
			if((peak+durPeak[2])<=length(M1@time)){
			for(i in (peak+durPeak[2]):length(M1@time)){
				comp1<-M[,(1:M1@subject-1)*length(M1@time)+i]
				comp2<-cbind(comp1,comp)
				sel<-sel+apply(comp2,1,test1)	
				uu<-uu+1			
			}
			}
			
			
			
			#print(uu)
			N1<-Select@name[sel==uu]
			
			M<--Select@microarray
			sel<-rep(0,length(M[,1]))
			comp<-M[,(1:M1@subject-1)*length(M1@time)+peak]
			test1<-function(y){
				if(wilcox.test(y[1:M1@subject],y[(M1@subject+1):(2*M1@subject)], alternative="less")$p.value<alpha_diff){
					return(1)
				}
				else{
					return(0)
					}
				}
				uu<-0
			if((peak-durPeak[1])>0){
			for(i in 1:(peak-durPeak[1])){
				comp1<-M[,(1:M1@subject-1)*length(M1@time)+i]
				comp2<-cbind(comp1,comp)
				sel<-sel+apply(comp2,1,test1)
				uu<-uu+1				
			}
			}
		
			if((peak+durPeak[2])<=length(M1@time)){
			for(i in (peak+durPeak[2]):length(M1@time)){
				comp1<-M[,(1:M1@subject-1)*length(M1@time)+i]
				comp2<-cbind(comp1,comp)
				sel<-sel+apply(comp2,1,test1)	
				uu<-uu+1			
			}
			}
			
			N11<-Select@name[sel==uu]
			#Select2<-geneSelection(M1,M2,tot.number=-1,data_log=data_log,wanted.patterns=t(as.matrix(c(0,1,0,0,1000))))
			#N2<-Select2@name
			
			N<-c(N1,N11)
			Mi<-new("micro_array",microarray=Select@microarray[which(Select@name %in% N ),],name=N,time=M1@time,subject=M1@subject,start_time=Select@start_time[which(Select@name %in% N)],group=rep(peak,length(N)))			
			return(Mi)
			} 		
		else{
			M<-abs(Select@microarray)
			sel<-rep(0,length(M[,1]))
			comp<-M[,(1:M1@subject-1)*length(M1@time)+peak]
			test1<-function(y){
				if(wilcox.test(y[1:M1@subject],y[(M1@subject+1):(2*M1@subject)], alternative="less")$p.value<0.05){
					return(1)
				}
				else{
					return(0)
					}
				}
				uu<-0
			if((peak-durPeak[1])>0){
			for(i in 1:(peak-durPeak[1])){
				comp1<-M[,(1:M1@subject-1)*length(M1@time)+i]
				comp2<-cbind(comp1,comp)
				sel<-sel+apply(comp2,1,test1)
				uu<-uu+1				
			}
			}
		
			if((peak+durPeak[2])<=length(M1@time)){
			for(i in (peak+durPeak[2]):length(M1@time)){
				comp1<-M[,(1:M1@subject-1)*length(M1@time)+i]
				comp2<-cbind(comp1,comp)
				sel<-sel+apply(comp2,1,test1)	
				uu<-uu+1			
			}
			}
			
			N<-Select@name[sel==uu]
			#Select2<-geneSelection(M1,M2,tot.number=-1,data_log=data_log,wanted.patterns=t(as.matrix(c(0,1,0,0,1000))))
			#N2<-Select2@name
			
		
			Mi<-new("micro_array",microarray=Select@microarray[which(Select@name %in% N ),],name=N,time=M1@time,subject=M1@subject,group=rep(peak,length(N)),start_time=Select@start_time[which(Select@name %in% N)])	
			
			
		}
		
		
		
		}
		)
		

#' Makes the union between two micro_array objects.
#' 
#' Makes the union between two micro_array objects.
#' 
#' 
#' @name unionMicro-methods
#' @aliases unionMicro unionMicro-methods
#' unionMicro,micro_array,micro_array-method unionMicro,list,ANY-method
#' @docType methods
#' @section Methods: \describe{
#' 
#' \item{list("signature(M1 = \"micro_array\", M2 = \"micro_array\")")}{
#' Returns a micro_array object which is the union of M1 and M2.  }
#' 
#' \item{list("signature(M1 = \"list\", M2 = \"ANY\")")}{ Returns a micro_array
#' object which is the union of the elements of M1.  } }
#' 
#' @param M1 a micro-array or a list of micro-arrays
#' @param M2 a micro-array or nothing if M1 is a list of micro-arrays
#' @keywords methods
#' @examples
#' 
#' data(M)
#' #Create another microarray object with 100 genes
#' Mbis<-M
#' #Rename the 100 genes
#' Mbis@name<-paste(M@name,"bis")
#' rownames(Mbis@microarray) <- Mbis@name
#' #Union (merge without duplicated names) of the two microarrays. 
#' str(unionMicro(M,Mbis))
#' 
#' @exportMethod unionMicro
setMethod(f="unionMicro", 
	signature=c("micro_array","micro_array"),
	definition=function(M1,M2){
		nom1<-rownames(M1@microarray)
		nom2<-rownames(M2@microarray)
		corres<-cbind(c(M1@name,M2@name),c(nom1,nom2))
		corres<-unique(unique(corres,MARGIN=2))
		
		NOM<-unique(c(nom1,nom2))
		n<-length(NOM)
		m1<-M1@microarray[which(nom1 %in% NOM),]
		NOM2<-NOM[-which(nom1 %in% NOM)]
		m2<-M2@microarray[which(nom2 %in% NOM2),]		
		M<-rbind(m1,m2)
		
		gr1<-M1@group[which(nom1 %in% NOM)]
		gr2<-M2@group[which(nom2 %in% NOM2)]
		gr<-c(gr1,gr2)
		str1<-M1@start_time[which(nom1 %in% NOM)]
		str2<-M2@start_time[which(nom2 %in% NOM2)]
		str<-c(str1,str2)
		rep<-new("micro_array",microarray=M,name=corres[,1],time=M1@time,subject=M1@subject,group=gr,start_time=str)
		return(rep)
				}
		
		) 
 
 
 setMethod(f="unionMicro", 
	signature=c("list","ANY"),
	definition=function(M1,M2){
		
		rep<-unionMicro(M1[[1]],M1[[1]])
	if(length(M1)>1){	
    for(i in 2:length(M1)){
		         rep<-unionMicro(rep,M1[[i]])
		
		}
         }
    return(rep)
				}
		
		) 

#' @rdname plot-methods
setMethod("plot"
          ,c("micropredict")
          ,function(x
                    ,time=NULL
                    ,label_v=NULL
                    ,frame.color="white"
                    ,ini=NULL
                    ,label.hub=FALSE
                    ,edge.arrow.size=0.7
                    ,edge.thickness=1){
	net<-x@network
	micro<-x@microarray_changed
	nv<-x@nv
	micro_pred<-x@microarray_predict
	targets<-x@targets
	O<-net@network
  	Omega<-net@network
    O[abs(O)<nv]<-0
    O[abs(O)>=nv]<-1
    if(is.null(label_v)){label_v<-1:dim(O)[1]}
	
	G<-graph_from_adjacency_matrix(O,weighted=TRUE)
	
	couleur1<-colorRamp(c(grey(0.95),"blue"))
	couleur2<-colorRamp(c(grey(0.95),"red"))
	color<-rep(grey(0.0999),length(V(G)))
	
	if(is.null(ini)){
	P<-position(net,nv)
	}else{
		P<-ini
		}
		
	if(is.null(time)){
		K<-2:(length(micro@time))
		} 		else{K<-time}
		for(time in K){
		if(length(K)>1){
	dev.new()}
		if(time==length(micro@time)){time<-0}
		sup_pred<-1:dim(micro@microarray)[2]
	sup_pred<-sup_pred[sup_pred%%length(micro@time)==time]
	M<-apply(micro@microarray[,sup_pred]-micro_pred@microarray[,sup_pred],1,mean)
	  
	maxi<-round(max(abs(M[-targets])),2)
	if(max(abs(M))!=0){M<-M/max(abs(M[-targets]))}
	long<-1:length(M)
	long<-long[-targets]
	for(j in long){
		
		if(M[j]>0){
			color[j]<-rgb(couleur2(max(M[j],0.2)),alpha=255,maxColorValue=255)
					}
					else{
						color[j]<-rgb(couleur1(max(-M[j],0.2)),alpha=255,maxColorValue=255)

					}
					if(M[j]==0){
            color[j]<- rgb(couleur2(0),alpha=255,maxColorValue=255)
					}
		
	}
	if(!is.null(targets)){color[targets]<-"green"}
	plot(net,nv=nv,color.vertex=color,ini=P,label_v=label_v,frame.color=frame.color,label.hub=label.hub,
	edge.arrow.size=edge.arrow.size,edge.thickness=edge.thickness)
couleur3<-colorRamp(c("blue","grey","red"))
nb.col<-300
coll<-rgb(couleur3(1:nb.col/nb.col),maxColorValue=255)

rect(seq(-1,-0.5,length.out=nb.col)[1:(nb.col-1)],-1,seq(-1,-0.5,length.out=nb.col)[2:nb.col],-0.95,border="transparent",col=coll)
text(-1,-1.05,-maxi)
text(-0.51,-1.05,maxi)
text(-0.751,-1.05,"0")	
if(time==0){time<-length(micro@time)}
 text(-0.8,1.1,paste("Time point prediction =",time, sep=" "))
		}
		
}
)

#' Analysing the network
#' 
#' Calculates some indicators for each node in the network.
#' 
#' 
#' @aliases analyze_network analyze_network-methods
#' analyze_network,network-method
#' @param Omega a network object
#' @param nv the level of cutoff at which the analysis should be done
#' @param label_v (optionnal) the name of the genes
#' @return A matrix containing, for each node, its betweenness,its degree, its
#' output, its closeness.
#' @author Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
#' @references Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
#' Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
#' simulate the diffusion of a signal through a temporal gene network.
#' \emph{Bioinformatics}, btt705.
#' 
#' Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
#' Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
#' circuitry of a cancer cell with predicted intervention in chronic
#' lymphocytic leukemia. \emph{Proceedings of the National Academy of
#' Sciences}, 110(2), 459-464.
#' @keywords methods
#' @examples
#' 
#' 	data(network)
#' 	analyze_network(network,nv=0)
#' 
#' @exportMethod analyze_network
setMethod("analyze_network","network",function(Omega,nv,label_v=NULL){
  if(is.null(label_v)){label_v<-1:dim(Omega@network)[1]}
  O<-Omega@network
  Omega<-Omega@network
  O[abs(O)<=nv]<-0
  G<-graph_from_adjacency_matrix(O,weighted=TRUE)
  
  as_edgelist(G)->Q	
  
  weight<-rep(0,dim(Q)[1])
  for(i in 1:dim(Q)[1]){weight[i]<-Omega[Q[i,1],Q[i,2]]}
  R<-cbind(Q,abs(weight))
  G<-as.tnet(R,type="weighted one-mode tnet")
  C<-data.frame(label_v,betweenness_w(G)[,2], degree_w(G)[,2:3],closeness_w(G,gconly=FALSE)[,2])
  colnames(C)<-c("node","betweenness","degree","output","closeness")
  return(C)
}
)

#' @rdname print-methods
setMethod("print","network",function(x,...){
  cat(paste("This is a S4 class with : \n - (@network) a matrix of dimension ",dim(x@network)[1],"*",dim(x@network)[2]," .... [the network] \n - (@name) a vector of length ",length(x@name)," .... [gene names] \n","- (@F) a array of dimension ",dim(x@F)[1],"*",dim(x@F)[2],"*",dim(x@F)[3]," .... [F matrices] \n","- (@convF) a matrix of dimension ",dim(x@convF)[1],"*",dim(x@convF)[2]," .... [convergence (L1 norm) of array F] \n","- (@convO)a vector of length ",length(x@convO)," .... [convergence (L1 norm) of matrix Omega]\n","- (@time_pt) an vector of length",length(x@time_pt),"  .... [time points]")) 
})


#' See the evolution of the network with change of cutoff
#' 
#' See the evolution of the network with change of cutoff.  This function may
#' be usefull to see if the global topology is changed while increasing the
#' cutoff.
#' 
#' @aliases evolution evolution-methods evolution,network-method
#' @param net a network object
#' @param list_nv a vector of cutoff at which the network should be shown
#' @param gr a vector giving the group of each gene 
#' @param color.vertex a vector giving the color of each node
#' @param fix logical, should the position of the node in the network be calculated once at the beginning ? Defaults to TRUE.
#' @param gif logical, TRUE
#' @param taille vector giving the size of the plot. Default to c(2000,1000)
#' @param label_v (optional) the name of the genes
#' @param legend.position (optional) the position of the legend, defaults to "topleft"
#' @param frame.color (optional) the color of the frame, defaults to "black"
#' @param label.hub (optional) boolean, defaults to FALSE
#' 
#' @return A HTML page with the evolution of the network.
#' @author Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
#' @references Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
#' Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
#' simulate the diffusion of a signal through a temporal gene network.
#' \emph{Bioinformatics}, btt705.
#' 
#' Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
#' Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
#' circuitry of a cancer cell with predicted intervention in chronic
#' lymphocytic leukemia. \emph{Proceedings of the National Academy of
#' Sciences}, 110(2), 459-464.
#' @keywords methods
#' @examples
#' 
#' \donttest{
#' 	data(network)
#' 	sequence<-seq(0,0.2,length.out=20)
#' 	#evolution(network, sequence) # writes a saveHTML() viewer to getwd()
#' }
#' 
#' @exportMethod evolution
setMethod("evolution","network",function(net
                                         ,list_nv
                                         ,gr=NULL
                                         ,color.vertex=NULL
                                         ,fix=TRUE
                                         ,gif=TRUE
                                         ,taille=c(2000,1000)
                                         ,label_v=1:dim(net@network)[1]
                                         ,legend.position="topleft"
                                         ,frame.color="black"
                                         ,label.hub=FALSE
                                         #,edge.arrow.size=0.6*(1+size.ed)
                                         #,edge.thickness=1
)
{
  Omega<-net
  
  if(length(list_nv)==1){
    list_nv=seq(0,max(net@network)-0.05,length.out=list_nv)	
  }
  
  if(gif==TRUE){
    require(animation)
    animation::ani.options(ani.height=taille[2],ani.width=taille[1],outdir = getwd())
    animation::saveHTML({
      
      POS<-position(net,nv=list_nv[1])
      
      for(i in list_nv){
        
        if(fix==TRUE){
          plot(Omega
               ,nv=i
               ,gr=gr
               ,ini=POS
               ,color.vertex=color.vertex
               ,label_v=label_v
               ,legend.position=legend.position
               ,frame.color=frame.color
               ,label.hub=label.hub
               #,edge.arrow.size=edge.arrow.size
               #,edge.thickness=edge.thickness
          )
        }
        else{
          plot(Omega
               ,nv=i
               ,gr=gr
               ,color.vertex=color.vertex
               ,label_v=label_v
               ,legend.position=legend.position
               ,frame.color=frame.color
               ,label.hub=label.hub
               #,edge.arrow.size=edge.arrow.size
               #,edge.thickness=edge.thickness
          )
          
        }
        text(-1,1,round(i,3))
      }
    })
  }
  
  else{
    par(ask=TRUE)
    POS<-position(net,nv=list_nv[1])
    if(is.null(gr)){gr<-rep(1,dim(Omega@network)[1])} 
    for(i in list_nv){
      plot(Omega
           ,nv=i
           ,gr=gr
           ,ini=POS
           ,color.vertex=color.vertex
           ,label_v=label_v
           ,legend.position=legend.position
           ,frame.color=frame.color
           ,label.hub=label.hub
           #,edge.arrow.size=edge.arrow.size
           #,edge.thickness=edge.thickness
      )
    }
    par(ask=FALSE)	
  }
}
)

#' Returns the position of edges in the network
#' 
#' Returns the position of edges in the network
#' 
#' 
#' @name position-methods
#' @aliases position position-methods position,network-method
#' @docType methods
#' @section Methods: \describe{
#' 
#' \item{list("signature(net = \"network\")")}{ Returns a matrix with the
#' position of the node. This matrix can then be used as an argument in the
#' plot function. } }
#' @param net a network object
#' @param nv the level of cutoff at which the analysis should be done
#' @keywords methods
#' @examples
#' 
#' data(Net)
#' position(Net)
#' 
#' @exportMethod position
setMethod("position","network",function(net,nv=0){
  O<-net@network
  Omega<-net@network
  O[abs(O)<=nv]<-0
  O[abs(O)>nv]<-1
  
  nom<-1:dim(O)[1]
  enle<-which(apply(O,1,sum)+apply(O,2,sum)==0)
  if(length(enle)!=0){
    nom<-nom[-enle]
    O<-O[-enle,-enle]
  }
  
  G<-graph_from_adjacency_matrix(O,weighted=TRUE)
  as_edgelist(G)->Q	
  
  Q[,1]<-nom[Q[,1]]
  Q[,2]<-nom[Q[,2]]
  
  # dev.new(width=150,heigth=300,xlim=c(min(L[,1]),max(L[,1])))
  #par(mar=c(0,0,0,0), oma=c(0,0,0,0),mai=c(0,0,0,0))
  L<-layout_with_fr(G,minx=rep(0,vcount(G)),maxx=rep(150,vcount(G)),miny=rep(0,vcount(G)),maxy=rep(300,vcount(G)))
  return(cbind(nom,L))
}
)

#######################################
#######################################
#######################################

#' @rdname plot-methods
setMethod("plot"
          ,"network"
          ,function(x
                    ,y
                    ,choice="network"
                    ,nv=0
                    ,gr=NULL
                    ,ini=NULL
                    ,color.vertex=NULL
                    ,video=TRUE
                    ,weight.node=NULL
                    ,ani=FALSE
                    ,taille=c(2000,1000)
                    ,label_v=1:dim(x@network)[1]
                    ,horiz=TRUE
                    ,legend.position="topleft"
                    ,frame.color="black"
                    ,label.hub=FALSE
                    #,edge.arrow.size=0.6*(1+size.ed)
                    #,edge.thickness=1
                    ,...
          )
{
            
            if(choice=="F"){
              F<-x@F
              
              par(mar=c(0,0,0,0), oma=c(0,0,0,0),mai=c(0,0,0,0))
              
              couleur<-rainbow(dim(F)[1])
              ymax<-max(F)
              nF<-dim(F)[3]
              par(mfcol=c(dim(F)[1],dim(F)[1]))
              ordre<-0
              u<-dim(F)[1]-1
              t<-0
              k<-1
              for(i in 1:dim(F)[1]){
                for(w in 1:k){
                  ordre<-ordre+1	
                  t<-t+1                                                                            
                  KK<-ordre%%(dim(F)[1]+1)
                  if(KK==0){
                    KK<-2
                  }
                  if(ordre<=dim(F)[1]){
                    par(mai=c(0,0.3,0,0))
                    plot(F[,1,t]
                         ,xlab=""
                         ,ylab=""
                         ,type="h"
                         ,lwd=3
                         ,xaxt="n"
                         ,ylim=c(0,ymax)
                         ,col=couleur[KK])
                    #axis(1,at=1:dim(F)[1],1:3)
                    
                    GG<-ordre%%dim(F)[1]
                    if(GG==0){
                      GG<-dim(F)[1]
                    }
                    GG2<-(ordre/dim(F)[1])
                    if(floor(GG2)==GG2){
                      GG2<-GG2-1
                    }
                    GG2<-floor(GG2)
                    text((dim(F)[1]+dim(F)[1]-1)/2,ymax*0.9,paste("F",GG,GG2+2,sep=""),cex=2)
                    
                    par(mai=c(0,0,0,0))
                  }
                  else{
                    
                    plot(F[,1,t]
                         ,xlab=""
                         ,ylab=""
                         ,type="h"
                         ,lwd=3
                         ,xaxt="n"
                         ,ylim=c(0,ymax)
                         ,col=couleur[KK]
                         ,yaxt="n" )
                    
                    GG<-ordre%%dim(F)[1]
                    if(GG==0){GG<-dim(F)[1]}
                    GG2<-(ordre/dim(F)[1])
                    if(floor(GG2)==GG2){GG2<-GG2-1}
                    GG2<-floor(GG2)
                    text((dim(F)[1]+dim(F)[1]-1)/2,ymax*0.9,paste("F",GG,GG2+2,sep=""),cex=2)
                    
                    #axis(1,at=1:dim(F)[1],1:3)
                  }
                  
                  
                }
                k<-k+1
                if(u !=0){
                  for(j in 1:u){
                    ordre<-ordre+1
                    if(ordre<=dim(F)[1]){
                      par(mai=c(0,0.3,0,0))
                      plot(c(0,0,0),xlab="",ylab="",xaxt="n",ylim=c(0,ymax),xaxt="n")
                      
                      GG<-ordre%%dim(F)[1]
                      if(GG==0){GG<-dim(F)[1]}
                      GG2<-(ordre/dim(F)[1])
                      if(floor(GG2)==GG2){GG2<-GG2-1}
                      GG2<-floor(GG2)
                      text((dim(F)[1]+dim(F)[1]-1)/2,ymax*0.9,paste("F",GG,GG2+2,sep=""),cex=2)
                      
                      par(mai=c(0,0,0,0))
                    }
                    else{
                      plot(c(0,0,0),xlab="",ylab="",xaxt="n",ylim=c(0,ymax),yaxt="n")
                      
                      GG<-ordre%%dim(F)[1]
                      if(GG==0){GG<-dim(F)[1]}
                      GG2<-(ordre/dim(F)[1])
                      if(floor(GG2)==GG2){GG2<-GG2-1}
                      GG2<-floor(GG2)
                      text((dim(F)[1]+dim(F)[1]-1)/2,ymax*0.9,paste("F",GG,GG2+2,sep=""),cex=2)
                      
                    }
                  }
                  u<-u-1
                }
              }
            }
            if(choice=="network"){
              
              couleur<-0	
              O<-x@network
              Omega<-x@network
              O[abs(O)<=nv]<-0
              O[abs(O)>nv]<-1
              
              if(is.null(gr)){gr<-rep(1,dim(O)[1])}
              if(is.null(color.vertex)){
                color.vertex<-rainbow(length(unique(gr)))
                couleur<-1
              }
              
              nom<-1:dim(O)[1]
              enle<-which(apply(O,1,sum)+apply(O,2,sum)==0)
              if(length(enle)!=0){
                nom<-nom[-enle]
                
                O<-O[-enle,-enle]
                
                if(!is.null(weight.node)){weight.node<-weight.node[-enle]}
              }
              
              if(!is.null(ini)){
                ini<-ini[which(ini[,1] %in% nom),]
              }
              nom2<-nom
              nom2<-label_v[nom]
              if(!is.null(weight.node)){
                size<-weight.node
              }else
              {
                size<-apply(O,1,sum)
              }
              
              size<-size/max(size)*10
              size[size<2]<-2 
              
              if(label.hub==TRUE){nom2[which(size<3)]<-NA}
              
              
              G<-graph_from_adjacency_matrix(O,weighted=TRUE)
              
              
              as_edgelist(G)->Q	
              
              Q[,1]<-nom[Q[,1]]
              Q[,2]<-nom[Q[,2]]
              size.ed<-rep(0,dim(Q)[1])
              
              for(i in 1:dim(Q)[1]){
                size.ed[i]<-abs(Omega[Q[i,1],Q[i,2]])
              }
              #fpl<-function(x){3*exp(8*x)/(14+exp(8*x))}
              size.ed<-size.ed/max(size.ed)
              #size.ed<-fpl(size.ed)
              #size.ed<-size.ed
              
              
              
              
              if(ani==TRUE){
                if(is.null(gr)){stop("Need groups")}
                T<-length(x@time_pt)
                require(animation)
                animation::ani.options(ani.height=taille[2],ani.width=taille[1],outdir = getwd())
                
                if(is.null(ini)){
                  ini<-position(x,nv)[]
                }
                
                
                #ini<-ini[which(ini[,1] %in% nom),]
                ini<-ini[,2:3]
                gr2<-gr[nom]
                
                animation::saveHTML({
                  for(i in c(0.999,0.99,0.98,0.97,0.96,0.95)){											
                    plot(G
                         ,layout=ini
                         ,vertex.size=size
                         ,edge.arrow.size=0.6*(1+size.ed)
                         ,edge.width=size.ed*5
                         ,edge.arrow.width=0.5*(1+size.ed)
                         #,edge.width=size.ed*5*edge.thickness
                         #,edge.arrow.size=edge.arrow.size*size.ed*edge.thickness
                         #,edge.arrow.width=edge.arrow.size*size.ed*edge.thickness
                         ,vertex.color=grey(i)
                         ,vertex.label.cex=1
                         ,edge.color=grey(i)
                         ,asp=0
                         ,vertex.label=nom2
                         ,vertex.frame.color=frame.color
                    )
                  }
                  for(i in 1:(T)){
                    
                    Ver.col<-rep(grey(0.95),length(nom))	
                    Edge.col<-rep(grey(0.95),dim(Q)[1])
                    mms<-20
                    PP<-1:mms
                    for(p in PP){	
                      coul<-col2rgb(color.vertex[i])
                      indi<-which(gr2==i)
                      Ver.col[indi]<-rgb(coul[1],coul[2],coul[3],alpha=(p-1)*255/(mms), maxColorValue = 255)
                      
                      
                      for(k in 1:i){
                        coul<-col2rgb(color.vertex[k])	
                        indi3<-which(gr[Q[,1]]==k & gr[Q[,2]]==i )
                        
                        Edge.col[indi3]<-rgb(coul[1],coul[2],coul[3],alpha=255-(p-1)*255/(mms), maxColorValue = 255)
                      }
                      
                      if(i!=T){
                        for(k in 1:i){
                          for(k2 in min((i+1),T):T){
                            coul<-col2rgb(color.vertex[k])	
                            indi3<-which(gr[Q[,1]]==k & gr[Q[,2]]==k2 )
                            
                            
                            
                            
                            al<-round(mms/(k2-k))
                            ald<-min(1+al*(i-k),mms)
                            if(k2==(i+1)){
                              alf<-mms
                            }
                            else{
                              alf<-min(al*(i-k+1),mms)}
                            alseq<-seq(25,255,length.out=mms)
                            
                            alphaseq2<-seq(alseq[ald],alseq[alf],length.out=mms)
                            
                            Edge.col[indi3]<-rgb(coul[1],coul[2],coul[3],alpha=alphaseq2[p], maxColorValue = 255)
                          }}}
                      
                      plot(G
                           ,layout=ini
                           ,vertex.size=size
                           #,vertex.size=size,edge.width=size.ed*5*edge.thickness
                           #,edge.arrow.size=edge.arrow.size*size.ed*edge.thickness
                           #,edge.arrow.width=edge.arrow.size*size.ed*edge.thickness
                           ,edge.arrow.size=0.6*(1+size.ed)
                           ,edge.width=size.ed*5
                           ,edge.arrow.size=0.5*(1+size.ed)
                           ,vertex.color=Ver.col
                           ,vertex.label.cex=1
                           ,edge.color=Edge.col
                           ,asp=0
                           ,vertex.label=nom2
                           ,vertex.frame.color=frame.color
                      )
                      if(length(unique(gr[nom])) >1){
                        legend(legend.position
                               ,horiz=horiz
                               ,pch=21
                               ,pt.bg=color.vertex[unique(gr[nom])[order(unique(gr[nom]))]]
                               ,col=frame.color
                               ,legend=paste("Cluster",unique(gr[nom])[order(unique(gr[nom]))])
                        )
                      } 
                    }
                    
                    
                    
                  }
                  
                  
                })
                
                
              }	
              else{
                if(length(unique(gr[nom])) >1){trr<-color.vertex[gr[Q[,2]]]}
                else{trr<-"grey"} 
                
                if(is.null(ini)){
                  L<-position(x,nv)[,2:3]
                  #maxL<-c(max(abs(L[,1])),max(abs(L[,2])))
                  #matMaxL<-matrix(rep(maxL,nrow(L)),ncol=2,byrow=T)
                  #L<-L/matMaxL
                  if(couleur==0){
                    
                    plot(G
                         ,layout=L
                         ,vertex.size=size
                         #,edge.arrow.size=edge.arrow.size*size.ed*edge.thickness
                         #,edge.width=size.ed*5*edge.thickness
                         #,edge.arrow.width=edge.arrow.size*size.ed*edge.thickness
                         ,edge.arrow.size=0.6*(1+size.ed)
                         ,edge.width=size.ed*5
                         ,edge.arrow.width=0.5*(1+size.ed)
                         ,vertex.color=color.vertex[nom]
                         ,vertex.label.cex=1
                         ,edge.color=trr
                         ,asp=0
                         ,vertex.label=nom2
                         ,vertex.frame.color=frame.color
                    )
                  }  
                  else{  
                    plot(G
                         ,layout=L
                         ,vertex.size=size
                         #,edge.width=size.ed*5*edge.thickness
                         #,edge.arrow.size=edge.arrow.size*size.ed*edge.thickness
                         #,edge.arrow.width=edge.arrow.size*size.ed*edge.thickness
                         ,edge.arrow.size=0.6*(1+size.ed)
                         ,edge.width=size.ed*5
                         ,edge.arrow.width=0.5*(1+size.ed)
                         ,vertex.color=color.vertex[gr[nom]]
                         ,vertex.label.cex=1
                         ,edge.color=trr
                         ,asp=0
                         ,vertex.label=nom2
                         ,vertex.frame.color=frame.color
                    )
                  }
                  
                  if(length(unique(gr[nom])) >1){
                    legend(legend.position
                           ,horiz=horiz
                           ,pch=21
                           ,pt.bg=color.vertex[unique(gr[nom])[order(unique(gr[nom]))]]
                           ,col=frame.color
                           ,legend=paste("Cluster",unique(gr[nom])[order(unique(gr[nom]))])
                    ) 
                  }
                }
                else{
                  L<-ini[,2:3]
                  if(couleur==0){
                    plot(G
                         ,layout=L
                         ,vertex.size=size
                         #,edge.arrow.size=edge.arrow.size*size.ed*edge.thickness
                         #,edge.width=size.ed*5*edge.thickness
                         #,edge.arrow.width=edge.arrow.size*size.ed*edge.thickness
                         ,edge.arrow.size=0.6*(1+size.ed)
                         ,edge.width=size.ed*5
                         ,edge.arrow.width=0.5*(1+size.ed)
                         ,vertex.color=color.vertex[nom]
                         ,vertex.label.cex=1
                         ,edge.color=trr
                         ,asp=0
                         ,vertex.label=nom2
                         ,vertex.frame.color=frame.color
                    )
                  }  
                  else{  
                    plot(G
                         ,layout=L
                         ,vertex.size=size
                         #,edge.width=size.ed*5*edge.thickness
                         #,edge.arrow.size=edge.arrow.size*size.ed*edge.thickness
                         #,edge.arrow.width=edge.arrow.size*size.ed*edge.thickness
                         ,edge.arrow.size=0.6*(1+size.ed)
                         ,edge.width=size.ed*5
                         ,edge.arrow.width=0.5*(1+size.ed)
                         ,vertex.color=color.vertex[gr[nom]]
                         ,vertex.label.cex=1
                         ,edge.color=trr
                         ,asp=0
                         ,vertex.label=nom2
                         ,vertex.frame.color=frame.color
                    )
                  }
                  if(length(unique(gr[nom])) >1){
                    legend(legend.position
                           ,horiz=horiz
                           ,pch=21
                           ,pt.bg=color.vertex[unique(gr[nom])[order(unique(gr[nom]))]]
                           ,col=frame.color
                           ,legend=paste("Cluster",unique(gr[nom])[order(unique(gr[nom]))])
                    ) 
                  }
                  L<-ini
                }
              }
            }	
          }
)


#' Find the neighborhood of a set of nodes.
#' 
#' Find the neighborhood of a set of nodes.
#' 
#' 
#' @aliases geneNeighborhood geneNeighborhood-methods
#' geneNeighborhood,network-method
#' @param net a network object
#' @param targets a vector containing the set of nodes
#' @param nv the level of cutoff. Defaut to 0.
#' @param order of the neighborhood. Defaut to `length(net@time_pt)-1`.
#' @param label_v vector defining the vertex labels.
#' @param ini using the ``position'' function, you can
#' fix the position of the nodes.
#' @param frame.color color of the frames.
#' @param label.hub logical ; if TRUE only the hubs are labeled.
#' @param graph plot graph of the network. Defaults to `TRUE`.
#' @param names return names of the neighbors. Defaults to `FALSE`.
#' 
#' @return The neighborhood of the targeted genes.
#' @author Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
#' @references Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
#' Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
#' simulate the diffusion of a signal through a temporal gene network.
#' \emph{Bioinformatics}, btt705.
#' 
#' Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
#' Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
#' circuitry of a cancer cell with predicted intervention in chronic
#' lymphocytic leukemia. \emph{Proceedings of the National Academy of
#' Sciences}, 110(2), 459-464.
#' @keywords methods
#' @examples
#' 
#' data(Selection)
#' data(network)
#' #A nv value can chosen using the cutoff function
#' nv=.11 
#' EGR1<-which(match(Selection@name,"EGR1")==1)
#' P<-position(network,nv=nv)
#' 
#' geneNeighborhood(network,targets=EGR1,nv=nv,ini=P,
#' label_v=network@name)
#' 
#' @exportMethod geneNeighborhood
setMethod("geneNeighborhood","network"
          ,function(net
                    ,targets
                    ,nv=0
                    ,order=length(net@time_pt)-1
                    ,label_v=NULL
                    ,ini=NULL
                    ,frame.color="white"
                    ,label.hub=FALSE
                    #,edge.arrow.size=0.7
                    #,edge.thickness=1
                    ,graph=TRUE
                    ,names=FALSE
          ){
            O<-net@network
            Omega<-net@network
            O[abs(O)<=nv]<-0
            O[abs(O)>nv]<-1
            G<-graph_from_adjacency_matrix(O,weighted=TRUE)
            couleur<-colorRamp(c("red","blue"))
            color<-rep(grey(0.92),length(V(G)))
            if(is.null(label_v)){label_v<-1:dim(O)[1]}
            K<-vector("list",order)
            for(j in order:1){
              
              #if(.Platform$OS.type=="unix"){
              #N<-neighborhood(G, order=j, nodes=targets-1, mode=c( "out"))
              #}else{
              N<-neighborhood(G, order=j, nodes=targets, mode=c( "out"))
              #}
              K[[j]]<-N
              gg<-length(targets)
              
              for(i in 1:gg){
                #if(.Platform$OS.type=="unix"){
                #color[N[[i]]+1]<-rgb(couleur((j-1)*1/(order-1)),alpha=255,maxColorValue=255)
                #}else{
                color[N[[i]]]<-rgb(couleur((j-1)*1/(order-1)),alpha=255,maxColorValue=255)
                
                #}
                
              }
            }
            if(graph==TRUE){
              color[targets]<-"green"
              plot(net
                   ,nv=nv
                   ,ini=ini
                   ,color.vertex=color
                   ,frame.color=frame.color
                   ,label.hub=label.hub
                   ,label_v=label_v
              )
              legend("topright"
                     ,legend=paste("order",1:order)
                     ,pch=16
                     ,col=rgb(couleur((1:order-1)*1/(order-1)),maxColorValue=255)
              )
            }
            ind1<-K[[1]][[1]]
            neighbors<-list(net@name[ind1])
            for(ord in 2:order){
              neighbors[[ord]]<-list()
              for(nod in 1:gg){
                neighbPrec<-K[[ord-1]][[nod]]
                neighbCur<-K[[ord]][[nod]]
                ind<-neighbCur[!(neighbCur %in% neighbPrec)]
                neighbors[[ord]][[nod]]<-net@name[ind]            
              }
            }
            if(names){
              return(neighbors)
            }
            else{
              return(K)
            }
          }
)


#' Choose the best cutoff
#' 
#' Allows estimating the best cutoff, in function of the scale-freeness of the
#' network.  For a sequence of cutoff, the corresponding p-value is then
#' calculated.
#' 
#' 
#' @aliases cutoff cutoff-methods cutoff,network-method
#' @param Omega a network object
#' @param sequence (optional) a vector corresponding to the sequence of cutoffs 
#' that will be tested.
#' @param x_min (optional) an integer ; only values over x_min are further 
#' retained for performing the test.
#' 
#' @return A list containing two objects : \item{p.value}{the p values
#' corresponding to the sequence of cutoff} \item{p.value.inter}{the smoothed p
#' value vector, using the loess function}
#' @author Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
#' @references Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
#' Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
#' simulate the diffusion of a signal through a temporal gene network.
#' \emph{Bioinformatics}, btt705.
#' 
#' Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
#' Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
#' circuitry of a cancer cell with predicted intervention in chronic
#' lymphocytic leukemia. \emph{Proceedings of the National Academy of
#' Sciences}, 110(2), 459-464.
#' @keywords methods
#' @examples
#' 
#' \donttest{
#' 	data(network)
#' 	cutoff(network)
#' 	#See vignette for more details
#' }
#' 
#' @exportMethod cutoff
setMethod("cutoff","network",function(Omega,sequence=NULL,x_min=0){
  
  plfit<-function(x=VGAM::rpareto(1000,10,2.5)
                  ,method="limit"
                  ,value=c()
                  ,finite=FALSE
                  ,nowarn=FALSE
                  ,nosmall=FALSE
  ){
    #init method value to NULL	
    vec <- c() ; sampl <- c() ; limit <- c(); fixed <- c()
    ###########################################################################################
    #
    #  test and trap for bad input
    #
    switch(method
           ,range = vec <- value
           ,sample = sampl <- value
           ,limit = limit <- value
           ,fixed = fixed <- value
           ,argok <- 0
    )
    
    if(exists("argok")){stop("(plfit) Unrecognized method")}
    
    if( !is.null(vec) && (!is.vector(vec) || min(vec)<=1 || length(vec)<=1) ){
      print(paste("(plfit) Error: ''range'' argument must contain a vector > 1; using default."))
      vec <- c()
    }
    if( !is.null(sampl) && ( !(sampl==floor(sampl)) ||  length(sampl)>1 || sampl<2 ) ){
      print(paste("(plfit) Error: ''sample'' argument must be a positive integer > 2; using default."))
      sample <- c()
    }
    if( !is.null(limit) && (length(limit)>1 || limit<1) ){
      print(paste("(plfit) Error: ''limit'' argument must be a positive >=1; using default."))
      limit <- c()
    }
    if( !is.null(fixed) && (length(fixed)>1 || fixed<=0) ){
      print(paste("(plfit) Error: ''fixed'' argument must be a positive >0; using default."))
      fixed <- c() 
    }   
    
    #  select method (discrete or continuous) for fitting and test if x is a vector
    fdattype<-"unknow"
    if( is.vector(x,"numeric") ){ fdattype<-"real" }
    if( all(x==floor(x)) && is.vector(x) ){ fdattype<-"integer" }
    if( all(x==floor(x)) && min(x) > 1000 && length(x) > 100 ){ fdattype <- "real" }
    if( fdattype=="unknow" ){ stop("(plfit) Error: x must contain only reals or only integers.") }
    
    #
    #  end test and trap for bad input
    #
    ###########################################################################################
    
    ###########################################################################################
    #   CONTINUOUS CASE
    #  estimate xmin and alpha in the continuous case
    #
    if( fdattype=="real" ){
      xmins <- sort(unique(x))
      xmins <- xmins[-length(xmins)]
      
      if( !is.null(limit) ){
        xmins <- xmins[xmins>=limit]
      } 
      if( !is.null(fixed) ){
        xmins <- fixed
      }
      if( !is.null(sampl) ){ 
        xmins <- xmins[unique(round(seq(1,length(xmins),length.out=sampl)))]
      }
      #Vecteur pour stocker les valeurs de D
      dat <- rep(0,length(xmins))
      #Echantillon trie
      z <- sort(x)
      
      #Boucle sur toutes les valeurs de xmin pour lesquelles on calcule D
      for( xm in 1:length(xmins) ){
        #xmin courant
        xmin <- xmins[xm]
        #Exces
        z    <- z[z>=xmin]
        n    <- length(z)
        # estimate alpha-1 using direct MLE
        a    <- n/sum(log(z/xmin))
        # truncate search if nosmall is selected
        if( nosmall ){
          if((a-1)/sqrt(n) > 0.1){
            dat <- dat[1:(xm-1)]
            print(paste("(plfit) Warning : xmin search truncated beyond",xmins[xm-1]))
            break
          }
        }
        # compute KS statistic
        cx   <- c(0:(n-1))/n
        cf   <- 1-(xmin/z)^a
        dat[xm] <- max(abs(cf-cx))
      }
      #min de D
      D     <- min(dat)
      #argmin de D
      xmin  <- xmins[min(which(dat<=D))]
      #exces et estimations finaux
      z     <- x[x>=xmin]
      n     <- length(z)
      alpha <- 1 + n/sum(log(z/xmin))
      
      if( finite ){
        # finite-size correction
        alpha <- alpha*(n-1)/n+1/n 
      }
    }
    #
    #  end continuous case
    #
    ###########################################################################################
    
    ###########################################################################################
    #  DISCRETE CASE
    #  estimate xmin and alpha in the discrete case
    #
    if( fdattype=="integer" ){
      
      if( is.null(vec) ){ vec<-seq(1.5,3.5,.01) } # covers range of most practical scaling parameters
      zvec <- VGAM::zeta(vec)
      #On enleve la plus grande valeur
      xmins <- sort(unique(x))
      xmins <- xmins[-length(xmins)]
      
      if( !is.null(limit) ){
        limit <- round(limit)
        xmins <- xmins[xmins>=limit]
      } 
      
      if( !is.null(fixed) ){
        xmins <- fixed
      }
      
      if( !is.null(sampl) ){ 
        xmins <- xmins[unique(round(seq(1,length(xmins),length.out=sampl)))]
      }
      
      if( is.null(xmins) || length(xmins) < 2){
        stop("(plfit) error: x must contain at least two unique values.")
      }
      
      if(length(which(xmins==0) > 0)){
        stop("(plfit) error: x must not contain the value 0.")
      }
      
      xmax <- max(x)
      dat <- matrix(0,nrow=length(xmins),ncol=2)
      z <- x
      for( xm in 1:length(xmins) ){
        xmin <- xmins[xm]
        z    <- z[z>=xmin]
        n    <- length(z)
        # estimate alpha via direct maximization of likelihood function
        #  vectorized version of numerical calculation
        # matlab: zdiff = sum( repmat((1:xmin-1)',1,length(vec)).^-repmat(vec,xmin-1,1) ,1);
        if(xmin==1){
          zdiff <- rep(0,length(vec))
        }else{
          zdiff <- apply(rep(t(1:(xmin-1)),length(vec))^-t(kronecker(t(array(1,xmin-1)),vec)),2,sum)
        }
        # matlab: L = -vec.*sum(log(z)) - n.*log(zvec - zdiff);
        L <- -vec*sum(log(z)) - n*log(zvec - zdiff);
        I <- which.max(L)
        # compute KS statistic
        fit <- cumsum((((xmin:xmax)^-vec[I])) / (zvec[I] - sum((1:(xmin-1))^-vec[I])))
        cdi <- cumsum(hist(z,c(min(z)-1,(xmin+.5):xmax,max(z)+1),plot=FALSE)$counts/n)
        dat[xm,] <- c(max(abs( fit - cdi )),vec[I])
      }
      D     <- min(dat[,1])
      I     <- which.min(dat[,1])
      xmin  <- xmins[I]
      n     <- sum(x>=xmin)
      alpha <- dat[I,2]
      
      if( finite ){
        alpha <- alpha*(n-1)/n+1/n # finite-size correction
      }
      
    }
    #
    #  end discrete case
    #
    ###########################################################################################
    
    #  return xmin, alpha and D in a list
    return(list(xmin=xmin,alpha=alpha,D=D))
  }
  
  ###########################################################################################
  ###########################################################################################
  ###########################################################################################
  ###########################################################################################
  ###########################################################################################
  ###########################################################################################
  plpva<-function(x=VGAM::rpareto(1000,10,2.5),xmin=1,method="limit",value=c(),Bt=1000,quiet=FALSE,vec=seq(1.5,3.5,.01)){
    #init method value to NULL  
    sampl <- c() ; limit <- c()
    ###########################################################################################
    #
    #  test and trap for bad input
    #
    switch(method, 
           sample = sampl <- value,
           limit = limit <- value,
           argok <- 0)
    
    if(exists("argok")){stop("(plvar) Unrecognized method")}
    
    if( !is.null(vec) && (!is.vector(vec) || min(vec)<=1 || length(vec)<=1) ){
      print(paste("(plvar) Error: ''range'' argument must contain a vector > 1; using default."))
      vec <- c()
    }
    if( !is.null(sampl) && ( !(sampl==floor(sampl)) ||  length(sampl)>1 || sampl<2 ) ){
      print(paste("(plvar) Error: ''sample'' argument must be a positive integer > 2; using default."))
      sample <- c()
    }
    if( !is.null(limit) && (length(limit)>1 || limit<1) ){
      print(paste("(plvar) Error: ''limit'' argument must be a positive >=1; using default."))
      limit <- c()
    }
    if( !is.null(Bt) && (!is.vector(Bt) || Bt<=1 || length(Bt)>1) ){
      print(paste("(plvar) Error: ''Bt'' argument must be a positive value > 1; using default."))
      vec <- c()
    }
    
    #  select method (discrete or continuous) for fitting and test if x is a vector
    fdattype<-"unknown"
    if( is.vector(x,"numeric") ){ fdattype<-"real" }
    if( all(x==floor(x)) && is.vector(x) ){ fdattype<-"integer" }
    if( all(x==floor(x)) && min(x) > 1000 && length(x) > 100 ){ fdattype <- "real" }
    if( fdattype=="unknown" ){ stop("(plfit) Error: x must contain only reals or only integers.") }
    
    N   <- length(x)
    nof <- rep(0,Bt)
    
    if( !quiet ){
      print("Power-law Distribution, parameter error calculation")
      print("Warning: This can be a slow calculation; please be patient.")
      print(paste(" n =",N,"xmin =",xmin,"- reps =",Bt,fdattype))
    } 
    #
    #  end test and trap for bad input
    #
    ###########################################################################################
    
    ###########################################################################################
    #
    #  estimate xmin and alpha in the continuous case
    #
    if( fdattype=="real" ){
      
      # compute D for the empirical distribution
      z     <- x[x>=xmin];     nz   <- length(z)
      y     <- x[x<xmin];      ny   <- length(y)
      alpha <- 1 + nz/sum(log(z/xmin))
      cz    <- (0:(nz-1))/nz
      cf    <- 1-(xmin/sort(z))^(alpha-1)
      gof   <- max(abs(cz-cf))
      pz    <- nz/N
      
      # compute distribution of gofs from semi-parametric bootstrap
      # of entire data set with fit
      for(B in 1:length(nof)){
        # semi-parametric bootstrap of data
        n1 <- sum(runif(N)>pz)
        q1 <- y[sample(ny,n1,replace=TRUE)]
        n2 <- N-n1
        q2 <- xmin*(1-runif(n2))^(-1/(alpha-1))
        q  <- sort(c(q1,q2))
        
        # estimate xmin and alpha via GoF-method
        qmins <- sort(unique(q))
        qmins <- qmins[-length(qmins)]
        if(!is.null(limit)){
          qmins<-qmins[qmins<=limit] 
        } 
        if(!is.null(sampl)){ 
          qmins <- qmins[unique(round(seq(1,length(qmins),length.out=sampl)))]
        }
        dat <-rep(0,length(qmins))
        for(qm in 1:length(qmins)){
          qmin <- qmins[qm]
          zq   <- q[q>=qmin]
          nq   <- length(zq)
          a    <- nq/sum(log(zq/qmin))
          cq   <- (0:(nq-1))/nq
          cf   <- 1-(qmin/zq)^a
          dat[qm] <- max(abs(cq-cf))
        }
        if(!quiet){
          print(paste(B,sum(nof[1:B]>=gof)/B))
        }
        # store distribution of estimated gof values
        nof[B] <- min(dat)
      } 
    }
    
    ###########################################################################################
    #
    #  estimate xmin and alpha in the discrete case
    #
    if( fdattype=="integer" ){
      
      if( is.null(vec) ){ vec<-seq(1.5,3.5,.01) } # covers range of most practical scaling parameters
      zvec <- VGAM::zeta(vec)
      
      # compute D for the empirical distribution
      z     <- x[x>=xmin];     nz   <- length(z);    xmax <- max(z)
      y     <- x[x<xmin];      ny   <- length(y)
      
      if(xmin==1){
        zdiff <- rep(0,length(vec))
      }else{
        zdiff <- apply(rep(t(1:(xmin-1)),length(vec))^-t(kronecker(t(array(1,xmin-1)),vec)),2,sum)
      }
      # matlab: L = -vec.*sum(log(z)) - n.*log(zvec - zdiff);
      L <- -vec*sum(log(z)) - nz*log(zvec - zdiff);
      I <- which.max(L)
      alpha <- vec[I]
      
      fit <- cumsum((((xmin:xmax)^-alpha))
                    / (zvec[I]
                       - (if (xmin == 1) 0 else sum((1:(xmin-1))^-alpha))))
      
      hist = aggregate(z, list(z), length)
      cdi = rep(0, xmax - xmin + 1)
      cdi[hist$Group.1 - xmin + 1] = hist$x / nz
      cdi = cumsum(cdi)
      
      gof <- max(abs( fit - cdi ))
      pz  <- nz/N
      
      mmax <- 20*xmax
      pdf <- c(rep(0,xmin-1),(((xmin:mmax)^-alpha))
               / (zvec[I]
                  - (if (xmin == 1) 0 else sum((1:(xmin-1))^-alpha))))
      cdf <- cbind(1:(mmax+1),c(cumsum(pdf),1))
      
      # compute distribution of gofs from semi-parametric bootstrap
      # of entire data set with fit
      for(B in 1:Bt){
        # semi-parametric bootstrap of data
        n1 <- sum(runif(N)>pz)
        q1 <- y[sample(ny,n1,replace=TRUE)]
        n2 <- N-n1
        # simple discrete zeta generator
        r2 <- sort(runif(n2));  d <- 1
        q2 <- rep(0,n2);        k <- 1
        for(i in xmin:(mmax+1)){
          while((d <= length(r2)) && (r2[d]<=cdf[i,2])){d <- d+1}
          if (k <= d - 1)
            q2[k:(d-1)] <- i
          k <- d
          if( k > n2 ){break}
        } 
        q <-c(q1,q2)
        ########################################
        # estimate xmin and alpha via GoF-method
        qmins <- sort(unique(q))
        qmins <- qmins[-length(qmins)]
        if(!is.null(limit)){
          qmins <- qmins[qmins<=limit]
        }
        if(!is.null(sampl)){
          qmins <- qmins[sort(unique(round(seq(1,length(qmins),length.out=sampl))))]
        }        
        dat   <- rep(0,length(qmins))
        qmax  <- max(q) 
        zq    <- q
        
        if (length(qmins) > 0)
          for(qm in 1:length(qmins)){
            qmin <- qmins[qm]
            zq   <- zq[zq>=qmin]
            nq    <- length(zq)
            if(nq>1){
              # vectorized version of numerical calculation
              if(qmin==1){
                #WARNING
                #zdiff <- rep(1,length(vec))
                #correction added the 6/10/2009 (following Naoki Masuda for plfit) 
                zdiff <- rep(0,length(vec))
              }else{
                zdiff <- apply(rep(t(1:(qmin-1)),length(vec))^-t(kronecker(t(array(1,qmin-1)),vec)),2,sum)
              }
              L <- -vec*sum(log(zq)) - nq*log(zvec - zdiff);
              I <- which.max(L)
              # compute KS statistic
              fit <- cumsum((((qmin:qmax)^-vec[I]))
                            / (zvec[I]
                               - (if (qmin == 1) 0 else sum((1:(qmin-1))^-vec[I]))))
              
              hist = aggregate(zq, list(zq), length)
              cdi = rep(0, qmax - qmin + 1)
              cdi[hist$Group.1 - qmin + 1] = hist$x / nq
              cdi = cumsum(cdi)
              
              dat[qm] <- max(abs( fit - cdi ))
            }else{
              dat[qm] <- -Inf
            }
          }
        if(!quiet){
          print(paste(B,"-",sum(nof[1:B]>=gof)/B))
        }
        # -- store distribution of estimated gof values
        nof[B] <- min(c(dat, Inf))
        
        
      }
      
      ####################################
      
    }
    #
    #  end discrete case
    #
    ###########################################################################################
    #  return p and gof in a list
    p <- sum(nof>=gof)/length(nof)
    return(list(p=p,gof=gof))
  }
  
  sequence_test<-sequence
  
#   if(is.null(x_min)){
#     x_min<-round(dim(Omega@network)[1]*0.02)
#   }
   if(is.null(sequence_test)){
    sequence_test<-seq(0,min(max(abs(Omega@network*0.9)),0.4),length.out=10)
   }
#   
 #install_github('poweRlaw', 'csgillespie')
  #cutoff courant
  cc<-0
  #compteur
  u<-0
  #pvaleur
  f<-rep(0,length(sequence_test))
  
  #Boucle sur les valeurs candidates pour le cutoff
print("This calculation may be long")
  for(cc in sequence_test){
   
    u<-u+1
    print(paste(u,length(sequence_test),sep="/"))
    O<-Omega@network
    #On garde les liens d'intensite superieure au cutoff
    O[abs(O)>cc]<-1
    O[abs(O)<=cc]<-0
    #nombre de liens pour chaque gene
    liste<-apply(O,1,sum)+1
    if(length(which(liste>round(x_min)+1))<4){break}
    #On prend les genes ayant plus d'un lien
    xmin = plpva(liste,quiet=TRUE)$p
    f[u]<-xmin
  }
  #index du premier element superieur a 0.25
#   p<-which(sequence_test>0.25)[1]
#   #On ne conserve que ceux qui sont inferieurs a 0.25
#   f<-f[1:p]
print(f)
  K<-1:length(f)
  f2<-f
  #interpolation des points de f pour lisser
  f<-predict(loess(f~K))
  par(mar=c(5,4,0.2,0.2))
  plot(sequence_test
       ,f
       ,xlab="cutoff sequence"
       ,ylab="scale-freeness test p-value"
       ,type="l"
       ,lwd=2
       ,ylim=c(min(f)-0.02,max(f))
  )
  abline(h=0.1,col="red")
  rect(0.10,0.10,0.15,1,col=rgb(0,1,0,0.5))
  rect(0.05,0.10,0.10,1,col=rgb(0.5,1,0,0.4))
  rect(-0.5,0.10,0.05,1,col=rgb(1,0,0,0.4))
  rect(0.15,0.10,0.5,1,col=rgb(0.5,1,0,0.4))
  abline(h=0.1,col="red",lwd=3)
  legend("bottomright"
         ,lty=c(1,0,0,0)
         ,col=c("red"
                ,rgb(0,1,0,0.5)
                ,rgb(0.5,1,0,0.4)
                ,rgb(1,0,0,0.4))
         ,pch=c(NA,15,15,15)
         ,legend=c("p-value=0.1:  above this line, scale-freeness may be assumed"
                   ,"best area of choice (determined by simulation)"
                   ,"less recommended area of choice (determined by simulation)"
                   ,"area of choice to be avoided (determined by simulation)")
         ,lwd=c(3,5,5,5)
         ,cex=1
  )
  
  return(list(p.value=f2,p.value.inter=f,sequence=sequence_test)	)
}
)


#' Some basic criteria of comparison between actual and inferred network.
#' 
#' Allows comparison between actual and inferred network.
#' 
#' 
#' @name compare-methods
#' @aliases compare-methods compare compare,network,network,numeric-method
#' @docType methods
#' @return A vector containing : sensibility, predictive positive value, and
#' the F-score
#' @section Methods: \describe{
#' \item{list("signature(Net = \"network\", Net_inf = \"network\", nv =
#' \"numeric\")")}{}
#' }
#' @param Net A network object containing the
#' actual network.
#' @param Net_inf A network object containing the inferred
#' network.
#' @param nv A number that indicates at which level of cutoff the
#' comparison should be done.
#' @author Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
#' @references Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
#' Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
#' simulate the diffusion of a signal through a temporal gene network.
#' \emph{Bioinformatics}, btt705.
#' 
#' Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
#' Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
#' circuitry of a cancer cell with predicted intervention in chronic
#' lymphocytic leukemia. \emph{Proceedings of the National Academy of
#' Sciences}, 110(2), 459-464.
#' @examples
#' 
#' data(Net)
#' data(Net_inf)
#' 
#' #Comparing true and inferred networks
#' F_score=NULL
#' 
#' #Here are the cutoff level tested
#' test.seq<-seq(0,max(abs(Net_inf@network*0.9)),length.out=200)
#' for(u in test.seq){
#' 	F_score<-rbind(F_score,Cascade::compare(Net,Net_inf,u))
#' }
#' matplot(test.seq,F_score,type="l",ylab="criterion value",xlab="cutoff level",lwd=2)
#' 
#' @exportMethod compare
setMethod("compare",c("network","network","numeric"),function(Net,
                                                    Net_inf,
                                                    nv=1){
  N1<-Net@network
  N2<-Net_inf@network
  N1[abs(N1)>0]<-1
  N1[abs(N1)<=0]<-0
  N2[abs(N2)>nv]<-1
  N2[abs(N2)<=nv]<-0
  Nb<-sum(N1)
  sens<-0
  if(sum(N1==1)){
    sens<-sum((N1-2*N2)==-1)/sum(N1==1)
  }
  spe<-0
  if(sum(N2==1)){
    spe<-sum((N1-2*N2)==-1)/sum(N2==1)
  }
  Fscore<-0
  Fscore12<-0
  Fscore2<-0
  if(sens !=0){
    Fscore<-2*spe*sens/(spe+sens)
    Fscore12<-(1+0.5^2)*spe*sens/(spe/4+sens)
    Fscore2<-(1+2^2)*spe*sens/(spe*4+sens)
    return(c(sens,spe,Fscore,Fscore12,Fscore2))
  }
  return(c(sens,spe,Fscore,Fscore12,Fscore2))
}
)

#' Prediction of the gene expressions after a knock-out experience
#' \code{predict}
#' 
#' Prediction of the gene expressions after a knock-out experience
#' 
#' 
#' @aliases predict predict-methods predict,ANY-method
#' predict,micro_array-method
#' @param object a micro_array object
#' @param Omega a network object.
#' @param nv [=0] numeric; the level of the cutoff
#' @param targets [NULL] vector; which genes are knocked out?
#' @param adapt [TRUE] boolean; do not raise an error if used with vectors 
#' instead of one column matrices.
#' @author Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
#' @references Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
#' Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
#' simulate the diffusion of a signal through a temporal gene network.
#' \emph{Bioinformatics}, btt705.
#' 
#' Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
#' Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
#' circuitry of a cancer cell with predicted intervention in chronic
#' lymphocytic leukemia. \emph{Proceedings of the National Academy of
#' Sciences}, 110(2), 459-464.
#' @keywords methods
#' @examples
#' 
#' data(Selection)
#' data(network)
#' #A nv value can chosen using the cutoff function
#' nv=.11
#' EGR1<-which(match(Selection@name,"EGR1")==1)
#' P<-position(network,nv=nv)
#' 
#' #We predict gene expression modulations within the network if EGR1 is experimentaly knocked-out. 
#' prediction_ko5<-predict(Selection,network,nv=nv,targets=EGR1)
#' 
#' #Then we plot the results. Here for example we see changes at time point t2:
#' plot(prediction_ko5,time=2,ini=P,label_v=Selection@name)
#' 
#' @exportMethod predict
setMethod("predict"
          ,c("micro_array")
          ,function(object
                    ,Omega
                    ,nv=0
                    ,targets=NULL
                    ,adapt=TRUE
          ){
            micro<-object
            if(!is.null(targets)){
              micro@microarray[targets,]<-0
            }
            #groups
            groupe<-object@group
            #microarray
            M<-micro@microarray
            #number of measurements (=number of timepoints)
            T<-length(object@time)
            #genes
            gene<-1:length(groupe)
            gene2<-gene
            #first times
            supp<-seq(1,T*object@subject,T)
            #all the times
            supp2<-1:(T*object@subject)
            #all times that are not first times
            supp2<-supp2[-supp]
            #removing silenced genes
            if(!is.null(targets)){
              gene<-gene[-targets]
            }
            #weights
            O<-Omega@network
            #cutoff
            O[abs(O)<nv]<-0
            #F matrices
            F<-Omega@F
            #microarray (again)
            microP<-micro
            #predictors
            sup_pred<-rep(1:(T-1),micro@subject)+rep(seq(0,T*(micro@subject-1),T),each=T-1)
            
            
            #microarray (again)
            micro2<-micro
            #index for the F matrices
            u<-0
            for(peak in 2:(T)){
              
              #keeping predictor groups
              IND<-which(groupe[gene2]%in%1:(peak-1))
              grIND<-groupe[IND]
              #silencing targets
              if(!is.null(targets)){
                micro2@microarray[targets,]<-micro@microarray[targets,]
              }
              #predictors genes (timepoints 1..T-1)
              pred<-micro2@microarray[IND,sup_pred]
              #print(pred[IND==72,])
              #for every group
              for(k in 1:(peak-1)){
                #indices for the kth group
                ind<-which(grIND %in% k)
                #increasing the F matrix index
                u<-u+1
                #product of the F matrix and a vector
                f<-function(x){(F[,,u]%*%(x))}#/(sqrt(sum((F[,,u]%*%(x))^2)))}
                #for each subject
                for(i in 1:micro@subject){
                  pred[ind,1:(T-1)+(i-1)*(T-1)]<-t(apply(pred[ind,1:(T-1)+(i-1)*(T-1)],1,f))
                }
              }
              pred[is.na(pred)]<-0
              #Indices des genes du groupe d'interet
              IND2<-which(groupe[gene2]==(peak))
              #Pour chaque gene du groupe reponse
              for(j in IND2){
                #predj<-(pred)*O[IND,j]
                #Pour chaque gene on cherche les genes du groupe predicteur qui ont un lien non nul
                predj<-pred[O[IND,j]!=0,]
                #Si il y en a
                if(length(predj)!=0){
                  #On isole les valeurs finales dans Y (reponse)
                  Y<-micro@microarray[j,sup_pred+1]
                  if(adapt==TRUE){
                    if(!is.null(dim(predj))){
                      mm<-lm(Y~t(predj)-1)
                    }
                    else{
                      mm<-lm(Y~(predj)-1)
                    }
                    #On remplace par la valeur inferee
                    micro2@microarray[j,sup_pred+1]<-predict(mm)
                    #On update les coefficients de Omega
                    O[IND,j][O[IND,j]!=0]<-coef(mm)[]
                  }
                  #Si il n'y en a pas
                }
                else{
                  #On prend la somme ponderee des sorties 
                  #des genes exprimes aux temps precedents
                  predj<-apply((pred)*O[IND,j],2,sum)	
                  micro2@microarray[j,sup_pred+1]<-predj
                }
                
              }
            }
            #Pour pouvoir faire un plot
            micro33<-micro2
            if(!is.null(targets)){ 
              pppp<- unique(unlist(geneNeighborhood(Omega,targets,nv,graph=FALSE)))
              genes3<-gene2[-pppp]                           
            }            else{
              genes3<-gene2
            }
            if(!is.null(targets)){
              micro33@microarray[ genes3,]<-micro@microarray[genes3,]
            }		
            
            if(is.null(targets)){
              targets<- -1
            }
            subjects<-object@subject
            times<-object@time
            ntimes<-length(times)
            patients<-paste(rep("P",subjects*ntimes),rep(1:subjects,each=ntimes),sep="")
            temps<-paste(rep("T",subjects*ntimes),rep(times,subjects),sep="")
            indicateurs<-paste(patients,temps,sep="")
            expr<-rep("log(S/US)",subjects*ntimes)
            nomscol<-paste(expr,":",indicateurs)
            #nomscol<-paste(patients,timestring1)
            
            colnames(object@microarray)<-nomscol
            colnames(micro@microarray)<-nomscol
            colnames(micro33@microarray)<-nomscol
            return(new("micropredict"
                       ,microarray_unchanged=object
                       ,microarray_changed=micro
                       ,microarray_predict=micro33
                       ,nv=nv
                       ,network=Omega
                       ,targets=targets
            )
            )	
          }
)


#' Reverse-engineer the network
#' 
#' Reverse-engineer the network.
#' 
#' 
#' @aliases inference inference-methods inference,micro_array-method
#' @param M a micro_array object.
#' @param tour.max maximal number of steps. Defaults to `tour.max=30`
#' @param g the new solution is choosen as
#' (the old solution + g(x) * the new solution)/(1+g(x)) where x is the number
#' of steps. Defaults to `g=function(x) 1/x` 
#' @param conv convergence criterion. Defaults to `conv=10e-3`
#' @param cv.subjects should the cross validation be done removing the
#' subject one by one ? Defaults to `cv.subjects=TRUE`.
#' @param nb.folds Relevant only if cv.subjects is
#' FALSE. The number of folds in cross validation. Defaults to `NULL`.
#' @param eps machine zero. Defaults to `10e-5`.
#' @param type.inf "iterative" or "noniterative" : should the
#' algorithm be computed iteratively. Defaults to `"iterative"`.
#' @return A network object.
#' @author Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
#' @references Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
#' Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
#' simulate the diffusion of a signal through a temporal gene network.
#' \emph{Bioinformatics}, btt705.
#' 
#' Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
#' Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
#' circuitry of a cancer cell with predicted intervention in chronic
#' lymphocytic leukemia. \emph{Proceedings of the National Academy of
#' Sciences}, 110(2), 459-464.
#' @keywords methods
#' @examples
#' 
#' \donttest{
#' #With simulated data
#' data(M)
#' infM <- inference(M)
#' str(infM)
#' 
#' #With selection of genes from GSE39411
#' data(Selection)
#' infSel <- inference(Selection)
#' str(infSel)
#' }
#' 
#' @exportMethod inference
setMethod(f="inference"
          ,signature=c("micro_array")
          ,definition=function(M
                               ,tour.max=30
                               ,g=function(x){1/x}
                               ,conv=0.001
                               ,cv.subjects=TRUE
                               ,nb.folds=NULL
                               ,eps=10^-5
                               ,type.inf="iterative"
          ){
            #Package requis
            require(nnls)
            #Quelques indicateurs
            mat<-M@microarray 
            #La matrice contenant les donnees
            gr<-M@group 
            #Le vecteur des groupes
            N<-dim(mat)[1] 
            #Nombre de genes
            T<-length(unique(M@group)) 
            #Nombre de temps de mesure
            P<-M@subject 
            #Nombre de patients
            
            #La condition suivante determine le nombre de folds 
            # pour la cross validation
            if(is.null(nb.folds)){
              K<-T-1
            }
            else{
              K<-nb.folds
            }
            
            nF<-(1+T-1)/2*(T-1) 
            #nb de matrices Fab
            #Simple calcul du nombre de matrices Fab 
            # (selon le modele initial)
            
            #Initialisation des matrices Fab
            F<-array(0,c(T-1,T-1,nF)) 
            #J'ai prefere construire une matrice F 
            # Tri dimensionnelle laquelle contient toutes les matrices Fab
            # F[,,1] correspond a la matrice F12
            # F[,,2] correspond a la matrice F13
            # F[,,3] correspond a la matrice F14
            # ... (les matrices nulles ne sont pas 
            # indicees F11 par exemple)
            
            for(i in 1:nF){
              for(j in 1:(T-1)){
                F[j,j,i]<-1
              }
            } 
            #L'initialisation prend pour toutes les matrices Fab 
            #la matrice identite T-1 * T-1
            
            
            #Initialisation de la matrice Omega
            Omega<-array(0,c(N,N))
            
            #Initialisation des deux indicateurs de convergence
            convF<-rep(mean(F^2),nF)
            convO<-mean(mat^2)
            
            #Support : correspond aux colonnes de mat 
            # qui servent pour la prediction
            #Attention le modele se sert que des temps 1 a 
            # T-1 pour chaque patient pour les predicteurs
            sup_pred<-rep(1:(T-1),P)+rep(seq(0,T*(P-1),T),each=T-1)
            
            #Initialisation du nombre de tours	
            tour<-1
            
            #Si on veut faire la version non iterative
            # il faut deux passages : dans le premier on infere
            # F et dans le second on infere omega
            if(type.inf=="noniterative"){
              tour.max<-2
            }
            
            #L'algorithme commence ici
            while(tour<= tour.max && convO[length(convO)]>conv){ 
              #Condition d'arret : soit nombre de tour max atteint, 
              #soit la convergence de la matrice Omega est suffisante
              
              cat(paste("We are at step : ",tour))
              cat("\n") 
              #Pour montrer que l'algorithme est en train de calculer
              
              OmegaS<-Omega 
              #OmegaS comme sauvegarde ; necessaire pour calculer
              # la convergence
              
              u<-0 
              #u a un role essentiel, puisque qu'il determine 
              # laquelle des matrices Fab est indicee. 
              # Se referer plus haut pour en connaitre l'ordre
              
              for(peak in 2:T){ 
                #Comprendre ici que peak correspond au groupe du 
                # gene REPONSE ; en consequence, nous 
                # commencons a deux.
                
                IND<-which(gr %in% 1:(peak-1)) 
                #Ici nous cherchons les genes possiblement 
                # PREDICTEURS.
                #En effet, le gene reponse etant de groupe peak, 
                # un predicteur ne peut etre que de groupe 1 a (peak-1)
                
                grIND<-gr[IND] 				
                #Nous recuperons le groupe des individus possiblement
                # predicteurs.
                pred<-mat[IND,sup_pred]		
                #Nous creons la matrice des predicteurs 
                
                
                for(k in 1:(peak-1)) { 
                  #Cette boucle sert a transformer les 
                  # predicteurs en fonction des matrices Fab
                  #Le groupe de la variable reponse 
                  # etant peak, les groupes des predicteurs 
                  # sont 1..(peak-1)
                  
                  ind<-which(grIND %in% k) 
                  #On regarde successivement, et dans 
                  # l'ordre, tous les groupes possibles
                  
                  u<-u+1 
                  #u est initialise a 0. La premiere fois, 
                  # u vaut donc 1, peak 2, et k =1. Donc F[,,u]=F12.
                  #La deuxieme fois qu'on arrive ici 
                  # peak est passe a 3, et k vaut de nouveau 1  
                  # F[,,u]=F13
                  #La troisieme fois peak reste a 2 car la boucle 
                  # avec k n'est pas finie, et k vaut 2 donc 
                  # F[,,u]=F23
                  # ... c'est bien l'ordre dans lequel nous 
                  # avons range les F
                  
                  f<-function(x){(F[,,u]%*%(x))} 
                  #On construit une fonction generique
                  
                  for(i in 1:P){
                    pred[ind,1:(T-1)+(i-1)*(T-1)]<-t(apply(pred[ind,1:(T-1)+(i-1)*(T-1)],1,f)) 
                    #La transformation est faite ici.
                  }
                }
                #En sortant de la boucle avec k, pred 
                # contient les genes predicteurs correctement 
                # transformes. 				
                
                pred[is.na(pred)]<-0 
                #Ceci est une securite, au cas ou une matrice F 
                # deviendrait nulle
                
                #On construit ci dessous la matrice des vecteurs 
                # reponse
                Y<-mat[which(gr %in% peak),sup_pred+1]
                Omega[IND, which(gr %in% peak)]<-Omega[IND, which(gr %in% peak)]*0
                
                #Nous allons passer au Lasso
                #retenir<-lars::cv.folds 
                #Ceci permet de changer une fonction interne de lars 
                # qui s'occupe de la validation croisee. 
                
                if(cv.subjects==TRUE){
                  cv.folds1=function(n,folds){
                                      split(1:dim(pred)[2]
                                            ,rep(1:P,each=dim(pred)[2]/P))}
#                  cv.fun.name="cv.folds1"
                  } else {
                    cv.folds1=lars::cv.folds
#                    cv.fun.name="lars::cv.folds"
                  }
#                cat(cv.fun.name)
                fun_lasso<-function(x){lasso_reg(pred,x,K=K,eps,cv.fun=cv.folds1
                                                 #,cv.fun.name=cv.fun.name
                                                 )} 
                
                  # assignInNamespace("cv.folds"
                  #                   ,function(n,folds){
                  #                     split(1:dim(pred)[2]
                  #                           ,rep(1:P,each=dim(pred)[2]/P)
                  #                     )
                  #                   }
                  #                   , ns="lars")
                  Omega[IND, which(gr %in% peak)]<-apply(Y,1,fun_lasso)
#                  assignInNamespace("cv.folds",retenir, ns="lars")
                # }
                # else{
                #   Omega[IND, which(gr %in% peak)]<-apply(Y,1,fun_lasso)
                # }
                
              } 
              #fin de la boucle for avec peak ; 
              # la matrice omega est inferee
              
              co<-apply(Omega,2,sumabso)
              Omega<-t(t(Omega)/co)
              
              if(tour!=1 && type.inf=="iterative"){
                Omega<-(g(tour)*Omega+OmegaS)/(1+g(tour)) 
                #On prend seulement une partie de l'innovation
              }
              
              
              convO<-c(convO,mean(abs(Omega-OmegaS)))
              
              if( type.inf=="iterative"){
                cat(paste("The convergence of the network is (L1 norm) :", round(convO[length(convO)],5)))	
                cat("\n")	
              }
              uuu<-0
              sauvF<-F
              
              if(tour==1 && type.inf=="noniterative"){
                Omega<-Omega*0+1 
                #Tous les predicteurs sont mis a un dans le 
                # cadre de l'inference non iterative
              }
              
              #Maintenant, il s'agit d'estimer la matrice F ; 
              # on reprend la meme maniere de faire que pour Omega
              #Nous annotons les differences
              
              for(peak in 2:T){
                IND<-which(gr %in% 1:(peak-1))
                grIND<-gr[IND]
                sup_pred<-rep(1:(T-1),P)+rep(seq(0,T*(P-1),T),each=T-1)
                pred<-(mat[IND,sup_pred])
                IND2<-which(gr %in% peak)
                
                Xf<-NULL
                
                #Important : etant donne qu'ils apparaissent dans 
                # les memes equations Fij, i=1...(j-1) doivent 
                #etre estimees en meme temps
                
                for(i in 1:(peak-1)){ 
                  #Cette  boucle permet de creer la matrice des 
                  # predicteurs selon une forme pratique
                  X<-NULL
                  suma<-function(x){sum(abs(x))}
                  f<-Vectorize(
                    function(x){
                      #Multiplie la matrice des pr??dicteurs du groupe i
                      # par les omega correspondants aux genes du groupe
                      # dont les indices sont dans x et on somme les 
                      # valeurs absolues
                      apply(pred[which(grIND==i),]*Omega[IND[which(grIND==i)],x],2,suma)
                    }
                  )
                  #On applique cette fonction aux genes de peak
                  Xa<-(f(IND2))
                  Xb<-NULL
                  
                  #Boucle sur les patients
                  for(p in 1:P){
                    Q<-NULL
                    for(r in 1:(T-1)){
                      q<-Xa[1:(T-1)+(p-1)*(T-1),]
                      q<-c(rep(0,(r-1)),q[1:(length(q)-(r-1))])
                      if(r!=1){q[which((1:length(q) %% (T-1)) %in% 1:(r-1))]<-0 }
                      Q<-cbind(Q,q)
                    }
                    X<-rbind(X,Q)
                  }
                  Xf<-cbind(Xf,X)
                }
                
                
                Y<-c(t(mat[IND2,sup_pred+1]))
                pond<-rep(0,P)
                coeffi<-array(0,c(P,dim(Xf)[2]))
                
                for(pat in 1:P){
                  support<-1:length(Y)
                  enl<-(1:(length(Y)/(P))+(pat-1)*(length(Y)/(P)))
                  support<-support[-enl]
                  model<-nnls(abs(Xf[support,]),abs(Y[support]))
                  pond[pat]<-1/(mean((Xf[enl,]%*%coef(model)-Y[enl])^2) )
                  coeffi[pat,]<-coef(model)
                }
                
                model<-apply(coeffi*( pond/sum(pond)),2,sum)
                pp<-length(model)/(T-1)
                
                pk<-uuu+1
                
                for(jj in 1:pp){
                  uuu<-uuu+1
                  F[,,uuu]<-F_f(F[,,uuu],model[1:(T-1)+(jj-1)*(T-1)])
                }
              } 
              #fin de la boucle peak
              
              if(type.inf=="iterative"){
                F<-(g(tour)*F+sauvF)/(1+g(tour))
              }
              cc<-rep(0,nF)
              for(i in 1:nF){cc[i]<-mean(abs((F[,,i]/sum(F[,,i])-sauvF[,,i]/sum(sauvF[,,i]))))}
              convF<-cbind(convF,cc)
              tour<-tour+1
            }
            
            if(type.inf=="iterative"){
              plot(convO[-1],type="l")
              #dev.new()
              matplot(t(convF),type="l")
            }
            else{
              F<-sauvF   
            }
            result<-new("network"
                        ,network=Omega
                        ,name=M@name
                        ,F=F
                        ,convF=convF
                        ,convO=convO
                        ,time_pt=M@time
            )
            return(result)
          }
          
)


#' Simulates microarray data based on a given network.
#' 
#' Simulates microarray data based on a given network.
#' 
#' 
#' @aliases gene_expr_simulation gene_expr_simulation-methods
#' gene_expr_simulation,network-method
#' @param network A network object.
#' @param time_label a vector containing the time labels.
#' @param subject the number of subjects
#' @param level_peak the mean level of peaks.
#' @return A micro_array object.
#' @author Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
#' @references Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
#' Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
#' simulate the diffusion of a signal through a temporal gene network.
#' \emph{Bioinformatics}, btt705.
#' 
#' Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
#' Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
#' circuitry of a cancer cell with predicted intervention in chronic
#' lymphocytic leukemia. \emph{Proceedings of the National Academy of
#' Sciences}, 110(2), 459-464.
#' @examples
#' 
#' data(Net)
#' set.seed(1)
#' 
#' #We simulate gene expression according to the network Net
#' Msim<-gene_expr_simulation(
#' 	network=Net,
#' 	time_label=rep(1:4,each=25),
#' 	subject=5,
#' 	level_peak=200)
#' head(Msim)
#' 
#' @exportMethod gene_expr_simulation
setMethod("gene_expr_simulation"
          ,"network"
          ,function(network
                    ,time_label=1:4
                    ,subject=5
                    ,level_peak=100
          ){
            require(VGAM)
            
            N<-network@network
            M<-matrix(0,dim(network@network)[1],length(unique(time_label))*subject)
            T<-length(unique(time_label))
            gene1<-which(time_label==1)
            supp<-seq(1,dim(M)[2],by=length(unique(time_label)))
            M[gene1,supp]<-VGAM::rlaplace(length(supp)*length(gene1),level_peak,level_peak*0.9)*(-1)^rbinom(length(supp)*length(gene1),1,0.5)
            supp<-(1:dim(M)[2])[-supp]
            M[gene1,supp]<-VGAM::rlaplace(length(supp)*length(gene1),0,level_peak*0.3) 
            
            
            for(i in 2:T){
              
              genei<-which(time_label==i)
              supp<-seq(1,dim(M)[2],by=length(unique(time_label)))
              M[genei,supp]<-VGAM::rlaplace(length(supp)*length(genei),0,level_peak*0.3)
              for(j in genei){
                for( t in 2:T){
                  M[j,supp+t-1]<-apply(N[,j]*M[,supp+(t-2)],2,sum) + 	rnorm(length(supp+t),0,50)	
                  
                }			
                
              }
              
              
            }
            
            MM<-as.micro_array(M,1:length(unique(time_label)),subject)
            MM@group<-time_label
            
            
            
            G<-predict(MM,network)@microarray_predict
            supp<-seq(1,dim(M)[2],by=length(unique(time_label)))
            G@microarray[,supp]<-M[, supp]	
            return(G)
          }
)

#' Coerce a matrix into a micro_array object.
#' 
#' Coerce a matrix into a micro_array object.
#' 
#' 
#' @param M A matrix. Contains the microarray measurements. Should of size N *
#' K, with N the number of genes and K=T*P with T the number of time points,
#' and P the number of individuals. This matrix should be created using
#' cbind(M1,M2,...) with M1 a N*T matrix with the measurements for individual
#' 1, M2 a N*T matrix with the measurements for individual 2.
#' @param time A vector. The time points measurements.
#' @param subject The number of subjects.
#' @return A micro_array object.
#' @author Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
#' @references Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
#' Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
#' simulate the diffusion of a signal through a temporal gene network.
#' \emph{Bioinformatics}, btt705.
#' 
#' Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
#' Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
#' circuitry of a cancer cell with predicted intervention in chronic
#' lymphocytic leukemia. \emph{Proceedings of the National Academy of
#' Sciences}, 110(2), 459-464.
#' @examples
#' 
#'   if(require(CascadeData)){
#' 	data(micro_US)
#' 	micro_US<-as.micro_array(micro_US,time=c(60,90,210,390),subject=6)
#' 	}
#' 
#' @export
as.micro_array<-function(M,time,subject){
  
  if(is.null(row.names(M))){row.names(M)<-paste("gene",1:dim(M)[1])}
  return(new("micro_array",microarray=as.matrix(M),name=row.names(M),time=time,subject=subject,group=0,start_time=0))	
}

cv.lars1 <- function (x, y, K = 10, index, trace = FALSE, plot.it = TRUE, 
          se = TRUE, type = c("lasso", "lar", "forward.stagewise", 
                              "stepwise"), mode = c("fraction", "step"), cv.fun
          #, cv.fun.name
          , ...) 
{
#  requireNamespace("lars")
#  cat(cv.fun.name)
  type = match.arg(type)
  if (missing(mode)) {
    mode = switch(type, lasso = "fraction", lar = "step", 
                  forward.stagewise = "fraction", stepwise = "step")
  }
  else mode = match.arg(mode)
  all.folds <- cv.fun(length(y), K)
#  cat(all.folds[[1]],"\n")
  if (missing(index)) {
    index = seq(from = 0, to = 1, length = 100)
    if (mode == "step") {
      fit = lars::lars(x, y, type = type, ...)
      nsteps = nrow(fit$beta)
      maxfold = max(sapply(all.folds, length))
      nsteps = min(nsteps, length(y) - maxfold)
      index = seq(nsteps)
    }
  }
  residmat <- matrix(0, length(index), K)
  for (i in seq(K)) {
    omit <- all.folds[[i]]
    fit <- lars::lars(x[-omit, , drop = FALSE], y[-omit], trace = trace, 
                type = type, ...)
    fit <- lars::predict.lars(fit, x[omit, , drop = FALSE], mode = mode, 
                   s = index)$fit
    if (length(omit) == 1) 
      fit <- matrix(fit, nrow = 1)
    residmat[, i] <- apply((y[omit] - fit)^2, 2, mean)
    if (trace) 
      cat("\n CV Fold", i, "\n\n")
  }
  cv <- apply(residmat, 1, mean)
  cv.error <- sqrt(apply(residmat, 1, var)/K)
  object <- list(index = index, cv = cv, cv.error = cv.error, 
                 mode = mode)
  if (plot.it) 
    lars::plotCVLars(object, se = se)
  invisible(object)
}

lasso_reg<-function(M,Y,K,eps,cv.fun=lars::cv.folds
                    #,cv.fun.name="lars::cv.folds"
                    ){
  #  require(lars)
#  cat("lasso_reg",cv.fun.name,"\n")
  model<-try(cv.lars1(t(M),(Y),intercept=FALSE,K=K,plot.it=FALSE,eps=10^-5,cv.fun=cv.fun
                      #, cv.fun.name=cv.fun.name
                      ))
  n<-try(model$index[which(model$cv %in% min(model$cv))])
  model<-try(lars::lars(t(M),(Y),intercept=FALSE,eps=10^-5))
  repu<-try(lars::coef.lars(model,s=n,mode="fraction"))
  if(!is.vector(repu)){repu<-rep(0,dim(M)[1])}
  return(repu)
}

lasso_reg_old<-function(M,Y,K,eps){
  #  require(lars)
  model<-try(lars::cv.lars(t(M),(Y),intercept=FALSE,K=K,plot.it=FALSE,eps=10^-5))
  n<-try(model$index[which(model$cv %in% min(model$cv))])
  model<-try(lars::lars(t(M),(Y),intercept=FALSE,eps=10^-5))
  repu<-try(lars::coef.lars(model,s=n,mode="fraction"))
  if(!is.vector(repu)){repu<-rep(0,dim(M)[1])}
  return(repu)
}

F_f<-function(F,x){
  for(i in 1:dim(F)[1]){
    F[col(F)==row(F)-(i-1)]<-x[i]
  }
  #for(i in 1:dim(F)[1]){
  #if(sum(abs(F[i,]))!=0){
  #F[i,]<-F[i,]/sum(abs(F[i,]))
  #}
  #}
  #for(i in  1:dim(F)[1]){
  #if(sum(F[i,])==0){F[i,i]<-1}
  #}
  return(F)
}

sumabso<-function(x){
  d<-sum(abs(x))
  if(d==0){
    d<-1
  }
  return(d)
}

expo<-function(x,hub){	
  sum(x>=hub)/sum(x>0)
}

choice_cutoff<-function(O,nb,eps,hub,plot.g=FALSE,prop.hub=NULL){
  O<-abs(O)
  #plus petit omega sup a eps
  minO<-min(O[O>eps])
  #mediane des omegas
  qq<-quantile(O[O>eps],0.50)
  #max des omega
  maxO<-max(O)
  #cutoffs
  sequence<-seq(minO,maxO,length.out=nb)
  
  Mcut<-array(0,c(dim(O)[1],nb))
  #pour chaque valeur du cutoff on calcule 
  for(i in 1:nb){
    Mcut[,i]<-apply(O>sequence[i],1,sum)
  }
  
  expoh<-function(x){expo(x,hub)}
  hh<-apply(Mcut[,1:(dim(Mcut)[2]-1)],2,expoh)
  if(plot.g==TRUE){
    matplot(t(Mcut),type="l")
    
    dev.new()
    plot(sequence[which(hh>0)],hh[hh>0],type="l",xlab="cut off",ylab="Proportion of hubs")
  }

  auto<-min(which(hh==min(hh[hh>0])))
  if(!is.null(prop.hub)){
    if(prop.hub>min(hh[hh>0])){
      auto<-min(which(hh==min(hh[hh>prop.hub])))
      #abline(h=prop.hub)
    }
    
  }
  
  #print(Mcut[,auto])
  return(sequence[auto])
  
}

choice_cutoff_final<-function(O,nb,eps,hub,plot.g=FALSE,prop.hub){
  
  if(length(hub)>1){
    choix<-NULL
    for(i in hub){
      choix<-c(choix,choice_cutoff(O,nb,eps,i,prop.hub=prop.hub)	)
    }
    plot(hub,choix,type="l")
    lines(hub,predict(loess(choix ~ hub, span=0.75)),col="red")
    return(choix)
  }
  else{
    choix<-NULL
    for(i in prop.hub){
      choix<-c(choix,choice_cutoff(O,nb,eps,hub,prop.hub=i)	)
    }
    plot(prop.hub,choix,type="l")
    lines(prop.hub,predict(loess(choix ~ prop.hub, span=0.75)),col="red")
    return(choix)
  }
}

#simulations



#' Generates a network.
#' 
#' Generates a network.
#' 
#' 
#' @param nb Integer. The number of genes.
#' @param time_label Vector. The time points measurements.
#' @param exp The exponential parameter, as in the barabasi.game function in
#' igraph package.
#' @param init The attractiveness of the vertices with no adjacent edges. See
#' barabasi.game function.
#' @param regul A vector mapping each gene with its number of regulators.
#' @param min_expr Minimum of strength of a non-zero link
#' @param max_expr Maximum of strength of a non-zero link
#' @param casc.level ...
#' @return A network object.
#' @author Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
#' @references Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
#' Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
#' simulate the diffusion of a signal through a temporal gene network.
#' \emph{Bioinformatics}, btt705.
#' 
#' Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
#' Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
#' circuitry of a cancer cell with predicted intervention in chronic
#' lymphocytic leukemia. \emph{Proceedings of the National Academy of
#' Sciences}, 110(2), 459-464.
#' @examples
#' 
#' set.seed(1)
#' Net<-network_random(
#' 	nb=100,
#' 	time_label=rep(1:4,each=25),
#' 	exp=1,
#' 	init=1,
#' 	regul=round(rexp(100,1))+1,
#' 	min_expr=0.1,
#' 	max_expr=2,
#' 	casc.level=0.4
#' 	)
#' plot(Net)
#' 
#' @export
network_random<-function(nb,time_label,exp,init,regul,min_expr,max_expr,casc.level){
  
  net<-matrix(0,nb,nb)
  net2<-net
  while(!identical(regul[which(time_label != 1)] , apply(net,2,sum)[which(time_label != 1)])) {
    for(i in 1:nb){
      if(time_label[i] !=1){
        
        if(rbinom(1,1,1-casc.level)==1){	
          reg<-which(time_label<time_label[i])}
        else{
          reg<-which(time_label==(time_label[i]-1))
        }
        if(length(reg)!=0 && sum(sum(net[,i])<regul[i])){
          pb<-apply(net[reg,],1,sum)^exp
          pb<-(pb+init)/(sum(pb+init))
          r<-rmultinom(1, 1, pb)
          net[reg[which(r==1)],i]<-1
          net2[reg[which(r==1)],i]<-runif(1,min_expr,max_expr)*(-1)^rbinom(1,1,0.5)
        }
      }
    }
  }
  length(unique(time_label))->T
  F<-array(0,c(T-1,T-1,T*(T-1)/2))
  for(i in 1:(T*(T-1)/2)){diag(F[,,i])<-1}
  N<-new("network",network=net2,name=paste("gene",1:nb),F=F,convO=0,convF=matrix(0,1,1),time_pt=1:length(unique(time_label)))
  return(N)
}

test_that("micro_array basic structure is consistent", {
  data(M, package = "Cascade")
  expect_s4_class(M, "micro_array")
  expect_true(is.matrix(M@microarray))
  # Columns must equal time points * subjects (validity rule)
  expect_equal(ncol(M@microarray), length(M@time) * M@subject)
  # Basic slot sanity
  expect_true(length(M@time) >= 1)
  expect_true(M@subject >= 1)
})

test_that("printing and showing micro_array work", {
  data(M, package = "Cascade")
#  expect_no_error(show(M))
  expect_no_error(print(M))
})

skip_on_cran()
skip_on_ci()

test_that("predict() returns a 'micropredict' object with coherent slots", {
  skip_if_not_installed("igraph")
  skip_if_not_installed("magic")
  data(M, package = "Cascade")
  data(Net, package = "Cascade")
  # Use a small target set
  targets <- c(5L, 10L)
  pred <- predict(M, Omega = Net, nv = 0, targets = targets)
  expect_s4_class(pred, "micropredict")
  # Slots and classes
  expect_s4_class(pred@network, "network")
  expect_s4_class(pred@microarray_unchanged, "micro_array")
  expect_s4_class(pred@microarray_changed, "micro_array")
  expect_s4_class(pred@microarray_predict, "micro_array")
  expect_equal(pred@nv, 0)
  expect_equal(as.integer(pred@targets), targets)
})

test_that("network dataset loads and has expected shape", {
  data(network, package = "Cascade")
  expect_s4_class(network, "network")
  expect_true(is.matrix(network@network))
  expect_equal(nrow(network@network), ncol(network@network))
})

test_that("position() returns a 2-column coordinate matrix", {
  skip_if_not_installed("igraph")
  data(network, package = "Cascade")
  pos <- Cascade::position(network, nv = 0)
  expect_true(is.matrix(pos))
  expect_equal(ncol(pos), 3)
  expect_equal(nrow(pos), nrow(network@network) - length(which(rowSums(abs(network@network) > 0) + colSums(abs(network@network) > 0) == 0)))
})

test_that("analyze_network() returns expected columns", {
  skip_if_not_installed("tnet")
  data(network, package = "Cascade")
  ana <- analyze_network(network, nv = 0)
  expect_s3_class(ana, "data.frame")
  expect_true(all(c("node","betweenness","degree","output","closeness") %in% colnames(ana)))
  expect_equal(nrow(ana), nrow(network@network))
})

test_that("geneNeighborhood() returns neighborhoods when graph=FALSE", {
  skip_if_not_installed("igraph")
  data(network, package = "Cascade")
  set.seed(123)
  tgt <- sample(seq_len(nrow(network@network)), size = 1)
  K <- geneNeighborhood(network, targets = tgt, nv = 0, order = 2, graph = FALSE)
  expect_type(K, "list")
  expect_equal(length(K), 2)
})

#!/bin/sh
#
# $Id: cascade.sh,v 1.2 2001/09/05 17:52:47 dan Exp $
#
#
# Cascade
#
# A program for performing noise figure, gain, and third order intercept
# analysis of a cascade.
#
# 
# Copyright (c) 1997-2001 Dan McMahill <mcmahill@alum.mit.edu>
# All rights reserved.
#
# This code is derived from software written by Dan McMahill
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#        This product includes software developed by Dan McMahill
#  4. The name of the author may not be used to endorse or promote products
#     derived from this software without specific prior written permission.
# 
#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
#  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
#  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#  SUCH DAMAGE.
# 

AWK=@awk@

#
# This program was developed using Gnu Awk (gawk) 2.15, patchlevel 6
# running on a MacIntosh IIci under NetBSD/mac68k 1.2.1
# It was later extended under NetBSD/mac68k 1.3 still using gawk
# 2.15 patchlevel 6.
# 
# It should work with any version of gawk that is at least 2.15, and
# at least with some versions of nawk (I have only tested it with nawk
# on SunOs4.1.4 and Solaris2.5.1).  It will not work with a more
# traditional awk.
#


##############################
# START OF PROGRAM
##############################

PROGNAME="cascade"
VERSIONNUM="@version@"

##################
# Check for help request and/or version request
##################

if test "$1" != ""
then
    case "$1" in
	--copyright)
	    echo " "
	    echo "Copyright 1997-2001, Dan McMahill" 
	    echo " "
	    echo "Cascade is freely available under a BSD-style copyright"
	    echo "Please refer to the file COPYING which should have come"
	    echo "with your copy of cascade."
	    echo " "
	    exit 0 ;;

	--help) 
	    echo ""
	    echo "USAGE:  $0 --option"
	    echo "        $0 infilename"
 	    echo "        $0 infilename > outfile "
	    echo ""
	    echo "Available options are:"
	    echo "  --copyright     Displays copyright information"
	    echo "  --help          Displays this help message"
	    echo "  --version       Displays version information for"\
			"the program"
	    echo ""
	    echo "$0 inputfile"
	    echo "  reads a description of a system level cascade from"\
			"the text inputfile"
	    echo "  and computes noise figure, third order intercept,"\
			"and C/N degredation"
	    echo "  at each stage of the cascade.  The results are sent"\
			"to the standard"
	    echo "  output.  The standard output may be redirected to a"\
			"file using:"
	    echo "  $0 infilename > outfilename"
	    echo ""
	    echo "  The results can also be piped to more to keep them"\
			"from scrolling off"
	    echo "  the screen by doing:"
	    echo "  $0 infilename | more"
	    echo ""
	    echo "For more information see the HTML documentation which should"
	    echo "have been installed along with this copy of cascade."
	    echo ""
	    echo "You can also obtain online help, report bugs, check for the"
	    echo "latest versions, and participate in online discussion forums"
	    echo "at the sourceforge home page for this project."
	    echo "http://www.sourceforge.net/projects/rfcascade/"
	    echo ""
	    exit 0 ;;

	--version)
	    echo ""
	    echo "$PROGNAME $VERSIONNUM"
	    echo ""
	    exit 0 ;;

	-*) echo "$0: Unknown argument $1" 1>&2; exit 1 ;;

	*)
    esac
else
    echo ""
    echo "USAGE:  $0  [--copyright] [--help] [--version]"
    echo "        $0  infilename"
    echo ""
    exit 0;
fi

#######################################################
# Check to see if the input file exists and is readable
#######################################################
if [ ! -r $1 ] 
then
    echo ""
    echo "ERROR:  Input file $1 not found or is not readable"
    echo ""
    exit 1
else
  infile=$1
fi


#
# run the main gawk/nawk program
#
$AWK  ' 
BEGIN{ 
        InFile = ARGV[1]

	# modify the field seperator to include = as well as whitespace
	# this will make it easy to parse the elements
	FS = "[ \t=]+"
    
        printf("************************************************************\n")
	printf("*                    CASCADE ANALYSIS                      *\n")
        printf("*                     Version %5s                        *\n",\
                                                                   ARGV[2]);
	printf("*               (c) 1997-2001 Dan McMahill                 *\n")
	printf("*                  mcmahill@alum.mit.edu                   *\n")
        printf("************************************************************\n")
	print("")
	InitializeSystem()

	print("Processing input file \""InFile"\"\n")

	ProcessInputFile(InFile)
	close(InFile)

        MakeSummary()

        printf("************************************************************\n")
	printf("*              CASCADE ANALYSIS COMPLETE                   *\n")
	
	if(ARGV[0] == "gawk"){
	  now = strftime("%a %b %d %H:%M:%S %Z %Y")
	  printf("*            %28s                  *\n",now)
	}
        printf("************************************************************\n")
	print("")

	exit 0
}
function InitializeSystem(){
    # initialize variables
	# total system power gain (ratio)
	gain=1

	# reference noise temperature
	tn0=290

	# total output noise temperature
	tn=tn0

	# source noise temperature (want to  read from infile)
	tns="none"

	# distortion present at each output
	d=0

	# carrier level at each output
	C="none"
	c="none"

	# current stage of the cascade
	stage=1

	# flag that says we are still reading things like the source line
	# and the default line

	# default input/output resistances of each block
	rin  = 50;
	rout = 50;

	# The default source resistance
	Rs = rout;
	routii=rout;

	# The default rho for IIP3 calculations
	rho_default = 0;

        # index into the array which holds NF and IIP3 due to
        # each stage only (used at the end to rank each stage
        # in terms of which hurts each spec the most)
        nf_index = 1
        iip_index = 1

}

function ProcessInputFile(file){
	while(getline < file > 0){
		ProcessInputLine($0)
	}
        print("")
    printf("************************************************************\n")
    printf("****** ANALYSIS OF THE %2d ELEMENT CASCADE IS COMPLETE ******\n",stage-1)
    printf("************************************************************\n")
}

function ProcessInputLine(line){
       	if(line~/^[\t ]*[#;*]/ || line~/^[\t ]*$/){
	    # skip over comment lines and blank lines
	}
       	else if(line~/^[\t ]*@/){
	    # echo @ style comment lines to the output
	    print $0
	}
	else{
	    ParseLine(line);
	}
}

function ParseLine(line){

    StageName=$1

    if(StageName~/[Ss][Oo][Uu][Rr][Cc][Ee]/){
	if(stage != 1){
	    print("WARNING:  Source line was ignored")
	    print("  \""$0"\"")
	    print("  This source line was ignored because "\
		    "the source MUST be the FIRST")
	    print("  element in the cascade and this line was line "stage)
	    print("")
	}
	else{
	    ParseSource(line)
	}
    }
    else if(StageName~/[dD][eE][fF][aA][uU][lL][tT][sS]/){
	    ParseDefaults(line);
    }
    else{

    Gi="";
    GVi="";
    NFi="";
    IIP3i="";
    rhoi=rho_default;
    rini=rin;
    routi=rout;

    i=2
    while($i != ""){
    if($i~/[Gg][Pp]/){
	# found power gain
	i++
	Gi=$i
	}
    else if($i~/[Gg][Vv]/){
	# found voltage gain
	i++
	GVi=$i
	}
    else if($i~/[Gg]/){
	# found old format gain which is assumed to be power gain
	i++
	Gi=$i
	}
    else if($i~/[Nn][Ff]/){
	# found noise figure
	i++
	NFi=$i
	}
    else if($i~/[Ii][Ii][Pp][3]/){
	# found iip3
	i++
	IIP3i=$i
	}
    else if($i~/[Rr][Ii][Nn]/){
	# found rin
	i++
	rini=$i
	}
    else if($i~/[Rr][Oo][Uu][Tt]/){
	# found rout
	i++
	routi=$i
	}
    else if($i~/[Rr]/){
	# found rho
	i++
	rhoi=$i
	}
    else{
	print("SYNTAX ERROR:  The entry \""$i"\" in the line ")
	print("\""$0"\"")
 	print("Is not understood\n\n")
	exit 1
    }

    i++
    }

    if((Gi=="") && (GVi == "")){
	print("ERROR:  Gain must be specified")
	print("Line with the error is:")
	print($0)
    }
    if((Gi!="") && (GVi != "")){
	print("ERROR:  You can only specify power gain _or_ voltage gain");
	print("        not both!");
	print("Line with the error is:")
	print($0)
    }

    if(Gi == ""){
	Gi = GVi + 10*log10(rini/routi);
    }

    if(GVi == ""){
	GVi = Gi + 10*log10(routi/rini);
    }

    if(NFi==""){
	NFi=0
    }
    if(IIP3i==""){
	IIP3i="nodist"
    }

    ComputeStage()
    PrintStageSummary()

    stage++

}

}



function ParseDefaults(line,i){

    i=2
    while($i != ""){
    if($i~/[Rr][Ii][Nn]/){
	# found rin
	i++;
	rin=$i;
	}
    else if($i~/[Rr][Oo][Uu][Tt]/){
	# found rout
	i++;
	rout=$i;
	}
    else if($i~/[Rr][Hh][Oo]/){
	# found rho
	i++;
	rho_default=$i;
	}
    else{
	print("SYNTAX ERROR:  The entry \""$i"\" in the line ")
	print("\""$0"\"");
 	print("Is not understood.  This line is supposed to be the defaults line\n\n");
	exit 1;
    }

    i++;
    }

    print("************************************************************")
    print("*    Default Values Changed                                *")
    print("************************************************************")
    print("");
    print("     Input  Resistance for each Stage = "rin " Ohms");
    print("     Output Resistance for each Stage = "rout" Ohms");
    print("     Default Rho (for IIP3 calc.)     = "rho_default);
    print("");

}

function ParseSource(line,i){

    Cs=""
    CNs=""
    BWs=""
    CNos=""
    TNs=""
    Nos=""
    
    # use the default value
    Rs=rout;
    
    need_source=0;

    i=2
    while($i != ""){
    if($i~/[Cc][Nn][Oo0]/){
	# found C/No
	i++
	CNos=$i
	need_source=1;
	}
    else if($i~/[Nn][0oO]/){
	# found noise spectral density
	i++
	Nos=$i
	need_source=1;
	}
    else if($i~/[Tt][Nn]/){
	# found input noise temperature
	i++
        TNs=$i
	need_source=1;
	}
    else if($i~/[Cc][Nn]/){
	# found carrier to noise
	i++
	CNs=$i
	need_source=1;
	}
    else if($i~/[Bb][Ww]/){
	# found carrier noise bandwidth
	i++
	BWs=$i
	need_source=1;
	}
    else if($i~/[Cc]/){
	# found the input carrier level
	i++
	Cs=$i
	}
    else if($i~/[Rr][Ss]/){
	# found the source resistance
	i++
	Rs=$i
	}
    else{
	print("SYNTAX ERROR:  The entry \""$i"\" in the line ")
	print("\""$0"\"")
 	print("Is not understood.  This line is supposed to be source\n\n")
	exit 1
    }

    i++
    }
  
    # done parsing the line, now do error checking
    if(Cs=="" && need_source){
	print("ERROR:  Input carrier level must be specified if a "\
		"source with");
	print("        noise parameters is specified");
	print("Line with the error is:");
	print($0);
	exit 1;
    }

    # If C/N is specified, we can not specify C/No, No, or Tn
    if(CNs!=""){
	if((CNos!="") || (Nos!="") || (TNs!="")){
	    print("ERROR:  Can not specify C/N and C/No or No or Tn,"\
		    " for the source.")
	    print("Line with the error is:")
	    print($0)
	    exit 1
	}
    }

    # If C/No is specified, we can not specify No, or Tn
    if(CNos!=""){
	if((Nos!="") || (TNs!="")){
	    print("ERROR:  Can not specify C/No and No or Tn for the source.")
	    print("Line with the error is:")
	    print($0)
	    exit 1
	}
    }

    # If No is specified, we can not specify Tn
    if(Nos!=""){
	if(TNs!=""){
	    print("ERROR:  Can not specify No and Tn for the source.")
	    print("Line with the error is:")
	    print($0)
	    exit 1
	}
    }

    if((CNs!="") && (BWs=="")){
	print("ERROR:  The receiver noise BW must be specified when "\
		"using C/N.")
	print("Line with the error is:")
	print($0)
	exit 1
    }

    # calculate initial values
    if(Cs != ""){
	C=Cs;
	c=dBmToW(C);
    }

    # C/N is specified, calculate C/No
    if(CNs != ""){
	CNos = CNs + RatioTodB10(BWs)
    }

    # No is specified, calculate C/No
    if(Nos != ""){
	CNos = C - Nos
	}

    # we have a C/No, so find No
    if(CNos != ""){
	NFs = (C+173) - CNos
	nfs=dB10ToRatio(NFs)
	tns = 290*nfs
	Nos = C-CNos
	if(Nos < -174){
	    print("  ** WARNING You have specified a No less than k*290"\
		"(-174 dBm/Hz) **\n")
		}
	if(BWs != ""){
	    CNs = CNos - RatioTodB10(BWs)
	    }
    }
    
    # Tn is specified
    if(TNs != ""){
	tns = TNs
	# k = 1.38e-23 Joules/Kelvin
	Nos = 10*log(TNs*1.38e-20)/log(10)
	CNos = C-Nos
	if(BWs != ""){
	    CNs = CNos - RatioTodB10(BWs)
	    }
	}

    if(CNs != ""){
	#CNos=""
    }
    
    # routii is used in finding loadings between stages i-1 (ii) and i.
    # routii is the output resistance of stage i-1.
    routii = Rs;

    print("************************************************************")
    print("*    Input Source                                          *")
    print("************************************************************")

    print("     Source Resistance   = "Rs" Ohms");
    print("");


    if(Cs !=""){
    print("     Input Carrier Level = "Cs" dBm")
    }

    if(Nos !=""){
    print("     Input Noise Density = "Nos" dBm/Hz")
    print("")
    }

    if(TNs !=""){
    print("     Input Noise Temp.   = "TNs" Kelvin")
    print("")
    }

    if(CNs !=""){
    print("     Input C/N           = "CNs" dB")
    print("     Receiver Noise BW   = "BWs/1e6" MHz")
    print("")
    }

    if(CNos !=""){
    print("     Input C/No          = "CNos" dB/Hz")
    print("")
    }


}


function ComputeStage(){

    # convert from dB to watts and ratios
    gi = dB10ToRatio(Gi);
    nfi = dB10ToRatio(NFi);
    if(iip3i != "nodist"){
	iip3i = dBmToW(IIP3i)
	}
    
    # add to total power gain
    gain = gain*gi;
    G = RatioTodB10(gain);

    # add to total noise temp
    tn=gi*(tn + (nfi-1)*tn0);

    # compute NF of the total cascade up to this point
    nf = (tn/gain)/tn0
    NFig = RatioTodB10(nf)


    # compute NF due to this stage only
    nf_single=1 + (nfi-1)/(gain/gi)
    NF_single=RatioTodB10(nf_single)
    NF_single_value_list[nf_index] = NF_single
    NF_value_list[nf_index] = NFi
    NF_name_list[nf_index] = StageName
    G_value_list[nf_index] = G - Gi
    nf_index_list[nf_index]=nf_index
    nf_index = nf_index+1

    # compute C/N degredation
    if(tns != "none"){
        # total
	loss=1 + (nf-1)*(tn0/tns)
	LOSS=RatioTodB10(loss)

        # from this stage only
	lossi=1 + ((nfi - 1)/(gain/gi))*(tn0/tns)
	LOSSi=RatioTodB10(lossi)
    }


    # add to distortion power
    if(IIP3i != "nodist"){
	cd = gain;
	oip3i = gi*iip3i;
	dm1 = d;
	d = gi*dm1 + (cd^3)/(oip3i^2) + 2*rhoi*sqrt(gi*dm1*(cd^3)/(oip3i^2));

        IIP3_single = IIP3i - (G-Gi);
        IIP3_value_list[iip_index] = IIP3_single;
        IIP3_name_list[iip_index] = StageName;
        iip_index_list[iip_index] = iip_index;
        iip_index = iip_index+1;
	}
    else{
	d = gi*d;
    }

    # compute IIP3
    if(d != 0){
	iip3=sqrt(gain/d);
	IIP3 = WTodBm(iip3);
    }
    else{
	iip3="nodist";
	IIP3="nodist";
	}

    # add to carrier level
    if(c != "none"){
	c=c*gi;
	C=WTodBm(c);
	}

}
function PrintStageSummary(){
    printf("************************************************************\n")
    printf("*    Stage #%-2d %-44s*\n",stage,sprintf("\"%s\"",StageName))
    printf("************************************************************\n")
    printf("     Power Gain=%6.2f dB, Voltage Gain=%6.2f dB\n",Gi,GVi);
    printf("     NF=%6.2f dB\n",NFi);
    print("     Input Res. = "rini" Ohms,  Output Res. = "routi" Ohms");
	
    if(IIP3i != "nodist"){
	printf("     IIP3=%6.2f dBm (%6.2f dBmV), RHO=%6.2f\n",IIP3i, 
	    IIP3i+30+10*log10(rini),rhoi);
    }
    else{
	print("     No Distortion In this Stage")
    }
    if(NFi<0){
	print("   **WARNING You have specified a NF < 0 dB ** ")
	}

    print(" ")
    printf("     Total Power   Gain                     =  %6.2f dB\n", G)
    printf("     Total Voltage Gain                     =  %6.2f dB\n", G + 10*log10(routi/Rs));
    printf("     Total Noise Figure                     =  %6.2f dB\n", NFig)
    printf("     Noise Figure from this stage only      =  %6.2f dB\n", NF_single)

    if(CNs != ""){
    print(" ")
    printf("     C/N                                    = %7.2f dB\n",CNs- LOSS)    
    if(CNos != ""){
    printf("     C/No                                   = %7.2f dB\n",CNos- LOSS)    
    }
    printf("     C/N Degradation                        = %7.3f dB\n", LOSS)    
    printf("     C/N Degradation from this stage only   = %7.3f dB\n", LOSSi)    
    }

    if((CNos != "") && (CNs == "")){
    print(" ")
    printf("     C/No                                   = %7.2f dB\n",CNos- LOSS)    
    printf("     C/No Degradation                       = %7.3f dB\n", LOSS)    
    printf("     C/No Degradation from this stage only  = %7.3f dB\n", LOSSi)    
    }

    if(IIP3 != "nodist"){
    print(" ")
    printf("     IIP3                                   = %7.2f dBm\n", IIP3)
    }

    if(IIP3i != "nodist"){
    printf("     IIP3 from this stage only              = %7.2f dBm\n", IIP3_single)
    }

    if(c != "none"){
    print(" ");
    printf("     Output Carrier Level                   = %7.2f dBm\n", C)
    printf("                                            = %7.2f dBmV\n", \
	    C+ 30 + 10*log10(rout));
    print(" ");
    printf("     Output Open Circuit Voltage            = %7.2f dBmV\n", \
	    C + 30 + 20*log10(2)+10*log10(rout));


	if(IIP3 != "nodist"){
	print(" ");
	printf("     Third Order Intermod Level             = %7.2f dBc\n", 2*(Cs-IIP3))
	}
    }

    print("")

}

function MakeSummary(){

    print("")
    print("************************************************************")
    print("*       Noise Figure Contribution Summary                  *")
    print("************************************************************")
    
    # calculate the best improvement to the system noise figure that
    # is possible by eliminating the noise from each block

    for(i=1;i<nf_index;i++){
      nfi  = dB10ToRatio(NF_value_list[i])
      gain = dB10ToRatio(G_value_list[i])
      NF_delta_list[i] = NFig - RatioTodB10(nf - ((nfi-1)/gain))
    }

    # sort the individual stage noise figures to rank them from worst to best
    # in terms of contribution to overall noise figure

    # NOTE:  this algorithm is order N^2 and should not be used elsewhere except
    #        to sort small (N<20 or so) lists.  The assumption here is that most
    #        of the systems analyzed will be relatively small and the overhead
    #        (including a larger programming effort) of quicksort or heapsort
    #        did not seem worth it.

    # iterate through each element
    for(j=2;j<nf_index;j++){
      i = j-1;
      
      # figure out where to insert it
      while((i>0) && (NF_single_value_list[nf_index_list[i]]<NF_single_value_list[j])){
        nf_index_list[i+1] = nf_index_list[i]
	i--
      }
      
      # insert it
      nf_index_list[i+1]=j

    }


    print("")
    print("     Stage          Noise Figure     Possible Noise Figure ")
    print("                    in the system          Improvement      ")
    print("---------------   ----------------   ----------------------")
    for(j=1;j<nf_index;j++){
      i = nf_index_list[j]
      printf("%-15s   %9.3f dB      %14.3f dB\n", NF_name_list[i], \
                NF_single_value_list[i] ,NF_delta_list[i])
    }



    print("")
    print("************************************************************")
    print("*             IIP3 Contribution Summary                    *")
    print("************************************************************")
    
    # sort the individual IIP3s to rank them from worst to best
    # in terms of contribution to overall IIP3

    # iterate through each element
    for(j=2;j<iip_index;j++){
      i = j-1;
      
      # figure out where to insert it
      while(i>0 && IIP3_value_list[iip_index_list[i]]<IIP3_value_list[j]){
        iip_index_list[i+1] = iip_index_list[i]
	i--
      }
      
      # insert it
      iip_index_list[i+1]=j
    }

    print(" ")
    print("     Stage         IIP3 in the system")
    print("---------------   --------------------")
    for(j=iip_index-1;j>0;j--){
      i = iip_index_list[j]
      printf("%-15s   %10.3f dBm\n", IIP3_name_list[i], \
                IIP3_value_list[i])
    }
   
    print("")

}

function log10(x){
	return(log(x)/log(10));
}

function dBmToW(pwrdb){
	return(0.001*10^(pwrdb/10));
}

function WTodBm(pwr){
	return(10*log(pwr/0.001)/log(10));
}

function dB10ToRatio(pwrdb){
	return(10^(pwrdb/10));
}
function RatioTodB10(rat){
	return(10*log(rat)/log(10));
}

' $infile $VERSIONNUM

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{M}
\alias{M}
\title{Simulated M data for examples.}
\description{
Simulated M microarray.
}
\examples{

data(M)
head(M)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Cascade-package.R
\docType{class}
\name{micropredict-class}
\alias{micropredict-class}
\title{Class \code{"micropredict"}}
\description{
The \code{"micropredict"} class
}
\section{Objects from the Class}{
 Objects can be created by calls of the form
\code{new("micropredict", ...)}.
}

\examples{

showClass("micropredict")

}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{Selection}
\alias{Selection}
\title{Selection of genes.}
\description{
20 (at most) genes with differential expression at t1, 20 (at most) genes
with differential expression at t2, 20 (at most) genes with differential
expression at t3, 20 (at most) genes with differential expression at t4 et
20 (at most) genes with global differential expression were selected.
}
\examples{

data(Selection)
head(Selection)
summary(Selection,3)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Cascade-package.R
\docType{package}
\name{Cascade-package}
\alias{Cascade-package}
\alias{Cascade}
\title{Cascade: Selection, Reverse-Engineering and Prediction in Cascade Networks}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

A modeling tool allowing gene selection, reverse engineering, and prediction in cascade networks. Jung, N., Bertrand, F., Bahram, S., Vallat, L., and Maumy-Bertrand, M. (2014) \doi{10.1093/bioinformatics/btt705}.
}
\references{
Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
simulate the diffusion of a signal through a temporal gene network.
\emph{Bioinformatics}, btt705.

Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
circuitry of a cancer cell with predicted intervention in chronic
lymphocytic leukemia. \emph{Proceedings of the National Academy of
Sciences}, 110(2), 459-464.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://fbertran.github.io/Cascade/}
  \item \url{https://github.com/fbertran/Cascade}
  \item Report bugs at \url{https://github.com/fbertran/Cascade/issues}
}

}
\author{
This package has been written by Frédéric Bertrand, Myriam
Maumy-Bertrand and Nicolas Jung with biological insights from Laurent
Vallat. Maintainer: Frédéric Bertrand <frederic.bertrand@lecnam.net>
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/micro_array.R, R/network.R
\docType{methods}
\name{print-methods}
\alias{print-methods}
\alias{print,ANY-method}
\alias{print,micro_array-method}
\alias{print,network-method}
\title{Methods for Function \code{print}}
\usage{
\S4method{print}{micro_array}(x, ...)

\S4method{print}{network}(x, ...)
}
\arguments{
\item{x}{an object of class micro-array or network}

\item{...}{additional parameters}
}
\description{
Methods for function \code{print}
}
\examples{

data(Net)
print(Net)

data(M)
print(M)

}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{Net_inf}
\alias{Net_inf}
\title{Reverse-engineered network of the simulated data.}
\description{
The reverse-engineered network of the simulated data (M and Net).
}
\examples{

data(Net_inf)
str(Net_inf)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/micro_array.R
\docType{methods}
\name{unionMicro-methods}
\alias{unionMicro-methods}
\alias{unionMicro}
\alias{unionMicro,micro_array,micro_array-method}
\alias{unionMicro,list,ANY-method}
\title{Makes the union between two micro_array objects.}
\usage{
\S4method{unionMicro}{micro_array,micro_array}(M1, M2)
}
\arguments{
\item{M1}{a micro-array or a list of micro-arrays}

\item{M2}{a micro-array or nothing if M1 is a list of micro-arrays}
}
\description{
Makes the union between two micro_array objects.
}
\section{Methods}{
 \describe{

\item{list("signature(M1 = \"micro_array\", M2 = \"micro_array\")")}{
Returns a micro_array object which is the union of M1 and M2.  }

\item{list("signature(M1 = \"list\", M2 = \"ANY\")")}{ Returns a micro_array
object which is the union of the elements of M1.  } }
}

\examples{

data(M)
#Create another microarray object with 100 genes
Mbis<-M
#Rename the 100 genes
Mbis@name<-paste(M@name,"bis")
rownames(Mbis@microarray) <- Mbis@name
#Union (merge without duplicated names) of the two microarrays. 
str(unionMicro(M,Mbis))

}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/global.R
\name{network_random}
\alias{network_random}
\title{Generates a network.}
\usage{
network_random(
  nb,
  time_label,
  exp,
  init,
  regul,
  min_expr,
  max_expr,
  casc.level
)
}
\arguments{
\item{nb}{Integer. The number of genes.}

\item{time_label}{Vector. The time points measurements.}

\item{exp}{The exponential parameter, as in the barabasi.game function in
igraph package.}

\item{init}{The attractiveness of the vertices with no adjacent edges. See
barabasi.game function.}

\item{regul}{A vector mapping each gene with its number of regulators.}

\item{min_expr}{Minimum of strength of a non-zero link}

\item{max_expr}{Maximum of strength of a non-zero link}

\item{casc.level}{...}
}
\value{
A network object.
}
\description{
Generates a network.
}
\examples{

set.seed(1)
Net<-network_random(
	nb=100,
	time_label=rep(1:4,each=25),
	exp=1,
	init=1,
	regul=round(rexp(100,1))+1,
	min_expr=0.1,
	max_expr=2,
	casc.level=0.4
	)
plot(Net)

}
\references{
Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
simulate the diffusion of a signal through a temporal gene network.
\emph{Bioinformatics}, btt705.

Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
circuitry of a cancer cell with predicted intervention in chronic
lymphocytic leukemia. \emph{Proceedings of the National Academy of
Sciences}, 110(2), 459-464.
}
\author{
Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{Net}
\alias{Net}
\title{Simulated network data for examples.}
\description{
Simulated network.
}
\examples{

data(Net)
str(Net)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/micro_array.R
\docType{methods}
\name{summary-methods}
\alias{summary-methods}
\alias{summary,ANY-method}
\alias{summary,micro_array-method}
\title{Methods for Function \code{summary}}
\usage{
\S4method{summary}{micro_array}(object, nb.graph = NULL, ...)
}
\arguments{
\item{object}{an object of class micro-array}

\item{nb.graph}{(optionnal) choose the graph to plot. Displays all graphs by default.}

\item{...}{additional parameters.}
}
\description{
Methods for function \code{summary}
}
\examples{

data(M)
summary(M)

}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/micro_array-network.R
\name{gene_expr_simulation,network-method}
\alias{gene_expr_simulation,network-method}
\alias{gene_expr_simulation}
\alias{gene_expr_simulation-methods}
\title{Simulates microarray data based on a given network.}
\usage{
\S4method{gene_expr_simulation}{network}(network, time_label = 1:4, subject = 5, level_peak = 100)
}
\arguments{
\item{network}{A network object.}

\item{time_label}{a vector containing the time labels.}

\item{subject}{the number of subjects}

\item{level_peak}{the mean level of peaks.}
}
\value{
A micro_array object.
}
\description{
Simulates microarray data based on a given network.
}
\examples{

data(Net)
set.seed(1)

#We simulate gene expression according to the network Net
Msim<-gene_expr_simulation(
	network=Net,
	time_label=rep(1:4,each=25),
	subject=5,
	level_peak=200)
head(Msim)

}
\references{
Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
simulate the diffusion of a signal through a temporal gene network.
\emph{Bioinformatics}, btt705.

Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
circuitry of a cancer cell with predicted intervention in chronic
lymphocytic leukemia. \emph{Proceedings of the National Academy of
Sciences}, 110(2), 459-464.
}
\author{
Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{geneNeighborhood,network-method}
\alias{geneNeighborhood,network-method}
\alias{geneNeighborhood}
\alias{geneNeighborhood-methods}
\title{Find the neighborhood of a set of nodes.}
\usage{
\S4method{geneNeighborhood}{network}(
  net,
  targets,
  nv = 0,
  order = length(net@time_pt) - 1,
  label_v = NULL,
  ini = NULL,
  frame.color = "white",
  label.hub = FALSE,
  graph = TRUE,
  names = FALSE
)
}
\arguments{
\item{net}{a network object}

\item{targets}{a vector containing the set of nodes}

\item{nv}{the level of cutoff. Defaut to 0.}

\item{order}{of the neighborhood. Defaut to `length(net@time_pt)-1`.}

\item{label_v}{vector defining the vertex labels.}

\item{ini}{using the ``position'' function, you can
fix the position of the nodes.}

\item{frame.color}{color of the frames.}

\item{label.hub}{logical ; if TRUE only the hubs are labeled.}

\item{graph}{plot graph of the network. Defaults to `TRUE`.}

\item{names}{return names of the neighbors. Defaults to `FALSE`.}
}
\value{
The neighborhood of the targeted genes.
}
\description{
Find the neighborhood of a set of nodes.
}
\examples{

data(Selection)
data(network)
#A nv value can chosen using the cutoff function
nv=.11 
EGR1<-which(match(Selection@name,"EGR1")==1)
P<-position(network,nv=nv)

geneNeighborhood(network,targets=EGR1,nv=nv,ini=P,
label_v=network@name)

}
\references{
Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
simulate the diffusion of a signal through a temporal gene network.
\emph{Bioinformatics}, btt705.

Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
circuitry of a cancer cell with predicted intervention in chronic
lymphocytic leukemia. \emph{Proceedings of the National Academy of
Sciences}, 110(2), 459-464.
}
\author{
Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\docType{methods}
\name{compare-methods}
\alias{compare-methods}
\alias{compare}
\alias{compare,network,network,numeric-method}
\title{Some basic criteria of comparison between actual and inferred network.}
\usage{
\S4method{compare}{network,network,numeric}(Net, Net_inf, nv = 1)
}
\arguments{
\item{Net}{A network object containing the
actual network.}

\item{Net_inf}{A network object containing the inferred
network.}

\item{nv}{A number that indicates at which level of cutoff the
comparison should be done.}
}
\value{
A vector containing : sensibility, predictive positive value, and
the F-score
}
\description{
Allows comparison between actual and inferred network.
}
\section{Methods}{
 \describe{
\item{list("signature(Net = \"network\", Net_inf = \"network\", nv =
\"numeric\")")}{}
}
}

\examples{

data(Net)
data(Net_inf)

#Comparing true and inferred networks
F_score=NULL

#Here are the cutoff level tested
test.seq<-seq(0,max(abs(Net_inf@network*0.9)),length.out=200)
for(u in test.seq){
	F_score<-rbind(F_score,Cascade::compare(Net,Net_inf,u))
}
matplot(test.seq,F_score,type="l",ylab="criterion value",xlab="cutoff level",lwd=2)

}
\references{
Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
simulate the diffusion of a signal through a temporal gene network.
\emph{Bioinformatics}, btt705.

Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
circuitry of a cancer cell with predicted intervention in chronic
lymphocytic leukemia. \emph{Proceedings of the National Academy of
Sciences}, 110(2), 459-464.
}
\author{
Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{analyze_network,network-method}
\alias{analyze_network,network-method}
\alias{analyze_network}
\alias{analyze_network-methods}
\title{Analysing the network}
\usage{
\S4method{analyze_network}{network}(Omega, nv, label_v = NULL)
}
\arguments{
\item{Omega}{a network object}

\item{nv}{the level of cutoff at which the analysis should be done}

\item{label_v}{(optionnal) the name of the genes}
}
\value{
A matrix containing, for each node, its betweenness,its degree, its
output, its closeness.
}
\description{
Calculates some indicators for each node in the network.
}
\examples{

	data(network)
	analyze_network(network,nv=0)

}
\references{
Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
simulate the diffusion of a signal through a temporal gene network.
\emph{Bioinformatics}, btt705.

Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
circuitry of a cancer cell with predicted intervention in chronic
lymphocytic leukemia. \emph{Proceedings of the National Academy of
Sciences}, 110(2), 459-464.
}
\author{
Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/micro_array-network.R
\name{inference,micro_array-method}
\alias{inference,micro_array-method}
\alias{inference}
\alias{inference-methods}
\title{Reverse-engineer the network}
\usage{
\S4method{inference}{micro_array}(
  M,
  tour.max = 30,
  g = function(x) {
     1/x
 },
  conv = 0.001,
  cv.subjects = TRUE,
  nb.folds = NULL,
  eps = 10^-5,
  type.inf = "iterative"
)
}
\arguments{
\item{M}{a micro_array object.}

\item{tour.max}{maximal number of steps. Defaults to `tour.max=30`}

\item{g}{the new solution is choosen as
(the old solution + g(x) * the new solution)/(1+g(x)) where x is the number
of steps. Defaults to `g=function(x) 1/x`}

\item{conv}{convergence criterion. Defaults to `conv=10e-3`}

\item{cv.subjects}{should the cross validation be done removing the
subject one by one ? Defaults to `cv.subjects=TRUE`.}

\item{nb.folds}{Relevant only if cv.subjects is
FALSE. The number of folds in cross validation. Defaults to `NULL`.}

\item{eps}{machine zero. Defaults to `10e-5`.}

\item{type.inf}{"iterative" or "noniterative" : should the
algorithm be computed iteratively. Defaults to `"iterative"`.}
}
\value{
A network object.
}
\description{
Reverse-engineer the network.
}
\examples{

\donttest{
#With simulated data
data(M)
infM <- inference(M)
str(infM)

#With selection of genes from GSE39411
data(Selection)
infSel <- inference(Selection)
str(infSel)
}

}
\references{
Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
simulate the diffusion of a signal through a temporal gene network.
\emph{Bioinformatics}, btt705.

Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
circuitry of a cancer cell with predicted intervention in chronic
lymphocytic leukemia. \emph{Proceedings of the National Academy of
Sciences}, 110(2), 459-464.
}
\author{
Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/micro_array.R
\name{head,micro_array-method}
\alias{head,micro_array-method}
\alias{methods}
\alias{head-methods}
\alias{head,ANY-method}
\title{Overview of a micro_array object}
\usage{
\S4method{head}{micro_array}(x, ...)
}
\arguments{
\item{x}{an object of class `micro_array`.}

\item{...}{additional parameters}
}
\description{
Overview of a micro_array object.
}
\section{Methods}{
 \describe{

\item{list("signature(x = \"ANY\")")}{ Gives an overview. }

\item{list("signature(x = \"micro_array\")")}{ Gives an overview. } }
}

\examples{

 if(require(CascadeData)){
	data(micro_US)
	micro_US<-as.micro_array(micro_US,time=c(60,90,210,390),subject=6)
	head(micro_US)
	}
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/micro_array-network.R
\name{predict,micro_array-method}
\alias{predict,micro_array-method}
\alias{predict}
\alias{predict-methods}
\alias{predict,ANY-method}
\title{Prediction of the gene expressions after a knock-out experience
\code{predict}}
\usage{
\S4method{predict}{micro_array}(object, Omega, nv = 0, targets = NULL, adapt = TRUE)
}
\arguments{
\item{object}{a micro_array object}

\item{Omega}{a network object.}

\item{nv}{[=0] numeric; the level of the cutoff}

\item{targets}{[NULL] vector; which genes are knocked out?}

\item{adapt}{[TRUE] boolean; do not raise an error if used with vectors 
instead of one column matrices.}
}
\description{
Prediction of the gene expressions after a knock-out experience
}
\examples{

data(Selection)
data(network)
#A nv value can chosen using the cutoff function
nv=.11
EGR1<-which(match(Selection@name,"EGR1")==1)
P<-position(network,nv=nv)

#We predict gene expression modulations within the network if EGR1 is experimentaly knocked-out. 
prediction_ko5<-predict(Selection,network,nv=nv,targets=EGR1)

#Then we plot the results. Here for example we see changes at time point t2:
plot(prediction_ko5,time=2,ini=P,label_v=Selection@name)

}
\references{
Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
simulate the diffusion of a signal through a temporal gene network.
\emph{Bioinformatics}, btt705.

Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
circuitry of a cancer cell with predicted intervention in chronic
lymphocytic leukemia. \emph{Proceedings of the National Academy of
Sciences}, 110(2), 459-464.
}
\author{
Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/micro_array.R, R/network.R, R/micropredict.R
\docType{methods}
\name{plot-methods}
\alias{plot-methods}
\alias{plot,ANY,ANY-method}
\alias{plot,micro_array,ANY-method}
\alias{plot,network,ANY-method}
\alias{plot,micropredict,ANY-method}
\title{Plot}
\usage{
\S4method{plot}{micro_array,ANY}(x, y, ...)

\S4method{plot}{network,ANY}(
  x,
  y,
  choice = "network",
  nv = 0,
  gr = NULL,
  ini = NULL,
  color.vertex = NULL,
  video = TRUE,
  weight.node = NULL,
  ani = FALSE,
  taille = c(2000, 1000),
  label_v = 1:dim(x@network)[1],
  horiz = TRUE,
  legend.position = "topleft",
  frame.color = "black",
  label.hub = FALSE,
  ...
)

\S4method{plot}{micropredict,ANY}(
  x,
  time = NULL,
  label_v = NULL,
  frame.color = "white",
  ini = NULL,
  label.hub = FALSE,
  edge.arrow.size = 0.7,
  edge.thickness = 1
)
}
\arguments{
\item{x}{a micro_array object, a network object or a micropredict object}

\item{y}{optional and not used if x is an appropriate structure}

\item{...}{additional parameters}

\item{choice}{what graphic should be plotted: either "F"
(for a representation of the matrices F) or "network".}

\item{nv}{the level of cutoff. Defaut to `0`.}

\item{gr}{a vector giving the group of each gene}

\item{ini}{using the ``position'' function, you can
fix the position of the nodes.}

\item{color.vertex}{a vector defining the color of the vertex.}

\item{video}{if ani is TRUE and video is TRUE, the result of the animation is saved as an animated GIF.}

\item{weight.node}{nodes weighting. Defaults to `NULL`.}

\item{ani}{animated plot?}

\item{taille}{vector giving the size of the plot. Default to `c(2000,1000)`.}

\item{label_v}{vector defining the vertex labels.}

\item{horiz}{landscape? Defaults to `TRUE`.}

\item{legend.position}{position of the legend.}

\item{frame.color}{color of the frames.}

\item{label.hub}{logical ; if TRUE only the hubs are labeled.}

\item{time}{sets the time for plot of the prediction. Defaults to `NULL`}

\item{edge.arrow.size}{size of the arrows ; default to 0.7.}

\item{edge.thickness}{edge thickness ; default to 1.}
}
\description{
Considering the class of the argument which is passed to plot, the graphical
output differs.
}
\section{Methods}{
 \describe{

\item{list("signature(x = \"micro_array\", y = \"ANY\",...)")}{ \describe{
\item{x}{a micro_array object} \item{list_nv}{a vector of cutoff at which
the network should be shown} } } \item{list("signature(x = \"network\", y =
\"ANY\",...)")}{ \describe{ \item{x}{a network object}
\item{list()}{Optionnal arguments: \describe{ \item{gr}{a vector giving the
group of each gene} \item{choice}{what graphic should be plotted: either "F"
(for a representation of the matrices F) or "network".} \item{nv}{the level
of cutoff. Defaut to 0.} \item{ini}{using the ``position'' function, you can
fix the position of the nodes} \item{color.vertex}{a vector defining the
color of the vertex} \item{ani}{animated plot?} \item{size}{vector giving the size of the plot. Default
to c(2000,1000)} \item{video}{if ani is TRUE and video is TRUE, the
animation result is a GIF video} \item{label_v}{vector defining the vertex
labels} \item{legend.position}{position of the legend}
\item{frame.color}{color of the frames} \item{label.hub}{logical ; if TRUE
only the hubs are labeled} \item{edge.arrow.size}{size of the arrows ;
default to 0.7} \item{edge.thickness}{edge thickness ; default to 1.} } }}}

\item{list("signature(x = \"micropredict\", y = \"ANY\",...)")}{ \describe{
\item{x}{a micropredict object} \item{list()}{Optionnal arguments: see plot
for network} }} }
}

\examples{

data(Net)
plot(Net)

data(M)
plot(M)

data(Selection)
data(network)
nv<-0.11
plot(network,choice="network",gr=Selection@group,nv=nv,label_v=Selection@name,
edge.arrow.size=0.9,edge.thickness=1.5)

}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/global.R
\name{as.micro_array}
\alias{as.micro_array}
\title{Coerce a matrix into a micro_array object.}
\usage{
as.micro_array(M, time, subject)
}
\arguments{
\item{M}{A matrix. Contains the microarray measurements. Should of size N *
K, with N the number of genes and K=T*P with T the number of time points,
and P the number of individuals. This matrix should be created using
cbind(M1,M2,...) with M1 a N*T matrix with the measurements for individual
1, M2 a N*T matrix with the measurements for individual 2.}

\item{time}{A vector. The time points measurements.}

\item{subject}{The number of subjects.}
}
\value{
A micro_array object.
}
\description{
Coerce a matrix into a micro_array object.
}
\examples{

  if(require(CascadeData)){
	data(micro_US)
	micro_US<-as.micro_array(micro_US,time=c(60,90,210,390),subject=6)
	}

}
\references{
Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
simulate the diffusion of a signal through a temporal gene network.
\emph{Bioinformatics}, btt705.

Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
circuitry of a cancer cell with predicted intervention in chronic
lymphocytic leukemia. \emph{Proceedings of the National Academy of
Sciences}, 110(2), 459-464.
}
\author{
Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{network}
\alias{network}
\title{A network object data.}
\description{
A network object. It is the same as the result in the vignette for the
inference of the network.
}
\examples{

data(network)
plot(network)
print(network)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/micro_array.R
\docType{methods}
\name{dim}
\alias{dim}
\alias{dim-methods}
\alias{dim,micro_array-method}
\title{Dimension of the data}
\usage{
\S4method{dim}{micro_array}(x)
}
\arguments{
\item{x}{an object of class "micro-array}
}
\description{
Dimension of the data
}
\section{Methods}{
 \describe{

\item{list("signature(x = \"micro_array\")")}{ Gives the dimension of the
matrix of measurements. } }
}

\examples{

 if(require(CascadeData)){
	data(micro_US)
	micro_US<-as.micro_array(micro_US,time=c(60,90,210,390),subject=6)
	dim(micro_US)
	}

}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{evolution,network-method}
\alias{evolution,network-method}
\alias{evolution}
\alias{evolution-methods}
\title{See the evolution of the network with change of cutoff}
\usage{
\S4method{evolution}{network}(
  net,
  list_nv,
  gr = NULL,
  color.vertex = NULL,
  fix = TRUE,
  gif = TRUE,
  taille = c(2000, 1000),
  label_v = 1:dim(net@network)[1],
  legend.position = "topleft",
  frame.color = "black",
  label.hub = FALSE
)
}
\arguments{
\item{net}{a network object}

\item{list_nv}{a vector of cutoff at which the network should be shown}

\item{gr}{a vector giving the group of each gene}

\item{color.vertex}{a vector giving the color of each node}

\item{fix}{logical, should the position of the node in the network be calculated once at the beginning ? Defaults to TRUE.}

\item{gif}{logical, TRUE}

\item{taille}{vector giving the size of the plot. Default to c(2000,1000)}

\item{label_v}{(optional) the name of the genes}

\item{legend.position}{(optional) the position of the legend, defaults to "topleft"}

\item{frame.color}{(optional) the color of the frame, defaults to "black"}

\item{label.hub}{(optional) boolean, defaults to FALSE}
}
\value{
A HTML page with the evolution of the network.
}
\description{
See the evolution of the network with change of cutoff.  This function may
be usefull to see if the global topology is changed while increasing the
cutoff.
}
\examples{

\donttest{
	data(network)
	sequence<-seq(0,0.2,length.out=20)
	#evolution(network, sequence) # writes a saveHTML() viewer to getwd()
}

}
\references{
Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
simulate the diffusion of a signal through a temporal gene network.
\emph{Bioinformatics}, btt705.

Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
circuitry of a cancer cell with predicted intervention in chronic
lymphocytic leukemia. \emph{Proceedings of the National Academy of
Sciences}, 110(2), 459-464.
}
\author{
Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/micro_array.R
\name{geneSelection}
\alias{geneSelection}
\alias{genePeakSelection}
\alias{genePeakSelection-methods}
\alias{geneSelection-methods}
\alias{geneSelection,list,list,numeric-method}
\alias{geneSelection,micro_array,micro_array,numeric-method}
\alias{genePeakSelection,micro_array,numeric-method}
\title{Methods for selecting genes}
\usage{
\S4method{geneSelection}{micro_array,micro_array,numeric}(
  x,
  y,
  tot.number,
  data_log = TRUE,
  wanted.patterns = NULL,
  forbidden.patterns = NULL,
  peak = NULL,
  alpha = 0.05,
  Design = NULL,
  lfc = 0
)

\S4method{geneSelection}{list,list,numeric}(
  x,
  y,
  tot.number,
  data_log = TRUE,
  alpha = 0.05,
  cont = FALSE,
  lfc = 0,
  f.asso = NULL
)

\S4method{genePeakSelection}{micro_array,numeric}(
  x,
  peak,
  y = NULL,
  data_log = TRUE,
  durPeak = c(1, 1),
  abs_val = TRUE,
  alpha_diff = 0.05
)
}
\arguments{
\item{x}{either a micro_array object or a list of micro_array objects. In
the first case, the micro_array object represents the stimulated
measurements. In the second case, the control unstimulated data (if present)
should be the first element of the list.}

\item{y}{either a micro_array object or a list of strings. In the first
case, the micro_array object represents the stimulated measurements. In the
second case, the list is the way to specify the contrast: \describe{
\item{First element:}{ condition, condition&time or pattern. The condition
specification is used when the overall is to compare two conditions.  The
condition&time specification is used when comparing two conditions at two
precise time points. The pattern specification allows to decide which time
point should be differentially expressed.} \item{Second element:}{a vector
of length 2. The two conditions which should be compared. If a condition is
used as control, it should be the first element of the vector. However, if
this control is not measured throught time, the option cont=TRUE should be
used.} \item{Third element:}{depends on the first element.  It is no needed
if condition has been specified.  If condition&time has been specified, then
this is a vector containing the time point at which the comparison should be
done. If pattern has been specified, then this is a vector of 0 and 1 of
length T, where T is the number of time points. The time points with desired
differential expression are provided with 1.  }}}

\item{tot.number}{an integer. The number of selected genes. If tot.number <0
all differentially genes are selected. If tot.number > 1, tot.number is the
maximum of diffenrtially genes that will be selected.  If 0<tot.number<1,
tot.number represents the proportion of diffenrentially genes that are
selected.}

\item{data_log}{logical (default to TRUE); should data be logged ?}

\item{wanted.patterns}{a matrix with wanted patterns [only for geneSelection].}

\item{forbidden.patterns}{a matrix with forbidden patterns [only for geneSelection].}

\item{peak}{interger. At which time points measurements should the genes be
selected [optionnal for geneSelection].}

\item{alpha}{float; the risk level. Default to `alpha=0.05`}

\item{Design}{the design matrix of the experiment. Defaults to `NULL`.}

\item{lfc}{log fold change value used in limma's `topTable`. Defaults to 0.}

\item{cont}{use contrasts. Defaults to `FALSE`.}

\item{f.asso}{function used to assess the association between the genes. 
The default value `NULL` implies the use of the usual `mean` function.}

\item{durPeak}{vector of size 2 (default to c(1,1)) ; the first elements gives the length of the peak at
the left, the second at the right. [only for genePeakSelection]}

\item{abs_val}{logical (default to TRUE) ; should genes be selected on the
basis of their absolute value expression ? [only for genePeakSelection]}

\item{alpha_diff}{float; the risk level}
}
\value{
A micro_array object.
}
\description{
Selection of differentially expressed genes.
}
\examples{

\donttest{
 if(require(CascadeData)){
	data(micro_US)
	micro_US<-as.micro_array(micro_US,time=c(60,90,210,390),subject=6)
	data(micro_S)
	micro_S<-as.micro_array(micro_S,time=c(60,90,210,390),subject=6)

  #Basically, to find the 50 more significant expressed genes you will use:
  Selection_1<-geneSelection(x=micro_S,y=micro_US,
  tot.number=50,data_log=TRUE)
  summary(Selection_1)
  
  #If we want to select genes that are differentially 
  #at time t60 or t90 :
  Selection_2<-geneSelection(x=micro_S,y=micro_US,tot.number=30,
  wanted.patterns=
  rbind(c(0,1,0,0),c(1,0,0,0),c(1,1,0,0)))
  summary(Selection_2)

  #To select genes that have a differential maximum of expression at a specific time point.
  
  Selection_3<-genePeakSelection(x=micro_S,y=micro_US,peak=1,
  abs_val=FALSE,alpha_diff=0.01)
  summary(Selection_3)
  }

 if(require(CascadeData)){
data(micro_US)
micro_US<-as.micro_array(micro_US,time=c(60,90,210,390),subject=6)
data(micro_S)
micro_S<-as.micro_array(micro_S,time=c(60,90,210,390),subject=6)
#Genes with differential expression at t1
Selection1<-geneSelection(x=micro_S,y=micro_US,20,wanted.patterns= rbind(c(1,0,0,0)))
#Genes with differential expression at t2
Selection2<-geneSelection(x=micro_S,y=micro_US,20,wanted.patterns= rbind(c(0,1,0,0)))
#Genes with differential expression at t3
Selection3<-geneSelection(x=micro_S,y=micro_US,20,wanted.patterns= rbind(c(0,0,1,0)))
#Genes with differential expression at t4
Selection4<-geneSelection(x=micro_S,y=micro_US,20,wanted.patterns= rbind(c(0,0,0,1)))
#Genes with global differential expression 
Selection5<-geneSelection(x=micro_S,y=micro_US,20)

#We then merge these selections:
Selection<-unionMicro(list(Selection1,Selection2,Selection3,Selection4,Selection5))
print(Selection)

#Prints the correlation graphics Figure 4:
summary(Selection,3)

##Uncomment this code to retrieve geneids.
#library(org.Hs.eg.db)
#
#ff<-function(x){substr(x, 1, nchar(x)-3)}
#ff<-Vectorize(ff)
#
##Here is the function to transform the probeset names to gene ID.
#
#library("hgu133plus2.db")
#
#probe_to_id<-function(n){  
#x <- hgu133plus2SYMBOL
#mp<-mappedkeys(x)
#xx <- unlist(as.list(x[mp]))
#genes_all = xx[(n)]
#genes_all[is.na(genes_all)]<-"unknown"
#return(genes_all)
#}
#Selection@name<-probe_to_id(Selection@name)
}
}

}
\references{
Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
simulate the diffusion of a signal through a temporal gene network.
\emph{Bioinformatics}, btt705.

Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
circuitry of a cancer cell with predicted intervention in chronic
lymphocytic leukemia. \emph{Proceedings of the National Academy of
Sciences}, 110(2), 459-464.
}
\author{
Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\docType{methods}
\name{position-methods}
\alias{position-methods}
\alias{position}
\alias{position,network-method}
\title{Returns the position of edges in the network}
\usage{
\S4method{position}{network}(net, nv = 0)
}
\arguments{
\item{net}{a network object}

\item{nv}{the level of cutoff at which the analysis should be done}
}
\description{
Returns the position of edges in the network
}
\section{Methods}{
 \describe{

\item{list("signature(net = \"network\")")}{ Returns a matrix with the
position of the node. This matrix can then be used as an argument in the
plot function. } }
}

\examples{

data(Net)
position(Net)

}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{cutoff,network-method}
\alias{cutoff,network-method}
\alias{cutoff}
\alias{cutoff-methods}
\title{Choose the best cutoff}
\usage{
\S4method{cutoff}{network}(Omega, sequence = NULL, x_min = 0)
}
\arguments{
\item{Omega}{a network object}

\item{sequence}{(optional) a vector corresponding to the sequence of cutoffs 
that will be tested.}

\item{x_min}{(optional) an integer ; only values over x_min are further 
retained for performing the test.}
}
\value{
A list containing two objects : \item{p.value}{the p values
corresponding to the sequence of cutoff} \item{p.value.inter}{the smoothed p
value vector, using the loess function}
}
\description{
Allows estimating the best cutoff, in function of the scale-freeness of the
network.  For a sequence of cutoff, the corresponding p-value is then
calculated.
}
\examples{

\donttest{
	data(network)
	cutoff(network)
	#See vignette for more details
}

}
\references{
Jung, N., Bertrand, F., Bahram, S., Vallat, L., and
Maumy-Bertrand, M. (2014). Cascade: a R-package to study, predict and
simulate the diffusion of a signal through a temporal gene network.
\emph{Bioinformatics}, btt705.

Vallat, L., Kemper, C. A., Jung, N., Maumy-Bertrand, M., Bertrand, F.,
Meyer, N., ... & Bahram, S. (2013). Reverse-engineering the genetic
circuitry of a cancer cell with predicted intervention in chronic
lymphocytic leukemia. \emph{Proceedings of the National Academy of
Sciences}, 110(2), 459-464.
}
\author{
Nicolas Jung, Frédéric Bertrand , Myriam Maumy-Bertrand.
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Cascade-package.R
\docType{class}
\name{micro_array-class}
\alias{micro_array-class}
\title{Class \code{"micro_array"}}
\description{
The \code{"micro_array"} class
}
\section{Objects from the Class}{
 Objects can be created by calls of the form
\code{new("micro_array", ...)}. %% ~~ describe objects here ~~
}

\examples{

showClass("micro_array")

}
\keyword{classes}
